@@ -19,9 +19,10 @@ Key terms used:
   to assess the performance of algorithms.
 
 """
-
+import logging
 import random
 import time
+from dataclasses import dataclass
 from types import FunctionType
 from typing import Callable, Dict, List, Optional, Tuple, Union
 
@@ -29,18 +30,27 @@ import numpy as np
 import torch
 from ax.benchmark import utils
 from ax.benchmark.benchmark_problem import BenchmarkProblem, SimpleBenchmarkProblem
-from ax.core.data import Data
+from ax.core.abstract_data import AbstractDataFrameData
+from ax.core.base_trial import BaseTrial
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
 from ax.core.observation import ObservationFeatures
 from ax.core.parameter import RangeParameter
+from ax.early_stopping.strategies import BaseEarlyStoppingStrategy
 from ax.modelbridge.base import gen_arms
 from ax.modelbridge.generation_strategy import GenerationStrategy
+from ax.runners.simulated_backend import SimulatedBackendRunner
 from ax.runners.synthetic import SyntheticRunner
 from ax.service.ax_client import AxClient
+from ax.service.scheduler import SchedulerOptions
 from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import not_none
 from ax.utils.measurement.synthetic_functions import SyntheticFunction
+from ax.utils.testing.backend_scheduler import AsyncSimulatedBackendScheduler
+from ax.utils.testing.backend_simulator import (
+    BackendSimulator,
+    BackendSimulatorOptions,
+)
 
 logger = get_logger(__name__)
 
@@ -58,14 +68,39 @@ class NonRetryableBenchmarkingError(ValueError):
     pass
 
 
+@dataclass
+class AsyncBenchmarkOptions:
+    """Options used in an async, Scheduler-based benchmark:
+
+    Args:
+        scheduler_options: Options passed to the ``AsyncSimulatedBackendScheduler``.
+        backend_options: Options passed to the ``BackendSimulator``.
+        sample_runtime_func: A method to sample a runtime given a trial.
+        timeout_hours: The number of hours to run before timing out, passed
+            to the ``AsyncSimulatedBackendScheduler``.
+        max_pending_trials: The maximum number of pending trials, which is
+            passed to the ``AsyncSimulatedBackendScheduler``.
+        early_stopping_strategy: The early stopping strategy.
+    """
+
+    scheduler_options: Optional[SchedulerOptions] = None
+    backend_options: Optional[BackendSimulatorOptions] = None
+    sample_runtime_func: Optional[Callable[[BaseTrial], float]] = None
+    timeout_hours: Optional[int] = None
+    max_pending_trials: int = 10
+    early_stopping_strategy: Optional[BaseEarlyStoppingStrategy] = None
+
+
 def benchmark_trial(
     parameterization: Optional[np.ndarray] = None,
     evaluation_function: Optional[Union[SyntheticFunction, FunctionType]] = None,
     experiment: Optional[Experiment] = None,
     trial_index: Optional[int] = None,
-) -> Union[Tuple[float, float], Data]:  # Mean and SEM or a Data object.
+) -> Union[
+    Tuple[float, float], AbstractDataFrameData
+]:  # Mean and SEM or a Data object.
     """Evaluates one trial from benchmarking replication (an Ax trial or batched
-    trial). Evaluation requires either the `parameterization` and `evalution_
+    trial). Evaluation requires either the `parameterization` and `evaluation_
     function` parameters or the `experiment` and `trial_index` parameters.
 
     Note: evaluation function relies on the ordering of items in the
@@ -92,7 +127,9 @@ def benchmark_trial(
         return evaluation_function(parameterization), sem  # pyre-ignore[29]: call err.
     else:
         trial_index = not_none(trial_index)
-        return not_none(not_none(experiment).trials.get(trial_index)).fetch_data()
+        trial = not_none(not_none(experiment).trials.get(trial_index))
+        trial.mark_completed()  # Some metrics only fetch data when `COMPLETED`.
+        return trial.fetch_data()  # This also automatically attaches the data.
 
 
 def benchmark_replication(  # One optimization loop.
@@ -106,6 +143,7 @@ def benchmark_replication(  # One optimization loop.
     verbose_logging: bool = True,
     # Number of trials that need to fail for a replication to be considered failed.
     failed_trials_tolerated: int = 5,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
 ) -> Experiment:
     """Runs one benchmarking replication (equivalent to one optimization loop).
 
@@ -123,6 +161,10 @@ def benchmark_replication(  # One optimization loop.
         verbose_logging: Whether logging level should be set to `INFO`.
         failed_trials_tolerated: How many trials can fail before a replication is
             considered failed and aborted. Defaults to 5.
+        async_benchmark_options: Options to use for the case of an async,
+            Scheduler-based benchmark. If omitted, a synchronous benchmark
+            (possibly with batch sizes greater than one) is run without using
+            a Scheduler.
     """
     torch.manual_seed(replication_index)
     np.random.seed(replication_index)
@@ -133,17 +175,19 @@ def benchmark_replication(  # One optimization loop.
         experiment_name += f"__v{replication_index}"
     # Make sure the generation strategy starts from the beginning.
     method = method.clone_reset()
-
-    # Choose whether to run replication via Service or Developer API, based on
-    # whether the problem was set up using Ax classes like `SearchSpace` and
-    # `OptimizationConfig` or using "RESTful" Service API-like constructs like
-    # dict parameter representations and `SyntheticFunction`-s or custom callables
-    # for evaluation function.
-    replication_runner = (
-        _benchmark_replication_Service_API
-        if isinstance(problem, SimpleBenchmarkProblem)
-        else _benchmark_replication_Dev_API
-    )
+    if async_benchmark_options is not None:
+        replication_runner = _benchmark_replication_Async_Scheduler
+    else:
+        # Choose whether to run replication via Service or Developer API, based on
+        # whether the problem was set up using Ax classes like `SearchSpace` and
+        # `OptimizationConfig` or using "RESTful" Service API-like constructs like
+        # dict parameter representations and `SyntheticFunction`-s or custom callables
+        # for evaluation function.
+        replication_runner = (
+            _benchmark_replication_Service_API
+            if isinstance(problem, SimpleBenchmarkProblem)
+            else _benchmark_replication_Dev_API
+        )
     experiment, exceptions = replication_runner(
         problem=problem,  # pyre-ignore[6]
         method=method,
@@ -154,8 +198,9 @@ def benchmark_replication(  # One optimization loop.
         benchmark_trial=benchmark_trial,
         verbose_logging=verbose_logging,
         failed_trials_tolerated=failed_trials_tolerated,
+        async_benchmark_options=async_benchmark_options,
     )
-
+    experiment.fetch_data()
     trial_exceptions.extend(exceptions)
     return experiment
 
@@ -174,6 +219,7 @@ def benchmark_test(  # One test, multiple replications.
     failed_trials_tolerated: int = 5,
     # Number of replications that need to fail for a test to be considered failed.
     failed_replications_tolerated: int = 3,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
 ) -> List[Experiment]:
     """Runs one benchmarking test (equivalent to one problem-method combination),
     translates into `num_replication` replications, ran for statistical
@@ -202,6 +248,10 @@ def benchmark_test(  # One test, multiple replications.
             considered failed and aborted. Defaults to 5.
         failed_replications_tolerated: How many replications can fail before a
             test is considered failed and aborted. Defaults to 3.
+        async_benchmark_options: Options to use for the case of an async,
+            Scheduler-based benchmark. If omitted, a synchronous benchmark
+            (possibly with batch sizes greater than one) is run without using
+            a Scheduler.
     """
     replication_exceptions = []
     test_replications = []
@@ -215,8 +265,10 @@ def benchmark_test(  # One test, multiple replications.
                     num_trials=num_trials,
                     batch_size=batch_size,
                     raise_all_exceptions=raise_all_exceptions,
+                    benchmark_trial=benchmark_trial,
                     verbose_logging=verbose_logging,
                     failed_trials_tolerated=failed_trials_tolerated,
+                    async_benchmark_options=async_benchmark_options,
                 )
             )
         except Exception as err:
@@ -250,6 +302,7 @@ def full_benchmark_run(  # Full run, multiple tests.
     failed_trials_tolerated: int = 5,
     # Number of replications that need to fail for a test to be considered failed.
     failed_replications_tolerated: int = 3,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
 ) -> Dict[str, Dict[str, List[Experiment]]]:
     """Full run of the benchmarking suite. To make benchmarking distrubuted at
     a level of a test, a replication, or a trial (or any combination of those),
@@ -304,6 +357,10 @@ def full_benchmark_run(  # Full run, multiple tests.
             considered failed and aborted. Defaults to 5.
         failed_replications_tolerated: How many replications can fail before a
             test is considered failed and aborted. Defaults to 3.
+        async_benchmark_options: Options to use for the case of an async,
+            Scheduler-based benchmark. If omitted, a synchronous benchmark
+            (possibly with batch sizes greater than one) is run without using
+            a Scheduler.
     """
     problem_groups = problem_groups or {}
     method_groups = method_groups or {}
@@ -338,6 +395,7 @@ def full_benchmark_run(  # Full run, multiple tests.
                         verbose_logging=verbose_logging,
                         failed_replications_tolerated=failed_replications_tolerated,
                         failed_trials_tolerated=failed_trials_tolerated,
+                        async_benchmark_options=async_benchmark_options,
                     )
                 except Exception as err:
                     if raise_all_exceptions:
@@ -358,11 +416,17 @@ def _benchmark_replication_Service_API(
     verbose_logging: bool = True,
     # Number of trials that need to fail for a replication to be considered failed.
     failed_trials_tolerated: int = 5,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
 ) -> Tuple[Experiment, List[Exception]]:
     """Run a benchmark replication via the Service API because the problem was
     set up in a simplified way, without the use of Ax classes like `OptimizationConfig`
     or `SearchSpace`.
     """
+    if async_benchmark_options is not None:
+        raise NonRetryableBenchmarkingError(
+            "`async_benchmark_options` not supported when using the Service API."
+        )
+
     exceptions = []
     if batch_size == 1:
         ax_client = AxClient(
@@ -421,11 +485,17 @@ def _benchmark_replication_Dev_API(
     verbose_logging: bool = True,
     # Number of trials that need to fail for a replication to be considered failed.
     failed_trials_tolerated: int = 5,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
 ) -> Tuple[Experiment, List[Exception]]:
     """Run a benchmark replication via the Developer API because the problem was
     set up with Ax classes (likely to allow for additional complexity like
     adding constraints or non-range parameters).
     """
+    if async_benchmark_options is not None:
+        raise NonRetryableBenchmarkingError(
+            "`async_benchmark_options` not supported when using the Dev API."
+        )
+
     exceptions = []
     experiment = Experiment(
         name=experiment_name,
@@ -442,8 +512,10 @@ def _benchmark_replication_Dev_API(
                 assert batch_size > 1
                 trial = experiment.new_batch_trial(generator_run=gr)
             trial.run()
-            benchmark_trial(experiment=experiment, trial_index=trial_index)
-            trial.mark_completed()
+            # TODO[T94059549]: Rm 3 lines below when attaching data in fetch is fixed.
+            data = benchmark_trial(experiment=experiment, trial_index=trial_index)
+            if not data.df.empty:
+                experiment.attach_data(data=data)
         except Exception as err:  # TODO[T53975770]: test
             if raise_all_exceptions:
                 raise
@@ -455,6 +527,79 @@ def _benchmark_replication_Dev_API(
     return experiment, exceptions
 
 
+def _benchmark_replication_Async_Scheduler(
+    problem: BenchmarkProblem,
+    method: GenerationStrategy,
+    num_trials: int,
+    experiment_name: str,
+    batch_size: int = 1,
+    raise_all_exceptions: bool = False,
+    benchmark_trial: FunctionType = benchmark_trial,
+    verbose_logging: bool = True,
+    # Number of trials that need to fail for a replication to be considered failed.
+    failed_trials_tolerated: int = 5,
+    async_benchmark_options: Optional[AsyncBenchmarkOptions] = None,
+) -> Tuple[Experiment, List[Exception]]:
+    """Run a benchmark replication with asynchronous evaluations through Scheduler.
+    The Scheduler interacts with a BackendSimulator.
+    """
+    if async_benchmark_options is None:
+        raise NonRetryableBenchmarkingError(
+            "`async_benchmark_options` required for Scheduler benchmarks."
+        )
+
+    backend_options = (
+        async_benchmark_options.backend_options
+        or BackendSimulatorOptions(
+            internal_clock=0.0,
+            max_concurrency=async_benchmark_options.max_pending_trials,
+        )
+    )
+    backend_simulator = BackendSimulator(
+        options=backend_options, verbose_logging=verbose_logging
+    )
+
+    experiment = Experiment(
+        name=experiment_name,
+        search_space=problem.search_space,
+        optimization_config=problem.optimization_config,
+        runner=SimulatedBackendRunner(
+            simulator=backend_simulator,
+            sample_runtime_func=async_benchmark_options.sample_runtime_func,
+        ),
+    )
+
+    scheduler_options = async_benchmark_options.scheduler_options or SchedulerOptions(
+        total_trials=None,
+        init_seconds_between_polls=1,
+        min_seconds_before_poll=1.0,
+        seconds_between_polls_backoff_factor=1.0,
+        logging_level=logging.INFO if verbose_logging else logging.WARNING,
+        early_stopping_strategy=async_benchmark_options.early_stopping_strategy,
+    )
+
+    scheduler = AsyncSimulatedBackendScheduler(
+        experiment=experiment,
+        generation_strategy=method,
+        max_pending_trials=async_benchmark_options.max_pending_trials,
+        options=scheduler_options,
+    )
+    scheduler.run_n_trials(
+        max_trials=num_trials, timeout_hours=async_benchmark_options.timeout_hours
+    )
+
+    # update the trial metadata with start time
+    # Note: we could also do it in the BackendSimulator if it got access to the Trial
+    for sim_trial in backend_simulator._completed:
+        metadata_dict = {
+            "start_time": sim_trial.sim_start_time,
+            "queued_time": sim_trial.sim_queued_time,
+            "completed_time": sim_trial.sim_completed_time,
+        }
+        experiment.trials[sim_trial.trial_index].update_run_metadata(metadata_dict)
+    return experiment, []
+
+
 def benchmark_minimize_callable(
     problem: BenchmarkProblem,
     num_trials: int,
