@@ -11,13 +11,13 @@ from datetime import datetime, timedelta
 from enum import Enum
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional
 
+from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
-from ax.core.data import Data
 from ax.core.generator_run import GeneratorRun
 from ax.core.metric import Metric
 from ax.core.runner import Runner
 from ax.core.types import TCandidateMetadata
-from ax.utils.common.base import Base
+from ax.utils.common.base import SortableBase
 from ax.utils.common.typeutils import not_none
 
 
@@ -33,16 +33,22 @@ class TrialStatus(int, Enum):
 
         CANDIDATE --> STAGED --> RUNNING --> COMPLETED
                   ------------->         --> FAILED (machine failure)
+                                         --> EARLY_STOPPED (deemed unpromising)
                   -------------------------> ABANDONED (human-initiated action)
 
     Trials may be abandoned at any time prior to completion or failure
     via human intervention. The difference between abandonment and failure
     is that the former is human-directed, while the latter is an internal
-    failure state.
+    failure state. Early-stopped refers to trials that were deemed
+    unpromising by an early-stopping strategy and therefore terminated.
 
     Additionally, when trials are deployed, they may be in an intermediate
     staged state (e.g. scheduled but waiting for resources) or immediately
     transition to running.
+
+    NOTE: Data for abandoned trials (or abandoned arms in batch trials) is
+    not passed to the model as part of training data, unless ``fit_abandoned``
+    option is specified to model bridge.
     """
 
     CANDIDATE = 0
@@ -52,6 +58,7 @@ class TrialStatus(int, Enum):
     RUNNING = 4
     ABANDONED = 5
     DISPATCHED = 6  # Deprecated.
+    EARLY_STOPPED = 7
 
     @property
     def is_terminal(self) -> bool:
@@ -60,12 +67,17 @@ class TrialStatus(int, Enum):
             self == TrialStatus.ABANDONED
             or self == TrialStatus.COMPLETED
             or self == TrialStatus.FAILED
+            or self == TrialStatus.EARLY_STOPPED
         )
 
     @property
     def expecting_data(self) -> bool:
         """True if trial is expecting data."""
-        return self == TrialStatus.RUNNING or self == TrialStatus.COMPLETED
+        return (
+            self == TrialStatus.RUNNING
+            or self == TrialStatus.COMPLETED
+            or self == TrialStatus.EARLY_STOPPED
+        )
 
     @property
     def is_deployed(self) -> bool:
@@ -97,6 +109,11 @@ class TrialStatus(int, Enum):
         """True if this trial is a running one."""
         return self == TrialStatus.RUNNING
 
+    @property
+    def is_early_stopped(self) -> bool:
+        """True if this trial is an early stopped one."""
+        return self == TrialStatus.EARLY_STOPPED
+
     # pyre-fixme[14]: `__format__` overrides method defined in `object` inconsistently.
     def __format__(self, fmt: str) -> str:
         """Define `__format__` to avoid pulling the `__format__` from the `int`
@@ -129,7 +146,7 @@ def immutable_once_run(func: Callable) -> Callable:
     return _immutable_once_run
 
 
-class BaseTrial(ABC, Base):
+class BaseTrial(ABC, SortableBase):
     """Base class for representing trials.
 
     Trials are containers for arms that are deployed together. There are
@@ -306,7 +323,9 @@ class BaseTrial(ABC, Base):
 
     def assign_runner(self) -> BaseTrial:
         """Assigns default experiment runner if trial doesn't already have one."""
-        self._runner = self._runner or self.experiment.runner_for_trial(self)
+        runner = self._runner or self.experiment.runner_for_trial(self)
+        if runner is not None:
+            self._runner = runner.clone()
         return self
 
     def update_run_metadata(self, metadata: Dict[str, Any]) -> Dict[str, Any]:
@@ -357,7 +376,9 @@ class BaseTrial(ABC, Base):
         self.mark_completed()
         return self
 
-    def fetch_data(self, metrics: Optional[List[Metric]] = None, **kwargs: Any) -> Data:
+    def fetch_data(
+        self, metrics: Optional[List[Metric]] = None, **kwargs: Any
+    ) -> AbstractDataFrameData:
         """Fetch data for this trial for all metrics on experiment.
 
         Args:
@@ -373,6 +394,26 @@ class BaseTrial(ABC, Base):
             trial_index=self.index, metrics=metrics, **kwargs
         )
 
+    def lookup_data(
+        self, merge_across_timestamps: bool = False
+    ) -> AbstractDataFrameData:
+        """Lookup cached data on experiment for this trial.
+
+        Args:
+            merge_across_timestamps: Whether to return all ``Data`` objects
+            for this trial, merged together. ``Data`` objects are identified
+            by trial index and timestamp in Ax.
+
+        Returns:
+            If not merging across timestamps, the latest ``Data`` object
+            associated with the trial. If merging, all data for trial, merged.
+
+        """
+        return self.experiment.lookup_data_for_trial(
+            trial_index=self.index,
+            merge_across_timestamps=merge_across_timestamps,
+        )[0]
+
     def _check_existing_and_name_arm(self, arm: Arm) -> None:
         """Sets name for given arm; if this arm is already in the
         experiment, uses the existing arm name.
@@ -524,6 +565,12 @@ class BaseTrial(ABC, Base):
     def mark_abandoned(self, reason: Optional[str] = None) -> BaseTrial:
         """Mark trial as abandoned.
 
+        NOTE: Arms in abandoned trials are considered to be 'pending points'
+        in experiment after their abandonment to avoid Ax models suggesting
+        the same arm again as a new candidate. Arms in abandoned trials are
+        also excluded from model training data unless ``fit_abandoned`` option
+        is specified to model bridge.
+
         Args:
             abandoned_reason: The reason the trial was abandoned.
 
@@ -551,6 +598,18 @@ class BaseTrial(ABC, Base):
         self._time_completed = datetime.now()
         return self
 
+    def mark_early_stopped(self) -> BaseTrial:
+        """Mark trial as early stopped.
+
+        Returns:
+            The trial instance.
+        """
+        if self._status != TrialStatus.RUNNING:
+            raise ValueError("Can only early stop trial that is currently running.")
+        self._status = TrialStatus.EARLY_STOPPED
+        self._time_completed = datetime.now()
+        return self
+
     def mark_as(self, status: TrialStatus, **kwargs: Any) -> BaseTrial:
         """Mark trial with a new TrialStatus.
 
@@ -573,10 +632,20 @@ class BaseTrial(ABC, Base):
             self.mark_failed()
         elif status == TrialStatus.COMPLETED:
             self.mark_completed()
+        elif status == TrialStatus.EARLY_STOPPED:
+            self.mark_early_stopped()
         else:
             raise ValueError(f"Cannot mark trial as {status}.")
         return self
 
+    def mark_arm_abandoned(
+        self, arm_name: str, reason: Optional[str] = None
+    ) -> BaseTrial:
+        raise NotImplementedError(
+            "Abandoning arms is only supported for `BatchTrial`. "
+            "Use `trial.mark_abandoned` if applicable."
+        )
+
     def _mark_failed_if_past_TTL(self) -> None:
         """If trial has TTL set and is running, check if the TTL has elapsed
         and mark the trial failed if so.
@@ -608,3 +677,7 @@ class BaseTrial(ABC, Base):
             self._experiment._trial_indices_by_status[self._status].remove(self.index)
         self._experiment._trial_indices_by_status[trial_status].add(self.index)
         self.__status = trial_status
+
+    @property
+    def _unique_id(self) -> str:
+        return str(self.index)
