@@ -4,18 +4,24 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from __future__ import annotations
+
 import logging
 from collections import OrderedDict, defaultdict
 from datetime import datetime
+from enum import Enum
 from functools import reduce
 from typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Type
 
 import pandas as pd
+from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial, TrialStatus
 from ax.core.batch_trial import BatchTrial
 from ax.core.data import Data
 from ax.core.generator_run import GeneratorRun
+from ax.core.map_data import MapData
+from ax.core.map_metric import MapMetric
 from ax.core.metric import Metric
 from ax.core.optimization_config import OptimizationConfig
 from ax.core.parameter import Parameter
@@ -28,11 +34,22 @@ from ax.utils.common.constants import Keys
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.logger import get_logger
 from ax.utils.common.timeutils import current_timestamp_in_millis
-
+from ax.utils.common.typeutils import checked_cast, not_none
 
 logger: logging.Logger = get_logger(__name__)
 
 
+class DataType(Enum):
+    DATA = 1
+    MAP_DATA = 2
+
+
+DATA_TYPE_LOOKUP: Dict[DataType, Type] = {
+    DataType.DATA: Data,
+    DataType.MAP_DATA: MapData,
+}
+
+
 # pyre-fixme[13]: Attribute `_search_space` is never initialized.
 class Experiment(Base):
     """Base class for defining an experiment."""
@@ -49,6 +66,7 @@ class Experiment(Base):
         is_test: bool = False,
         experiment_type: Optional[str] = None,
         properties: Optional[Dict[str, Any]] = None,
+        default_data_type: Optional[DataType] = None,
     ) -> None:
         """Inits Experiment.
 
@@ -63,6 +81,7 @@ class Experiment(Base):
             is_test: Convenience metadata tracker for the user to mark test experiments.
             experiment_type: The class of experiments this one belongs to.
             properties: Dictionary of this experiment's properties.
+            default_data_type: Enum representing the data type this experiment uses.
         """
         # appease pyre
         self._search_space: SearchSpace
@@ -73,13 +92,21 @@ class Experiment(Base):
         self.runner = runner
         self.is_test = is_test
 
-        self._data_by_trial: Dict[int, OrderedDict[int, Data]] = {}
+        self._data_by_trial: Dict[int, OrderedDict[int, AbstractDataFrameData]] = {}
         self._experiment_type: Optional[str] = experiment_type
         self._optimization_config = None
         self._tracking_metrics: Dict[str, Metric] = {}
         self._time_created: datetime = datetime.now()
         self._trials: Dict[int, BaseTrial] = {}
         self._properties: Dict[str, Any] = properties or {}
+        self._default_data_type = default_data_type or (
+            DataType.MAP_DATA
+            if (
+                optimization_config is not None
+                and isinstance(optimization_config.objective.metrics[0], MapMetric)
+            )
+            else DataType.DATA
+        )
         # Used to keep track of whether any trials on the experiment
         # specify a TTL. Since trials need to be checked for their TTL's
         # expiration often, having this attribute helps avoid unnecessary
@@ -258,7 +285,8 @@ class Experiment(Base):
     @optimization_config.setter
     def optimization_config(self, optimization_config: OptimizationConfig) -> None:
         if (
-            getattr(self, "_optimization_config", None) is not None
+            len(self.trials) > 0
+            and getattr(self, "_optimization_config", None) is not None
             and self.immutable_search_space_and_opt_config
         ):
             raise UnsupportedError(
@@ -378,7 +406,9 @@ class Experiment(Base):
             metrics_by_class[metric.fetch_multi_group_by_metric].append(metric)
         return metrics_by_class
 
-    def fetch_data(self, metrics: Optional[List[Metric]] = None, **kwargs: Any) -> Data:
+    def fetch_data(
+        self, metrics: Optional[List[Metric]] = None, **kwargs: Any
+    ) -> AbstractDataFrameData:
         """Fetches data for all trials on this experiment and for either the
         specified metrics or all metrics currently on the experiment, if `metrics`
         argument is not specified.
@@ -406,7 +436,7 @@ class Experiment(Base):
         trial_indices: Iterable[int],
         metrics: Optional[List[Metric]] = None,
         **kwargs: Any,
-    ) -> Data:
+    ) -> AbstractDataFrameData:
         """Fetches data for specific trials on the experiment.
 
         NOTE: For metrics that are not available while trial is running, the data
@@ -435,14 +465,15 @@ class Experiment(Base):
         trials: List[BaseTrial],
         metrics: Optional[Iterable[Metric]] = None,
         **kwargs: Any,
-    ) -> Data:
+    ) -> AbstractDataFrameData:
         if not self.metrics and not metrics:
             raise ValueError(
                 "No metrics to fetch data for, as no metrics are defined for "
                 "this experiment, and none were passed in to `fetch_data`."
             )
         if not any(t.status.expecting_data for t in trials):
-            return Data()
+            logger.info("No trials are in a state expecting data. Returning empty data")
+            return self.default_data_constructor()
         metrics_to_fetch = list(metrics or self.metrics.values())
         metrics_by_class = self._metrics_by_class(metrics=metrics_to_fetch)
         data_list = []
@@ -455,18 +486,20 @@ class Experiment(Base):
                     **kwargs,
                 )
             )
-        return Data.from_multiple_data(data=data_list)
+        return self.default_data_constructor.from_multiple_data(data=data_list)
 
     @copy_doc(BaseTrial.fetch_data)
     def _fetch_trial_data(
         self, trial_index: int, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> Data:
+    ) -> AbstractDataFrameData:
         trial = self.trials[trial_index]
         return self._lookup_or_fetch_trials_data(
             trials=[trial], metrics=metrics, **kwargs
         )
 
-    def attach_data(self, data: Data, combine_with_last_data: bool = False) -> int:
+    def attach_data(
+        self, data: AbstractDataFrameData, combine_with_last_data: bool = False
+    ) -> int:
         """Attach data to experiment. Stores data in `experiment._data_by_trial`,
         to be looked up via `experiment.lookup_data_for_trial`.
 
@@ -474,20 +507,26 @@ class Experiment(Base):
             data: Data object to store.
             combine_with_last_data: By default, when attaching data, it's identified
                 by its timestamp, and `experiment.lookup_data_for_trial` returns
-                data by most recent timestamp. In some cases, however, the goal
-                is to combine all data attached for a trial into a single `Data`
-                object. To achieve that goal, every call to `attach_data` after
+                data by most recent timestamp. Sometimes, however, we attach
+                data for some metrics at one point and data for the rest of the
+                metrics later on. In this case, we actually want to combine the
+                data so that one dataframe contains data for all metrics.
+                To achieve that goal, every call to `attach_data` after
                 the initial data is attached to trials, should be set to `True`.
-                Then, the newly attached data will be appended to existing data,
-                rather than stored as a separate object, and `lookup_data_for_trial`
-                will return the combined data object, rather than just the most
-                recently added data. This will validate that the newly added data
-                does not contain observations for the metrics that already have
-                observations in the most recent data stored.
+                In this case, we will take the most recent previously attached
+                data, append the newly attached data to it, and attach a new
+                Data object with the merged result. Afterwards, calls to
+                `lookup_data_for_trial` will return this combined data object,
+                rather than just the most recently added data. This operation
+                will also validate that the newly added data does not contain
+                observations for the metrics that already have observations in
+                the most recent data stored.
 
         Returns:
             Timestamp of storage in millis.
         """
+        data_type = type(data)
+        data_init_args = data.serialize_init_args(data)
         if data.df.empty:
             raise ValueError("Data to attach is empty.")
         metrics_not_on_exp = set(data.df["metric_name"].values) - set(
@@ -522,16 +561,23 @@ class Experiment(Base):
                         f"Last data for trial {trial_index} already contained an "
                         f"observation for metric {merged.head()['metric_name']}."
                     )
-                current_trial_data[cur_time_millis] = Data.from_multiple_data(
-                    [last_data, Data(trial_df)]
+                last_data_type = type(last_data)
+                current_trial_data[cur_time_millis] = last_data_type.from_multiple_data(
+                    [
+                        last_data,
+                        last_data_type(trial_df, **data_init_args),
+                    ]
                 )
             else:
-                current_trial_data[cur_time_millis] = Data(trial_df)
+                # pyre-ignore [45]: Cannot instantiate `AbstractDataFrameData`.
+                current_trial_data[cur_time_millis] = data_type(
+                    trial_df, **data_init_args
+                )
             self._data_by_trial[trial_index] = current_trial_data
 
         return cur_time_millis
 
-    def lookup_data_for_ts(self, timestamp: int) -> Data:
+    def lookup_data_for_ts(self, timestamp: int) -> AbstractDataFrameData:
         """Collect data for all trials stored at this timestamp.
 
         Useful when many trials' data was fetched and stored simultaneously
@@ -551,9 +597,11 @@ class Experiment(Base):
             if timestamp in ts_to_data:
                 trial_datas.append(ts_to_data[timestamp])
 
-        return Data.from_multiple_data(trial_datas)
+        return self.default_data_constructor.from_multiple_data(trial_datas)
 
-    def lookup_data_for_trial(self, trial_index: int) -> Tuple[Data, int]:
+    def lookup_data_for_trial(
+        self, trial_index: int, merge_across_timestamps: bool = False
+    ) -> Tuple[AbstractDataFrameData, int]:
         """Lookup stored data for a specific trial.
 
         Returns latest data object, and its storage timestamp, present for this trial.
@@ -561,6 +609,8 @@ class Experiment(Base):
 
         Args:
             trial_index: The index of the trial to lookup data for.
+            merge_across_timestamps: Whether to return Data from all timestamps instead
+                of only the latest.
 
         Returns:
             The requested data object, and its storage timestamp in milliseconds.
@@ -568,15 +618,51 @@ class Experiment(Base):
         try:
             trial_data_dict = self._data_by_trial[trial_index]
         except KeyError:
-            return (Data(), -1)
+            return (self.default_data_constructor(), -1)
 
         if len(trial_data_dict) == 0:
-            return (Data(), -1)
+            return (self.default_data_constructor(), -1)
 
         storage_time = max(trial_data_dict.keys())
         trial_data = trial_data_dict[storage_time]
+        if merge_across_timestamps:
+            trial_data = trial_data.from_multiple_data(
+                data=list(trial_data_dict.values())
+            )
+            trial_data = checked_cast(AbstractDataFrameData, trial_data)
         return trial_data, storage_time
 
+    def lookup_data(
+        self,
+        trial_indices: Optional[Iterable[int]] = None,
+        merge_across_timestamps: bool = False,
+    ) -> AbstractDataFrameData:
+        """Lookup data for all trials on this experiment and for either the
+        specified metrics or all metrics currently on the experiment, if `metrics`
+        argument is not specified.
+
+        Args:
+            trial_indices: Indices of trials, for which to fetch data.
+            merge_across_timestamps: Whether to return data across all timestamps.
+
+        Returns:
+            Data for the experiment.
+        """
+        data_by_trial = []
+        trial_indices = trial_indices or list(self.trials.keys())
+        for trial_index in trial_indices:
+            data_by_trial.append(
+                self.lookup_data_for_trial(
+                    trial_index=trial_index,
+                    merge_across_timestamps=merge_across_timestamps,
+                )[0]
+            )
+        if not data_by_trial:
+            return self.default_data_constructor()
+        last_data = data_by_trial[-1]
+        last_data_type = type(last_data)
+        return last_data_type.from_multiple_data(data_by_trial)
+
     @property
     def num_trials(self) -> int:
         """How many trials are associated with this experiment."""
@@ -617,6 +703,14 @@ class Experiment(Base):
         self._check_TTL_on_running_trials()  # Marks past-TTL trials as failed.
         return self._trial_indices_by_status
 
+    @property
+    def default_data_type(self) -> DataType:
+        return self._default_data_type
+
+    @property
+    def default_data_constructor(self) -> Type:
+        return DATA_TYPE_LOOKUP[self.default_data_type]
+
     def new_trial(
         self,
         generator_run: Optional[GeneratorRun] = None,
@@ -730,7 +824,7 @@ class Experiment(Base):
                 raise ValueError("BatchTrial already attached to experiment.")
 
         if index is not None and index in self._trials:
-            logger.warning(  # pragma: no cover
+            logger.debug(  # pragma: no cover
                 f"Trial index {index} already exists on the experiment. Overwriting."
             )
         index = (
@@ -741,6 +835,88 @@ class Experiment(Base):
         self._trials[index] = trial
         return index
 
+    def warm_start_from_old_experiment(self, old_experiment: Experiment) -> List[Trial]:
+        """Copy all completed trials with data from an old Ax expeirment to this one.
+        This function checks that the parameters of each trial are members of the
+        current experiment's search_space.
+
+        NOTE: Currently only handles experiments with 1-arm ``Trial``-s, not
+        ``BatchTrial``-s as there has not yet been need for support of the latter.
+
+        Args:
+            old_experiment: The experiment from which to transfer trials and data
+
+        Returns:
+            List of trials successfully copied from old_experiment to this one
+        """
+        if len(self.trials) > 0:
+            raise ValueError(  # pragma: no cover
+                f"Can only warm-start experiments that don't yet have trials. "
+                f"Experiment {self.name} has {len(self.trials)} trials."
+            )
+
+        old_parameter_names = set(old_experiment.search_space.parameters.keys())
+        parameter_names = set(self.search_space.parameters.keys())
+        if old_parameter_names.symmetric_difference(parameter_names):
+            raise ValueError(  # pragma: no cover
+                f"Cannot warm-start experiment '{self.name}' from experiment "
+                f"'{old_experiment.name}' due to mismatch in search space parameters."
+                f"Parameters in '{self.name}' but not in '{old_experiment.name}': "
+                f"{old_parameter_names - parameter_names}. Vice-versa: "
+                f"{parameter_names - old_parameter_names}."
+            )
+
+        old_completed_trials = old_experiment.trials_by_status[TrialStatus.COMPLETED]
+        copied_trials = []
+        for trial in old_completed_trials:
+            if not isinstance(trial, Trial):
+                raise NotImplementedError(  # pragma: no cover
+                    "Only experiments with 1-arm trials currently supported."
+                )
+            self.search_space.check_membership(
+                not_none(trial.arm).parameters, raise_error=True
+            )
+            dat, ts = old_experiment.lookup_data_for_trial(trial_index=trial.index)
+            if ts != -1 and not dat.df.empty:
+                # Trial has data, so we replicate it on the new experiment.
+                new_trial = self.new_trial()
+                new_trial.add_arm(not_none(trial.arm).clone(clear_name=True))
+                new_trial.mark_running(no_runner_required=True)
+                new_trial.update_run_metadata(
+                    {"run_id": trial.run_metadata.get("run_id")}
+                )
+                new_trial._properties["source"] = (
+                    f"Warm start from Experiment: `{old_experiment.name}`, "
+                    f"trial: `{trial.index}`"
+                )
+                # Set trial index and arm name to their values in new trial.
+                new_df = dat.df.copy()
+                new_df["trial_index"].replace(
+                    {trial.index: new_trial.index}, inplace=True
+                )
+                new_df["arm_name"].replace(
+                    {not_none(trial.arm).name: not_none(new_trial.arm).name},
+                    inplace=True,
+                )
+                # Attach updated data to new trial on experiment and mark trial
+                # as completed.
+                self.attach_data(data=Data(df=new_df))
+                new_trial.mark_completed()
+                copied_trials.append(new_trial)
+
+        if self._name is not None:
+            logger.info(
+                f"Copied {len(copied_trials)} completed trials and their data "
+                f"from {old_experiment.name} to {self.name}."
+            )
+        else:
+            logger.info(
+                f"Copied {len(copied_trials)} completed trials and their data "
+                f"from {old_experiment.name}."
+            )
+
+        return copied_trials
+
     def _name_and_store_arm_if_not_exists(self, arm: Arm, proposed_name: str) -> None:
         """Tries to lookup arm with same signature, otherwise names and stores it.
 
