@@ -10,7 +10,7 @@ import warnings
 from typing import Any, Iterable, List, Optional, Tuple
 
 from ax.core.metric import Metric
-from ax.utils.common.base import Base
+from ax.utils.common.base import SortableBase
 from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import not_none
 
@@ -18,7 +18,7 @@ from ax.utils.common.typeutils import not_none
 logger = get_logger(__name__)
 
 
-class Objective(Base):
+class Objective(SortableBase):
     """Base class for representing an objective.
 
     Attributes:
@@ -85,6 +85,10 @@ class Objective(Base):
         """Return a list of metrics that are incompatible with OutcomeConstraints."""
         return self.metrics
 
+    @property
+    def _unique_id(self) -> str:
+        return str(self)
+
 
 class MultiObjective(Objective):
     """Class for an objective composed of a multiple component objectives.
@@ -92,40 +96,51 @@ class MultiObjective(Objective):
     The Acquisition function determines how the objectives are weighted.
 
     Attributes:
-        metrics: List of metrics.
+        objectives: List of objectives.
     """
 
     weights: List[float]
 
     def __init__(
         self,
-        metrics: List[Metric],
-        minimize: bool = False,
+        objectives: Optional[List[Objective]] = None,
         **extra_kwargs: Any,  # Here to satisfy serialization.
     ) -> None:
         """Create a new objective.
 
         Args:
-            metrics: The list of metrics to be jointly optimized.
-            minimize: If true, minimize the aggregate of these metrics.
+            objectives: The list of objectives to be jointly optimized.
 
         """
-        self._metrics = metrics
-        self.weights = []
-        for metric in metrics:
-            # Set weights from "lower_is_better"
-            if metric.lower_is_better is None:
-                logger.warning(
-                    f"metric {metric.name} has not set `lower_is_better`. "
-                    "Treating as `False` (Metric should be maximized)."
+        # Support backwards compatibility for old API in which
+        # MultiObjective constructor accepted `metrics` and `minimize`
+        # rather than `objectives`
+        if objectives is None:
+            if "metrics" not in extra_kwargs:
+                raise ValueError(
+                    "Must either specify `objectives` or `metrics` "
+                    "as input to `MultiObjective` constructor."
                 )
-            self.weights.append(-1.0 if metric.lower_is_better is True else 1.0)
-        self.minimize = minimize
+            metrics = extra_kwargs["metrics"]
+            minimize = extra_kwargs.get("minimize", False)
+            warnings.warn(
+                "Passing `metrics` and `minimize` as input to the `MultiObjective` "
+                "constructor will soon be deprecated. Instead, pass a list of "
+                "`objectives`. This will become an error in the future.",
+                DeprecationWarning,
+            )
+            objectives = []
+            for metric in metrics:
+                lower_is_better = metric.lower_is_better or False
+                _minimize = not lower_is_better if minimize else lower_is_better
+                objectives.append(Objective(metric=metric, minimize=_minimize))
 
-    @property
-    def metric_weights(self) -> Iterable[Tuple[Metric, float]]:
-        """Get the objective metrics and weights."""
-        return zip(self.metrics, self.weights)
+        self._objectives = not_none(objectives)
+
+        # For now, assume all objectives are weighted equally.
+        # This might be used in the future to change emphasis on the
+        # relative focus of the exploration during the optimization.
+        self.weights = [1.0 for _ in range(len(objectives))]
 
     @property
     def metric(self) -> Metric:
@@ -137,25 +152,31 @@ class MultiObjective(Objective):
     @property
     def metrics(self) -> List[Metric]:
         """Get the objective metrics."""
-        return self._metrics
+        return [o.metric for o in self._objectives]
+
+    @property
+    def objectives(self) -> List[Objective]:
+        """Get the objectives."""
+        return self._objectives
+
+    @property
+    def objective_weights(self) -> Iterable[Tuple[Objective, float]]:
+        """Get the objectives and weights."""
+        return zip(self.objectives, self.weights)
 
     def clone(self) -> Objective:
         """Create a copy of the objective."""
-        return MultiObjective(
-            metrics=[m.clone() for m in self.metrics], minimize=self.minimize
-        )
+        return MultiObjective(objectives=[o.clone() for o in self.objectives])
 
     def __repr__(self) -> str:
-        return "MultiObjective(metric_names={}, minimize={})".format(
-            [metric.name for metric in self.metrics], self.minimize
-        )
+        return f"MultiObjective(objectives={self.objectives})"
 
     def get_unconstrainable_metrics(self) -> List[Metric]:
         """Return a list of metrics that are incompatible with OutcomeConstraints."""
         return []
 
 
-class ScalarizedObjective(MultiObjective):
+class ScalarizedObjective(Objective):
     """Class for an objective composed of a linear scalarization of metrics.
 
     Attributes:
@@ -184,8 +205,27 @@ class ScalarizedObjective(MultiObjective):
         else:
             if len(weights) != len(metrics):
                 raise ValueError("Length of weights must equal length of metrics")
-        super().__init__(metrics, minimize)
+
+        self._metrics = metrics
         self.weights = weights
+        self.minimize = minimize
+
+    @property
+    def metric(self) -> Metric:
+        """Override base method to error."""
+        raise NotImplementedError(
+            f"{type(self).__name__} is composed of multiple metrics"
+        )
+
+    @property
+    def metrics(self) -> List[Metric]:
+        """Get the metrics."""
+        return self._metrics
+
+    @property
+    def metric_weights(self) -> Iterable[Tuple[Metric, float]]:
+        """Get the metrics and weights."""
+        return zip(self.metrics, self.weights)
 
     def clone(self) -> Objective:
         """Create a copy of the objective."""
@@ -199,3 +239,7 @@ class ScalarizedObjective(MultiObjective):
         return "ScalarizedObjective(metric_names={}, weights={}, minimize={})".format(
             [metric.name for metric in self.metrics], self.weights, self.minimize
         )
+
+    def get_unconstrainable_metrics(self) -> List[Metric]:
+        """Return a list of metrics that are incompatible with OutcomeConstraints."""
+        return []
