@@ -8,11 +8,12 @@ from itertools import groupby
 from typing import Dict, List, Optional
 
 from ax.core.metric import Metric
-from ax.core.objective import MultiObjective, Objective
+from ax.core.objective import MultiObjective, Objective, ScalarizedObjective
 from ax.core.outcome_constraint import (
     ComparisonOp,
     ObjectiveThreshold,
     OutcomeConstraint,
+    ScalarizedOutcomeConstraint,
 )
 from ax.utils.common.base import Base
 from ax.utils.common.logger import get_logger
@@ -102,10 +103,22 @@ class OptimizationConfig(Base):
     @property
     def metrics(self) -> Dict[str, Metric]:
         constraint_metrics = {
-            oc.metric.name: oc.metric for oc in self._outcome_constraints
+            oc.metric.name: oc.metric
+            for oc in self._outcome_constraints
+            if not isinstance(oc, ScalarizedOutcomeConstraint)
+        }
+        scalarized_constraint_metrics = {
+            metric.name: metric
+            for oc in self._outcome_constraints
+            if isinstance(oc, ScalarizedOutcomeConstraint)
+            for metric in oc.metrics
         }
         objective_metrics = {metric.name: metric for metric in self.objective.metrics}
-        return {**constraint_metrics, **objective_metrics}
+        return {
+            **constraint_metrics,
+            **scalarized_constraint_metrics,
+            **objective_metrics,
+        }
 
     @outcome_constraints.setter
     def outcome_constraints(self, outcome_constraints: List[OutcomeConstraint]) -> None:
@@ -140,6 +153,12 @@ class OptimizationConfig(Base):
                 )
             )
         outcome_constraints = outcome_constraints or []
+        # only vaidate outcome_constraints
+        outcome_constraints = [
+            constraint
+            for constraint in outcome_constraints
+            if isinstance(constraint, ScalarizedOutcomeConstraint) is False
+        ]
         unconstrainable_metrics = objective.get_unconstrainable_metrics()
         OptimizationConfig._validate_outcome_constraints(
             unconstrainable_metrics=unconstrainable_metrics,
@@ -276,9 +295,23 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
 
     @property
     def metrics(self) -> Dict[str, Metric]:
-        constraint_metrics = {oc.metric.name: oc.metric for oc in self.all_constraints}
+        constraint_metrics = {
+            oc.metric.name: oc.metric
+            for oc in self.all_constraints
+            if not isinstance(oc, ScalarizedOutcomeConstraint)
+        }
+        scalarized_constraint_metrics = {
+            metric.name: metric
+            for oc in self.all_constraints
+            if isinstance(oc, ScalarizedOutcomeConstraint)
+            for metric in oc.metrics
+        }
         objective_metrics = {metric.name: metric for metric in self.objective.metrics}
-        return {**constraint_metrics, **objective_metrics}
+        return {
+            **constraint_metrics,
+            **scalarized_constraint_metrics,
+            **objective_metrics,
+        }
 
     @property
     def objective_thresholds(self) -> List[ObjectiveThreshold]:
@@ -295,6 +328,13 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         )
         self._objective_thresholds = objective_thresholds
 
+    @property
+    def objective_thresholds_dict(self) -> Dict[str, ObjectiveThreshold]:
+        """Get a mapping from objective metric name to the corresponding
+        threshold.
+        """
+        return {ot.metric.name: ot for ot in self._objective_thresholds}
+
     @staticmethod
     def _validate_optimization_config(
         objective: Objective,
@@ -312,36 +352,21 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         Args:
             outcome_constraints: Constraints to validate.
         """
-        if not isinstance(objective, MultiObjective):
+        if not isinstance(objective, (MultiObjective, ScalarizedObjective)):
             raise TypeError(
                 (
                     "`MultiObjectiveOptimizationConfig` requires an objective "
-                    "of type `MultiObjective`. Use `OptimizationConfig` instead "
-                    "if using a single-metric objective."
+                    "of type `MultiObjective` or `ScalarizedObjective`. "
+                    "Use `OptimizationConfig` instead if using a "
+                    "single-metric objective."
                 )
             )
         outcome_constraints = outcome_constraints or []
         objective_thresholds = objective_thresholds or []
 
-        # Verify we aren't optimizing too many objectives.
-        objective_metrics_by_name = {
-            metric.name: metric for metric in objective.metrics
-        }
-        # Warn if thresholds on objective_metrics bound from the wrong direction.
-        for threshold in objective_thresholds:
-            metric_name = threshold.metric.name
-            if metric_name in objective_metrics_by_name:
-                lower_is_better = threshold.metric.lower_is_better
-                bounded_above = threshold.op == ComparisonOp.LEQ
-                is_aligned = lower_is_better == bounded_above
-                if not (is_aligned or lower_is_better is None):
-                    raise ValueError(
-                        make_wrong_direction_warning(
-                            metric_name=metric_name,
-                            bounded_above=bounded_above,
-                            lower_is_better=lower_is_better,
-                        )
-                    )
+        maybe_raise_wrong_direction_warning(
+            objective=objective, objective_thresholds=objective_thresholds
+        )
 
         unconstrainable_metrics = objective.get_unconstrainable_metrics()
         OptimizationConfig._validate_outcome_constraints(
@@ -361,12 +386,26 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         )
 
 
-def make_wrong_direction_warning(
-    metric_name: str, bounded_above: bool, lower_is_better: Optional[bool]
-) -> str:
-    return (
-        f"Constraint on {metric_name} bounds from "
-        f"{'above' if bounded_above else 'below'} "
-        f"but {metric_name} is being "
-        f"{'minimized' if lower_is_better else 'maximized'}."
-    ).format(metric_name)
+def maybe_raise_wrong_direction_warning(
+    objective: Objective,
+    objective_thresholds: List[ObjectiveThreshold],
+) -> None:
+    """Warn if thresholds on objective_metrics bound from the wrong direction."""
+    if not isinstance(objective, MultiObjective):
+        return
+
+    objectives_by_name = {obj.metric.name: obj for obj in objective.objectives}
+    for threshold in objective_thresholds:
+        metric_name = threshold.metric.name
+        if metric_name in objectives_by_name:
+            minimize = objectives_by_name[metric_name].minimize
+            bounded_above = threshold.op == ComparisonOp.LEQ
+            is_aligned = minimize == bounded_above
+            if not is_aligned:
+                message = (
+                    f"Constraint on {metric_name} bounds from "
+                    f"{'above' if bounded_above else 'below'} "
+                    f"but {metric_name} is being "
+                    f"{'minimized' if minimize else 'maximized'}."
+                ).format(metric_name)
+                raise ValueError(message)
