@@ -11,7 +11,9 @@ import pandas as pd
 from ax.core.arm import Arm
 from ax.core.base_trial import TrialStatus
 from ax.core.data import Data
-from ax.core.experiment import Experiment
+from ax.core.experiment import DataType, Experiment
+from ax.core.map_data import MapData
+from ax.core.map_metric import MapMetric
 from ax.core.metric import Metric
 from ax.core.parameter import FixedParameter, ParameterType
 from ax.core.search_space import SearchSpace
@@ -25,11 +27,18 @@ from ax.utils.testing.core_stubs import (
     get_branin_arms,
     get_branin_optimization_config,
     get_branin_search_space,
+    get_branin_experiment,
+    get_branin_experiment_with_timestamp_map_metric,
     get_data,
     get_experiment,
+    get_experiment_with_map_data,
+    get_experiment_with_map_data_type,
+    get_map_data,
     get_optimization_config,
     get_search_space,
+    get_sobol,
     get_status_quo,
+    get_scalarized_outcome_constraint,
 )
 
 
@@ -141,6 +150,21 @@ class ExperimentTest(TestCase):
             len(get_optimization_config().metrics) + 1, len(self.experiment.metrics)
         )
 
+        # Add optimization config with 1 scalarized constraint composed of 2 metrics
+        opt_config = get_optimization_config()
+        opt_config.outcome_constraints = opt_config.outcome_constraints + [
+            get_scalarized_outcome_constraint()
+        ]
+        self.experiment.optimization_config = opt_config
+
+        # Verify total metrics size is the same.
+        self.assertEqual(len(opt_config.metrics) + 1, len(self.experiment.metrics))
+        self.assertEqual(
+            len(get_optimization_config().metrics) + 3, len(self.experiment.metrics)
+        )
+        # set back
+        self.experiment.optimization_config = get_optimization_config()
+
         # Test adding new tracking metric
         self.experiment.add_tracking_metric(Metric(name="m4"))
         self.assertEqual(
@@ -330,8 +354,8 @@ class ExperimentTest(TestCase):
             {"not_yet_on_experiment"},
         )
 
-        # Verify data lookup is empty for trial that does not yet have data.
-        self.assertEqual(len(exp.lookup_data_for_trial(1)[0].df), 0)
+        # Verify data lookup includes trials attached from `fetch_data`.
+        self.assertEqual(len(exp.lookup_data_for_trial(1)[0].df), 30)
 
         # Test local storage
         t1 = exp.attach_data(batch_data)
@@ -339,7 +363,7 @@ class ExperimentTest(TestCase):
 
         full_dict = exp.data_by_trial
         self.assertEqual(len(full_dict), 2)  # data for 2 trials
-        self.assertEqual(len(full_dict[0]), 3)  # 3 data objs for batch 0
+        self.assertEqual(len(full_dict[0]), 5)  # 5 data objs for batch 0
 
         # Test retrieving original batch 0 data
         self.assertEqual(len(exp.lookup_data_for_ts(t1).df), n)
@@ -348,6 +372,11 @@ class ExperimentTest(TestCase):
         # Test retrieving full exp data
         self.assertEqual(len(exp.lookup_data_for_ts(t2).df), 4 * n)
 
+        # Test merging multiple timestamps of data
+        self.assertEqual(
+            len(exp.lookup_data_for_trial(0, merge_across_timestamps=True)), 2
+        )
+
         with self.assertRaisesRegex(ValueError, ".* for metric"):
             exp.attach_data(batch_data, combine_with_last_data=True)
 
@@ -365,7 +394,7 @@ class ExperimentTest(TestCase):
             )
         )
         t3 = exp.attach_data(new_data, combine_with_last_data=True)
-        self.assertEqual(len(full_dict[0]), 4)  # 4 data objs for batch 0 now
+        self.assertEqual(len(full_dict[0]), 6)  # 6 data objs for batch 0 now
         self.assertIn("z", exp.lookup_data_for_ts(t3).df["metric_name"].tolist())
 
         # Verify we don't get the data if the trial is abandoned
@@ -491,7 +520,7 @@ class ExperimentTest(TestCase):
         # Try to fetch data when there are only metrics and no attached data.
         exp.remove_tracking_metric(metric_name="b")  # Remove implemented metric.
         exp.add_tracking_metric(Metric(name="b"))  # Add unimplemented metric.
-        self.assertTrue(exp.fetch_trials_data(trial_indices=[0]).df.empty)
+        self.assertEqual(len(exp.fetch_trials_data(trial_indices=[0]).df), 5)
         # Try fetching attached data.
         exp.attach_data(batch_0_data)
         exp.attach_data(batch_1_data)
@@ -514,9 +543,9 @@ class ExperimentTest(TestCase):
             properties={Keys.IMMUTABLE_SEARCH_SPACE_AND_OPT_CONF: True},
         )
         self.assertTrue(immutable_exp.immutable_search_space_and_opt_config)
+        immutable_exp.new_batch_trial()
         with self.assertRaises(UnsupportedError):
             immutable_exp.optimization_config = get_branin_optimization_config()
-        immutable_exp.new_batch_trial()
         with self.assertRaises(UnsupportedError):
             immutable_exp.search_space = get_branin_search_space()
 
@@ -558,12 +587,14 @@ class ExperimentTest(TestCase):
         # Since metric is available while trial is running, we should be
         # refetching the data and no data should be attached to experiment.
         mock_fetch_exp_data_multi.assert_called_once()
-        self.assertEqual(len(exp._data_by_trial), 0)
+        self.assertEqual(len(exp._data_by_trial), 2)
 
         with patch(
             f"{BraninMetric.__module__}.BraninMetric.is_available_while_running",
             return_value=False,
         ):
+            exp = self._setupBraninExperiment(n=5)
+            exp.fetch_data()
             # 1. No completed trials => no fetch case.
             mock_fetch_exp_data_multi.reset_mock()
             dat = exp.fetch_data()
@@ -588,3 +619,185 @@ class ExperimentTest(TestCase):
             # All fetched data should get cached, so no fetch should happen next time.
             exp.fetch_data()
             mock_fetch_exp_data_multi.assert_not_called()
+
+
+class ExperimentWithMapDataTest(TestCase):
+    def setUp(self):
+        self.experiment = get_experiment_with_map_data_type()
+
+    def _setupBraninExperiment(self, n: int) -> Experiment:
+        exp = get_branin_experiment_with_timestamp_map_metric()
+        batch = exp.new_batch_trial()
+        batch.add_arms_and_weights(arms=get_branin_arms(n=n, seed=0))
+        batch.run()
+
+        batch_2 = exp.new_batch_trial()
+        batch_2.add_arms_and_weights(arms=get_branin_arms(n=3 * n, seed=1))
+        batch_2.run()
+        return exp
+
+    def testEmptyMetrics(self):
+        empty_experiment = Experiment(
+            name="test_experiment",
+            search_space=get_search_space(),
+            default_data_type=DataType.MAP_DATA,
+        )
+        self.assertEqual(empty_experiment.num_trials, 0)
+        empty_experiment.add_tracking_metric(Metric(name="ax_test_metric"))
+        self.assertTrue(empty_experiment.fetch_data().df.empty)
+        empty_experiment.attach_data(get_map_data())
+
+    def testFetchDataWithMapData(self):
+        evaluations = {
+            "0_0": [
+                ({"epoch": 1}, {"no_fetch_impl_metric": (3.7, 0.5)}),
+                ({"epoch": 2}, {"no_fetch_impl_metric": (3.8, 0.5)}),
+                ({"epoch": 3}, {"no_fetch_impl_metric": (3.9, 0.5)}),
+                ({"epoch": 4}, {"no_fetch_impl_metric": (4.0, 0.5)}),
+            ],
+        }
+
+        self.experiment.add_tracking_metric(
+            metric=MapMetric(name="no_fetch_impl_metric")
+        )
+        self.experiment.new_trial()
+        self.experiment.trials[0].mark_running(no_runner_required=True)
+        first_epoch = MapData.from_map_evaluations(
+            evaluations={
+                arm_name: partial_results[0:1]
+                for arm_name, partial_results in evaluations.items()
+            },
+            trial_index=0,
+        )
+        self.experiment.attach_data(first_epoch)
+        remaining_epochs = MapData.from_map_evaluations(
+            evaluations={
+                arm_name: partial_results[1:4]
+                for arm_name, partial_results in evaluations.items()
+            },
+            trial_index=0,
+        )
+        self.experiment.attach_data(remaining_epochs)
+        self.experiment.trials[0].mark_completed()
+
+        # merge_across_timestamps = False
+        expected_data = remaining_epochs
+        actual_data = self.experiment.lookup_data()  # False by default.
+        self.assertEqual(expected_data, actual_data)
+
+        # merge_across_timestamps = True
+        expected_data = MapData.from_map_evaluations(
+            evaluations=evaluations, trial_index=0
+        )
+        actual_data = self.experiment.lookup_data(merge_across_timestamps=True)
+        self.assertEqual(expected_data, actual_data)
+
+    def testFetchTrialsData(self):
+        exp = self._setupBraninExperiment(n=5)
+        batch_0 = exp.trials[0]
+        batch_1 = exp.trials[1]
+        batch_0.mark_completed()
+        batch_1.mark_completed()
+        batch_0_data = exp.fetch_trials_data(trial_indices=[0])
+        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
+        self.assertEqual(
+            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
+        )
+        batch_1_data = exp.fetch_trials_data(trial_indices=[1])
+        self.assertEqual(set(batch_1_data.df["trial_index"].values), {1})
+        self.assertEqual(
+            set(batch_1_data.df["arm_name"].values), {a.name for a in batch_1.arms}
+        )
+        self.assertEqual(
+            exp.fetch_trials_data(trial_indices=[0, 1]),
+            MapData.from_multiple_data([batch_0_data, batch_1_data]),
+        )
+        with self.assertRaisesRegex(ValueError, ".* not associated .*"):
+            exp.fetch_trials_data(trial_indices=[2])
+        # Try to fetch data when there are only metrics and no attached data.
+        exp.remove_tracking_metric(metric_name="b")  # Remove implemented metric.
+        exp.add_tracking_metric(MapMetric(name="b"))  # Add unimplemented metric.
+        self.assertEqual(len(exp.fetch_trials_data(trial_indices=[0]).df), 30)
+        # Try fetching attached data.
+        exp.attach_data(batch_0_data)
+        exp.attach_data(batch_1_data)
+        self.assertEqual(exp.fetch_trials_data(trial_indices=[0]), batch_0_data)
+        self.assertEqual(exp.fetch_trials_data(trial_indices=[1]), batch_1_data)
+        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
+        self.assertEqual(
+            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
+        )
+
+    def testFetchTrialDataLookupOnly(self):
+        experiment = get_experiment_with_map_data()
+        self.assertTrue(experiment.optimization_config.objective.metric)
+        experiment.trials[0].mark_running(no_runner_required=True)
+        actual_data = experiment.fetch_trials_data(trial_indices=[0], lookup_only=True)
+        expected_data = get_map_data()
+        self.assertNotEqual(expected_data, actual_data)
+        with patch(
+            f"{MapMetric.__module__}.MapMetric.is_available_while_running",
+            return_value=True,
+        ):
+            experiment = get_experiment_with_map_data()
+            self.assertTrue(experiment.optimization_config.objective.metric)
+            experiment.trials[0].mark_running(no_runner_required=True)
+            actual_data = experiment.lookup_data()
+            expected_data = get_map_data()
+            self.assertEqual(expected_data, actual_data)
+
+    def testWarmStartFromOldExperiment(self):
+        # create old_experiment
+        len_old_trials = 5
+        i_failed_trial = 3
+        old_experiment = get_branin_experiment()
+        for i_old_trial in range(len_old_trials):
+            sobol_run = get_sobol(search_space=old_experiment.search_space).gen(n=1)
+            trial = old_experiment.new_trial(generator_run=sobol_run)
+            trial.mark_running(no_runner_required=True)
+            if i_old_trial == i_failed_trial:
+                trial.mark_failed()
+            else:
+                trial.mark_completed()
+        # make metric noiseless for exact reproducibility
+        old_experiment.optimization_config.objective.metric.noise_sd = 0
+        old_experiment.fetch_data()
+
+        # should fail if new_experiment has trials
+        new_experiment = get_branin_experiment(with_trial=True)
+        with self.assertRaisesRegex(ValueError, "Experiment.*has.*trials"):
+            new_experiment.warm_start_from_old_experiment(old_experiment=old_experiment)
+
+        # should fail if search spaces are different
+        with self.assertRaisesRegex(ValueError, "mismatch in search space parameters"):
+            self.experiment.warm_start_from_old_experiment(
+                old_experiment=old_experiment
+            )
+
+        # check that all non-failed trials are copied to new_experiment
+        new_experiment = get_branin_experiment()
+        # make metric noiseless for exact reproducibility
+        new_experiment.optimization_config.objective.metric.noise_sd = 0
+        new_experiment.warm_start_from_old_experiment(old_experiment=old_experiment)
+        self.assertEqual(len(new_experiment.trials), len(old_experiment.trials) - 1)
+        i_old_trial = 0
+        for _, trial in new_experiment.trials.items():
+            # skip failed trial
+            i_old_trial += i_old_trial == i_failed_trial
+            self.assertEqual(
+                trial.arm.parameters, old_experiment.trials[i_old_trial].arm.parameters
+            )
+            self.assertRegex(
+                trial._properties["source"], "Warm start.*Experiment.*trial"
+            )
+            i_old_trial += 1
+
+        # Check that the data was attached for correct trials
+        old_df = old_experiment.fetch_data().df
+        new_df = new_experiment.fetch_data().df
+
+        self.assertEqual(len(new_df), len_old_trials - 1)
+        pd.testing.assert_frame_equal(
+            old_df.drop(["arm_name", "trial_index"], axis=1),
+            new_df.drop(["arm_name", "trial_index"], axis=1),
+        )
