@@ -6,7 +6,13 @@
 
 # flake8: noqa F401
 from ax.metrics.branin import BraninMetric
+from ax.metrics.chemistry import ChemistryMetric
 from ax.metrics.factorial import FactorialMetric
+from ax.metrics.sklearn import SklearnMetric
 
-
-__all__ = ["BraninMetric", "FactorialMetric"]
+__all__ = [
+    "BraninMetric",
+    "ChemistryMetric",
+    "FactorialMetric",
+    "SklearnMetric",
+]
