@@ -4,6 +4,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from dataclasses import dataclass
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple
 
 import numpy as np
@@ -11,6 +12,7 @@ from ax.core.observation import ObservationData, ObservationFeatures
 from ax.core.optimization_config import (
     OptimizationConfig,
 )
+from ax.core.outcome_constraint import ScalarizedOutcomeConstraint
 from ax.core.search_space import SearchSpace
 from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
 from ax.modelbridge.base import ModelBridge
@@ -19,13 +21,14 @@ from ax.modelbridge.modelbridge_utils import (
     extract_objective_weights,
     extract_outcome_constraints,
     extract_parameter_constraints,
-    get_bounds_and_task,
+    extract_search_space_digest,
     get_fixed_features,
     observation_data_to_array,
     observation_features_to_array,
     parse_observation_features,
     pending_observations_as_array,
     transform_callback,
+    SearchSpaceDigest,
 )
 from ax.utils.common.typeutils import not_none
 
@@ -33,15 +36,34 @@ from ax.utils.common.typeutils import not_none
 FIT_MODEL_ERROR = "Model must be fit before {action}."
 
 
+@dataclass
+class ArrayModelGenArgs:
+    search_space_digest: SearchSpaceDigest
+    objective_weights: np.ndarray
+    outcome_constraints: Optional[Tuple[np.ndarray, np.ndarray]]
+    linear_constraints: Optional[Tuple[np.ndarray, np.ndarray]]
+    fixed_features: Optional[Dict[int, float]]
+    pending_observations: Optional[List[np.ndarray]]
+    rounding_func: Callable[[np.ndarray], np.ndarray]
+    extra_model_gen_kwargs: Dict[str, Any]
+
+
 # pyre-fixme[13]: Attribute `model` is never initialized.
 # pyre-fixme[13]: Attribute `outcomes` is never initialized.
 # pyre-fixme[13]: Attribute `parameters` is never initialized.
 class ArrayModelBridge(ModelBridge):
     """A model bridge for using array-based models.
 
-    Requires that all non-task parameters have been transformed to
-    RangeParameters with float type and no log scale. Task parameters must be
-    transformed to RangeParameters with int type.
+    Requires that all non-task parameters have been transformed to RangeParameters.
+
+    If there are any (non-task) discrete parameters (e.g. as obtained via a
+    ChoiceEncode transform), those need to be of integer type with parameter
+    space normalized to `{0, 1, ..., num_choices-1}`. The `num_choices` information
+    is passed to the model and optimization needs to take this into account and return
+    only candidates that take values in this parameter space (specifically, there is
+    no relaxation and no rounding is applied).
+
+    All other parameters need to be of float type on a regular (non-log) scale.
 
     This will convert all parameter types to float and put data into arrays.
     """
@@ -70,22 +92,18 @@ class ArrayModelBridge(ModelBridge):
             outcomes=self.outcomes,
             parameters=self.parameters,
         )
-        # Extract bounds and task features
-        bounds, task_features, target_fidelities = get_bounds_and_task(
+        # Get all relevant information on the parameters
+        search_space_digest = extract_search_space_digest(
             search_space=search_space, param_names=self.parameters
         )
-
         # Fit
         self._model_fit(
             model=model,
             Xs=Xs_array,
             Ys=Ys_array,
             Yvars=Yvars_array,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=self.parameters,
+            search_space_digest=search_space_digest,
             metric_names=self.outcomes,
-            fidelity_features=list(target_fidelities.keys()),
             candidate_metadata=candidate_metadata,
         )
 
@@ -95,11 +113,8 @@ class ArrayModelBridge(ModelBridge):
         Xs: List[np.ndarray],
         Ys: List[np.ndarray],
         Yvars: List[np.ndarray],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
         candidate_metadata: Optional[List[List[TCandidateMetadata]]],
     ) -> None:
         """Fit the model, given numpy types."""
@@ -108,11 +123,8 @@ class ArrayModelBridge(ModelBridge):
             Xs=Xs,
             Ys=Ys,
             Yvars=Yvars,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=feature_names,
+            search_space_digest=search_space_digest,
             metric_names=metric_names,
-            fidelity_features=fidelity_features,
             candidate_metadata=candidate_metadata,
         )
 
@@ -129,23 +141,17 @@ class ArrayModelBridge(ModelBridge):
             outcomes=self.outcomes,
             parameters=self.parameters,
         )
-
-        bounds, task_features, target_fidelities = get_bounds_and_task(
+        search_space_digest = extract_search_space_digest(
             search_space=search_space, param_names=self.parameters
         )
-
         # Update in-design status for these new points.
         self._model_update(
             Xs=Xs_array,
             Ys=Ys_array,
             Yvars=Yvars_array,
-            candidate_metadata=candidate_metadata,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=self.parameters,
+            search_space_digest=search_space_digest,
             metric_names=self.outcomes,
-            fidelity_features=list(target_fidelities.keys()),
-            target_fidelities=target_fidelities,
+            candidate_metadata=candidate_metadata,
         )
 
     def _model_update(
@@ -153,26 +159,17 @@ class ArrayModelBridge(ModelBridge):
         Xs: List[np.ndarray],
         Ys: List[np.ndarray],
         Yvars: List[np.ndarray],
-        candidate_metadata: Optional[List[List[TCandidateMetadata]]],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
-        target_fidelities: Optional[Dict[int, float]],
+        candidate_metadata: Optional[List[List[TCandidateMetadata]]],
     ) -> None:
         self.model.update(
             Xs=Xs,
             Ys=Ys,
             Yvars=Yvars,
-            candidate_metadata=candidate_metadata,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=self.parameters,
+            search_space_digest=search_space_digest,
             metric_names=self.outcomes,
-            fidelity_features=list(target_fidelities.keys())
-            if target_fidelities
-            else None,
+            candidate_metadata=candidate_metadata,
         )
 
     def _predict(
@@ -193,34 +190,21 @@ class ArrayModelBridge(ModelBridge):
     ) -> Dict[str, Any]:
         return {}
 
-    def _gen(
+    def _get_transformed_model_gen_args(
         self,
-        n: int,
         search_space: SearchSpace,
         pending_observations: Dict[str, List[ObservationFeatures]],
         fixed_features: ObservationFeatures,
         model_gen_options: Optional[TConfig] = None,
         optimization_config: Optional[OptimizationConfig] = None,
-    ) -> Tuple[
-        List[ObservationFeatures],
-        List[float],
-        Optional[ObservationFeatures],
-        TGenMetadata,
-    ]:
-        """Generate new candidates according to search_space and
-        optimization_config.
-
-        The outcome constraints should be transformed to no longer be relative.
-        """
+    ) -> ArrayModelGenArgs:
         # Validation
         if not self.parameters:  # pragma: no cover
             raise ValueError(FIT_MODEL_ERROR.format(action="_gen"))
-        # Extract bounds
-        bounds, _, target_fidelities = get_bounds_and_task(
+        # Extract search space info
+        search_space_digest = extract_search_space_digest(
             search_space=search_space, param_names=self.parameters
         )
-        target_fidelities = {i: float(v) for i, v in target_fidelities.items()}
-
         if optimization_config is None:
             raise ValueError(
                 "ArrayModelBridge requires an OptimizationConfig to be specified"
@@ -246,32 +230,72 @@ class ArrayModelBridge(ModelBridge):
         pending_array = pending_observations_as_array(
             pending_observations, self.outcomes, self.parameters
         )
-        # Generate the candidates
-        X, w, gen_metadata, candidate_metadata = self._model_gen(
-            n=n,
-            bounds=bounds,
+        return ArrayModelGenArgs(
+            search_space_digest=search_space_digest,
             objective_weights=objective_weights,
             outcome_constraints=outcome_constraints,
             linear_constraints=linear_constraints,
             fixed_features=fixed_features_dict,
             pending_observations=pending_array,
-            model_gen_options=model_gen_options,
             rounding_func=transform_callback(self.parameters, self.transforms),
-            target_fidelities=target_fidelities,
-            **extra_model_gen_kwargs,
+            extra_model_gen_kwargs=extra_model_gen_kwargs,
+        )
+
+    def _gen(
+        self,
+        n: int,
+        search_space: SearchSpace,
+        pending_observations: Dict[str, List[ObservationFeatures]],
+        fixed_features: ObservationFeatures,
+        model_gen_options: Optional[TConfig] = None,
+        optimization_config: Optional[OptimizationConfig] = None,
+    ) -> Tuple[
+        List[ObservationFeatures],
+        List[float],
+        Optional[ObservationFeatures],
+        TGenMetadata,
+    ]:
+        """Generate new candidates according to search_space and
+        optimization_config.
+
+        The outcome constraints should be transformed to no longer be relative.
+        """
+        array_model_gen_args = self._get_transformed_model_gen_args(
+            search_space=search_space,
+            pending_observations=pending_observations,
+            fixed_features=fixed_features,
+            model_gen_options=model_gen_options,
+            optimization_config=optimization_config,
+        )
+
+        # Generate the candidates
+        search_space_digest = array_model_gen_args.search_space_digest
+        # TODO: pass array_model_gen_args to _model_gen
+        X, w, gen_metadata, candidate_metadata = self._model_gen(
+            n=n,
+            bounds=search_space_digest.bounds,
+            objective_weights=array_model_gen_args.objective_weights,
+            outcome_constraints=array_model_gen_args.outcome_constraints,
+            linear_constraints=array_model_gen_args.linear_constraints,
+            fixed_features=array_model_gen_args.fixed_features,
+            pending_observations=array_model_gen_args.pending_observations,
+            model_gen_options=model_gen_options,
+            rounding_func=array_model_gen_args.rounding_func,
+            target_fidelities=search_space_digest.target_fidelities,
+            **array_model_gen_args.extra_model_gen_kwargs,
         )
         # Transform array to observations
         observation_features = parse_observation_features(
             X=X, param_names=self.parameters, candidate_metadata=candidate_metadata
         )
         xbest = self._model_best_point(
-            bounds=bounds,
-            objective_weights=objective_weights,
-            outcome_constraints=outcome_constraints,
-            linear_constraints=linear_constraints,
-            fixed_features=fixed_features_dict,
+            bounds=search_space_digest.bounds,
+            objective_weights=array_model_gen_args.objective_weights,
+            outcome_constraints=array_model_gen_args.outcome_constraints,
+            linear_constraints=array_model_gen_args.linear_constraints,
+            fixed_features=array_model_gen_args.fixed_features,
             model_gen_options=model_gen_options,
-            target_fidelities=target_fidelities,
+            target_fidelities=search_space_digest.target_fidelities,
         )
         best_obsf = (
             None
@@ -355,10 +379,9 @@ class ArrayModelBridge(ModelBridge):
             outcomes=self.outcomes,
             parameters=self.parameters,
         )
-        bounds, task_features, target_fidelities = get_bounds_and_task(
+        search_space_digest = extract_search_space_digest(
             search_space=search_space, param_names=self.parameters
         )
-
         X_test = np.array(
             [[obsf.parameters[p] for p in self.parameters] for obsf in cv_test_points]
         )
@@ -368,11 +391,8 @@ class ArrayModelBridge(ModelBridge):
             Ys_train=Ys_train,
             Yvars_train=Yvars_train,
             X_test=X_test,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=self.parameters,
+            search_space_digest=search_space_digest,
             metric_names=self.outcomes,
-            fidelity_features=list(target_fidelities.keys()),
         )
         # Convert array back to ObservationData
         return array_to_observation_data(f=f_test, cov=cov_test, outcomes=self.outcomes)
@@ -383,22 +403,16 @@ class ArrayModelBridge(ModelBridge):
         Ys_train: List[np.ndarray],
         Yvars_train: List[np.ndarray],
         X_test: np.ndarray,
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
     ) -> Tuple[np.ndarray, np.ndarray]:  # pragma: no cover
         return self.model.cross_validate(
             Xs_train=Xs_train,
             Ys_train=Ys_train,
             Yvars_train=Yvars_train,
             X_test=X_test,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=feature_names,
+            search_space_digest=search_space_digest,
             metric_names=metric_names,
-            fidelity_features=fidelity_features,
         )
 
     def _evaluate_acquisition_function(
@@ -456,7 +470,9 @@ class ArrayModelBridge(ModelBridge):
             - an (n x m x m) array of covariances
         """
         try:
-            return observation_data_to_array(observation_data=observation_data)
+            return observation_data_to_array(
+                outcomes=self.outcomes, observation_data=observation_data
+            )
         except (KeyError, TypeError):  # pragma: no cover
             raise ValueError("Invalid formatting of observation data.")
 
@@ -547,7 +563,14 @@ def validate_optimization_config(
     for c in optimization_config.outcome_constraints:
         if c.relative:
             raise ValueError(f"{c} is a relative constraint.")
-        if c.metric.name not in outcomes:  # pragma: no cover
+        if isinstance(c, ScalarizedOutcomeConstraint):
+            for c_metric in c.metrics:
+                if c_metric.name not in outcomes:  # pragma: no cover
+                    raise ValueError(
+                        f"Scalarized constraint metric component {c.metric.name} "
+                        + "not found in fitted data."
+                    )
+        elif c.metric.name not in outcomes:  # pragma: no cover
             raise ValueError(
                 f"Outcome constraint metric {c.metric.name} not found in fitted data."
             )
