@@ -8,6 +8,7 @@ import time
 from abc import ABC
 from collections import OrderedDict
 from copy import deepcopy
+from dataclasses import dataclass
 from typing import Any, Dict, List, MutableMapping, Optional, Set, Tuple, Type
 
 from ax.core.arm import Arm
@@ -43,6 +44,14 @@ from ax.utils.common.typeutils import not_none
 logger = get_logger(__name__)
 
 
+@dataclass
+class BaseGenArgs:
+    search_space: SearchSpace
+    optimization_config: OptimizationConfig
+    pending_observations: Dict[str, List[ObservationFeatures]]
+    fixed_features: ObservationFeatures
+
+
 class ModelBridge(ABC):
     """The main object for using models in Ax.
 
@@ -80,6 +89,7 @@ class ModelBridge(ABC):
         status_quo_features: Optional[ObservationFeatures] = None,
         optimization_config: Optional[OptimizationConfig] = None,
         fit_out_of_design: bool = False,
+        fit_abandoned: bool = False,
     ) -> None:
         """
         Applies transforms and fits model.
@@ -105,6 +115,9 @@ class ModelBridge(ABC):
                 the model.
             fit_out_of_design: If specified, all training data is returned.
                 Otherwise, only in design points are returned.
+            fit_abandoned: Whether data for abandoned arms or trials should be
+                included in model training data. If ``False``, only
+                non-abandoned points are returned.
         """
         t_fit_start = time.time()
         transforms = transforms or []
@@ -126,6 +139,7 @@ class ModelBridge(ABC):
         self._raw_transforms = transforms
         self._transform_configs: Optional[Dict[str, TConfig]] = transform_configs
         self._fit_out_of_design = fit_out_of_design
+        self._fit_abandoned = fit_abandoned
         imm = experiment and experiment.immutable_search_space_and_opt_config
         self._experiment_has_immutable_search_space_and_opt_config = imm
         if experiment is not None:
@@ -134,7 +148,11 @@ class ModelBridge(ABC):
             self._arms_by_signature = experiment.arms_by_signature
 
         observations = (
-            observations_from_data(experiment, data)
+            observations_from_data(
+                experiment=experiment,
+                data=data,
+                include_abandoned=self._fit_abandoned,
+            )
             if experiment is not None and data is not None
             else []
         )
@@ -524,7 +542,11 @@ class ModelBridge(ABC):
         """
         t_update_start = time.time()
         observations = (
-            observations_from_data(experiment=experiment, data=new_data)
+            observations_from_data(
+                experiment=experiment,
+                data=new_data,
+                include_abandoned=self._fit_abandoned,
+            )
             if experiment is not None and new_data is not None
             else []
         )
@@ -563,40 +585,18 @@ class ModelBridge(ABC):
         """
         raise NotImplementedError  # pragma: no cover
 
-    def gen(
+    def _get_transformed_gen_args(
         self,
-        n: int,
-        search_space: Optional[SearchSpace] = None,
+        search_space: SearchSpace,
         optimization_config: Optional[OptimizationConfig] = None,
         pending_observations: Optional[Dict[str, List[ObservationFeatures]]] = None,
         fixed_features: Optional[ObservationFeatures] = None,
-        model_gen_options: Optional[TConfig] = None,
-    ) -> GeneratorRun:
-        """
-        Args:
-            n: Number of points to generate
-            search_space: Search space
-            optimization_config: Optimization config
-            pending_observations: A map from metric name to pending
-                observations for that metric.
-            fixed_features: An ObservationFeatures object containing any
-                features that should be fixed at specified values during
-                generation.
-            model_gen_options: A config dictionary that is passed along to the
-                model.
-        """
-        t_gen_start = time.time()
+    ) -> BaseGenArgs:
         if pending_observations is None:
             pending_observations = {}
         if fixed_features is None:
             fixed_features = ObservationFeatures({})
 
-        # Get modifiable versions
-        if search_space is None:
-            search_space = self._model_space
-        orig_search_space = search_space
-        search_space = search_space.clone()
-
         if optimization_config is None:
             optimization_config = (
                 # pyre-fixme[16]: `Optional` has no attribute `clone`.
@@ -623,17 +623,57 @@ class ModelBridge(ABC):
             for metric, po in pending_observations.items():
                 pending_observations[metric] = t.transform_observation_features(po)
             fixed_features = t.transform_observation_features([fixed_features])[0]
-
-        # Apply terminal transform and gen
-        observation_features, weights, best_obsf, gen_metadata = self._gen(
-            n=n,
+        return BaseGenArgs(
             search_space=search_space,
             optimization_config=optimization_config,
             pending_observations=pending_observations,
             fixed_features=fixed_features,
-            model_gen_options=model_gen_options,
         )
 
+    def gen(
+        self,
+        n: int,
+        search_space: Optional[SearchSpace] = None,
+        optimization_config: Optional[OptimizationConfig] = None,
+        pending_observations: Optional[Dict[str, List[ObservationFeatures]]] = None,
+        fixed_features: Optional[ObservationFeatures] = None,
+        model_gen_options: Optional[TConfig] = None,
+    ) -> GeneratorRun:
+        """
+        Args:
+            n: Number of points to generate
+            search_space: Search space
+            optimization_config: Optimization config
+            pending_observations: A map from metric name to pending
+                observations for that metric.
+            fixed_features: An ObservationFeatures object containing any
+                features that should be fixed at specified values during
+                generation.
+            model_gen_options: A config dictionary that is passed along to the
+                model.
+        """
+        t_gen_start = time.time()
+        # Get modifiable versions
+        if search_space is None:
+            search_space = self._model_space
+        orig_search_space = search_space
+        search_space = search_space.clone()
+        base_gen_args = self._get_transformed_gen_args(
+            search_space=search_space,
+            optimization_config=optimization_config,
+            pending_observations=pending_observations,
+            fixed_features=fixed_features,
+        )
+
+        # Apply terminal transform and gen
+        observation_features, weights, best_obsf, gen_metadata = self._gen(
+            n=n,
+            search_space=base_gen_args.search_space,
+            optimization_config=base_gen_args.optimization_config,
+            pending_observations=base_gen_args.pending_observations,
+            fixed_features=base_gen_args.fixed_features,
+            model_gen_options=model_gen_options,
+        )
         # Apply reverse transforms
         for t in reversed(self.transforms.values()):  # noqa T484
             observation_features = t.untransform_observation_features(
@@ -642,13 +682,16 @@ class ModelBridge(ABC):
             if best_obsf is not None:
                 best_obsf = t.untransform_observation_features([best_obsf])[0]
 
-        # Clamp the untransformed data to the original search space
-        observation_features = clamp_observation_features(
-            observation_features, orig_search_space
-        )
-        if best_obsf is not None:
-            best_obsf = clamp_observation_features([best_obsf], orig_search_space)[0]
-
+        # Clamp the untransformed data to the original search space if
+        # we don't fit/gen OOD points
+        if not self._fit_out_of_design:
+            observation_features = clamp_observation_features(
+                observation_features, orig_search_space
+            )
+            if best_obsf is not None:
+                best_obsf = clamp_observation_features([best_obsf], orig_search_space)[
+                    0
+                ]
         best_point_predictions = None
         try:
             model_predictions = self.predict(observation_features)
@@ -677,11 +720,12 @@ class ModelBridge(ABC):
         immutable = getattr(
             self, "_experiment_has_immutable_search_space_and_opt_config", False
         )
+        optimization_config = None if immutable else base_gen_args.optimization_config
         gr = GeneratorRun(
             arms=arms,
             weights=weights,
-            optimization_config=None if immutable else optimization_config,
-            search_space=None if immutable else search_space,
+            optimization_config=optimization_config,
+            search_space=None if immutable else base_gen_args.search_space,
             model_predictions=model_predictions,
             best_arm_predictions=None
             if best_arm is None
