@@ -21,6 +21,8 @@ from ax.modelbridge.multi_objective_torch import MultiObjectiveTorchModelBridge
 from ax.modelbridge.random import RandomModelBridge
 from ax.modelbridge.torch import TorchModelBridge
 from ax.modelbridge.transforms.base import Transform
+from ax.modelbridge.transforms.centered_unit_x import CenteredUnitX
+from ax.modelbridge.transforms.choice_encode import OrderedChoiceEncode, ChoiceEncode
 from ax.modelbridge.transforms.convert_metric_names import ConvertMetricNames
 from ax.modelbridge.transforms.derelativize import Derelativize
 from ax.modelbridge.transforms.int_range_to_choice import IntRangeToChoice
@@ -28,7 +30,6 @@ from ax.modelbridge.transforms.int_to_float import IntToFloat
 from ax.modelbridge.transforms.ivw import IVW
 from ax.modelbridge.transforms.log import Log
 from ax.modelbridge.transforms.one_hot import OneHot
-from ax.modelbridge.transforms.ordered_choice_encode import OrderedChoiceEncode
 from ax.modelbridge.transforms.remove_fixed import RemoveFixed
 from ax.modelbridge.transforms.search_space_to_choice import SearchSpaceToChoice
 from ax.modelbridge.transforms.standardize_y import StandardizeY
@@ -42,6 +43,7 @@ from ax.models.discrete.full_factorial import FullFactorialGenerator
 from ax.models.discrete.thompson import ThompsonSampler
 from ax.models.random.sobol import SobolGenerator
 from ax.models.random.uniform import UniformGenerator
+from ax.models.torch.alebo import ALEBO
 from ax.models.torch.botorch import BotorchModel
 from ax.models.torch.botorch_kg import KnowledgeGradient
 from ax.models.torch.botorch_mes import MaxValueEntropySearch
@@ -82,6 +84,14 @@ Cont_X_trans: List[Type[Transform]] = [
 
 Discrete_X_trans: List[Type[Transform]] = [IntRangeToChoice]
 
+Mixed_transforms: List[Type[Transform]] = [
+    RemoveFixed,
+    ChoiceEncode,
+    IntToFloat,
+    Log,
+    UnitX,
+]
+
 Y_trans: List[Type[Transform]] = [IVW, Derelativize, StandardizeY]
 
 # Expected `List[Type[Transform]]` for 2nd anonymous parameter to
@@ -112,6 +122,9 @@ Specified_Task_ST_MTGP_trans: List[Type[Transform]] = Cont_X_trans + [
     TaskEncode,
 ]
 
+ALEBO_X_trans: List[Type[Transform]] = [RemoveFixed, IntToFloat, CenteredUnitX]
+ALEBO_Y_trans: List[Type[Transform]] = [Derelativize, StandardizeY]
+
 STANDARD_TORCH_BRIDGE_KWARGS: Dict[str, Any] = {"torch_dtype": torch.double}
 
 
@@ -195,6 +208,29 @@ MODEL_KEY_TO_MODEL_SETUP: Dict[str, ModelSetup] = {
         transforms=Cont_X_trans + Y_trans,
         standard_bridge_kwargs=STANDARD_TORCH_BRIDGE_KWARGS,
     ),
+    "MOO_Modular": ModelSetup(
+        bridge_class=MultiObjectiveTorchModelBridge,
+        model_class=ModularBoTorchModel,
+        transforms=Cont_X_trans + Y_trans,
+        standard_bridge_kwargs=STANDARD_TORCH_BRIDGE_KWARGS,
+    ),
+    "ST_MTGP": ModelSetup(
+        bridge_class=TorchModelBridge,
+        model_class=BotorchModel,
+        transforms=ST_MTGP_trans,
+        standard_bridge_kwargs=STANDARD_TORCH_BRIDGE_KWARGS,
+    ),
+    "ALEBO": ModelSetup(
+        bridge_class=TorchModelBridge,
+        model_class=ALEBO,
+        transforms=ALEBO_X_trans + ALEBO_Y_trans,
+    ),
+    "BO_MIXED": ModelSetup(
+        bridge_class=TorchModelBridge,
+        model_class=ModularBoTorchModel,
+        transforms=Mixed_transforms + Y_trans,
+        standard_bridge_kwargs=STANDARD_TORCH_BRIDGE_KWARGS,
+    ),
 }
 
 
@@ -360,6 +396,10 @@ class Models(ModelRegistryBase):
     EMPIRICAL_BAYES_THOMPSON = "EB"
     UNIFORM = "Uniform"
     MOO = "MOO"
+    MOO_MODULAR = "MOO_Modular"
+    ST_MTGP = "ST_MTGP"
+    ALEBO = "ALEBO"
+    BO_MIXED = "BO_MIXED"
 
 
 def get_model_from_generator_run(
