@@ -6,6 +6,11 @@
 
 from collections import OrderedDict
 
+import numpy as np
+import pandas as pd
+import torch
+from ax.core.data import Data
+from ax.core.observation import ObservationFeatures
 from ax.modelbridge.discrete import DiscreteModelBridge
 from ax.modelbridge.random import RandomModelBridge
 from ax.modelbridge.registry import (
@@ -19,6 +24,7 @@ from ax.modelbridge.torch import TorchModelBridge
 from ax.models.base import Model
 from ax.models.discrete.eb_thompson import EmpiricalBayesThompsonSampler
 from ax.models.discrete.thompson import ThompsonSampler
+from ax.models.torch.alebo import ALEBO
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.model import BoTorchModel
 from ax.models.torch.botorch_modular.surrogate import Surrogate
@@ -198,6 +204,7 @@ class ModelRegistryTest(TestCase):
                     "status_quo_name": None,
                     "status_quo_features": None,
                     "fit_out_of_design": False,
+                    "fit_abandoned": False,
                 },
             ),
         )
@@ -218,6 +225,7 @@ class ModelRegistryTest(TestCase):
                     "status_quo_name",
                     "status_quo_features",
                     "fit_out_of_design",
+                    "fit_abandoned",
                 ]
             ),
         )
@@ -289,3 +297,68 @@ class ModelRegistryTest(TestCase):
             bridge_args = set(get_function_argument_names(bridge_class))
             # Intersection of two sets should be empty
             self.assertEqual(model_args & bridge_args, set())
+
+    def test_ST_MTGP(self):
+        """Tests single type MTGP instantiation."""
+        # Test Single-type MTGP
+        exp = get_branin_experiment()
+        sobol = Models.SOBOL(search_space=exp.search_space)
+        self.assertIsInstance(sobol, RandomModelBridge)
+        for _ in range(5):
+            sobol_run = sobol.gen(n=1)
+            t = exp.new_batch_trial().add_generator_run(sobol_run)
+            t.set_status_quo_with_weight(status_quo=t.arms[0], weight=0.5)
+            t.run().mark_completed()
+        status_quo_features = ObservationFeatures(
+            parameters=exp.trials[0].status_quo.parameters,
+            trial_index=0,
+        )
+        mtgp = Models.ST_MTGP(
+            experiment=exp,
+            data=exp.fetch_data(),
+            status_quo_features=status_quo_features,
+        )
+        self.assertIsInstance(mtgp, TorchModelBridge)
+
+        exp = get_branin_experiment()
+        sobol = Models.SOBOL(search_space=exp.search_space)
+        self.assertIsInstance(sobol, RandomModelBridge)
+        sobol_run = sobol.gen(n=1)
+        t = exp.new_batch_trial().add_generator_run(sobol_run)
+        t.set_status_quo_with_weight(status_quo=t.arms[0], weight=0.5)
+        t.run().mark_completed()
+
+        with self.assertRaises(ValueError):
+            status_quo_features = ObservationFeatures(
+                parameters=exp.trials[0].status_quo.parameters,
+                trial_index=0,
+            )
+            Models.ST_MTGP(
+                experiment=exp,
+                data=exp.fetch_data(),
+                status_quo_features=status_quo_features,
+            )
+
+    def test_ALEBO(self):
+        """Tests Alebo fitting and generations"""
+        experiment = get_branin_experiment(with_batch=True)
+        B = np.array([[1.0, 2.0]])
+        data = Data(
+            pd.DataFrame(
+                {
+                    "arm_name": ["0_0", "0_1", "0_2"],
+                    "metric_name": "y",
+                    "mean": [-1.0, 0.0, 1.0],
+                    "sem": 0.1,
+                }
+            )
+        )
+        m = Models.ALEBO(
+            experiment=experiment,
+            search_space=None,
+            data=data,
+            B=torch.from_numpy(B).float(),
+        )
+        self.assertIsInstance(m, TorchModelBridge)
+        self.assertIsInstance(m.model, ALEBO)
+        self.assertTrue(np.array_equal(m.model.B.numpy(), B))
