@@ -6,8 +6,10 @@
 
 from typing import TYPE_CHECKING, Optional
 
+import numpy as np
 from ax.core.observation import ObservationFeatures
 from ax.core.optimization_config import OptimizationConfig
+from ax.core.outcome_constraint import ScalarizedOutcomeConstraint
 from ax.modelbridge.base import unwrap_observation_data
 from ax.modelbridge.transforms.base import Transform
 from ax.modelbridge.transforms.ivw import ivw_metric_merge
@@ -25,6 +27,11 @@ class Derelativize(Transform):
     If status quo is in-design, uses model estimate at status quo. If not, uses
     raw observation at status quo.
 
+    Will raise an error if status quo is in-design and model fails to predict
+    for it, unless the flag "use_raw_status_quo" is set to True in the
+    transform config, in which case it will fall back to using the observed
+    value in the training data.
+
     Transform is done in-place.
     """
 
@@ -34,6 +41,7 @@ class Derelativize(Transform):
         modelbridge: Optional["modelbridge_module.base.ModelBridge"],
         fixed_features: ObservationFeatures,
     ) -> OptimizationConfig:
+        use_raw_sq = self.config.get("use_raw_status_quo", False)
         has_relative_constraint = any(
             c.relative for c in optimization_config.all_constraints
         )
@@ -53,7 +61,7 @@ class Derelativize(Transform):
             f, _ = modelbridge.predict([modelbridge.status_quo.features])
         except Exception:
             # Check if it is out-of-design.
-            if not modelbridge.model_space.check_membership(
+            if use_raw_sq or not modelbridge.model_space.check_membership(
                 modelbridge.status_quo.features.parameters
             ):
                 # Out-of-design: use the raw observation
@@ -70,7 +78,17 @@ class Derelativize(Transform):
         # Plug in the status quo value to each relative constraint.
         for c in optimization_config.all_constraints:
             if c.relative:
-                # Compute new bound.
-                c.bound = (1 + c.bound / 100.0) * f[c.metric.name][0]
+                if isinstance(c, ScalarizedOutcomeConstraint):
+                    # The sq_val of scalarized outcome is the weighted
+                    # sum of its component metrics
+                    sq_val = np.sum(
+                        [
+                            c.weights[i] * f[metric.name][0]
+                            for i, metric in enumerate(c.metrics)
+                        ]
+                    )
+                else:
+                    sq_val = f[c.metric.name][0]
+                c.bound = (1 + c.bound / 100.0) * sq_val
                 c.relative = False
         return optimization_config
