@@ -6,11 +6,14 @@
 
 from __future__ import annotations
 
+import itertools
+import warnings
 from collections import defaultdict
 from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Set, Tuple, Union
 
 import numpy as np
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.core.types import TConfig, TParamCounter
 from ax.exceptions.core import SearchSpaceExhausted
 from ax.models.numpy_base import NumpyModel
@@ -364,17 +367,9 @@ def best_in_sample_point(
     # Parse options
     if options is None:
         options = {}
-    # pyre-fixme[9]: method has type `str`; used as `Union[AcquisitionFunction,
-    #  float, int, str]`.
     method: str = options.get("best_point_method", "max_utility")
-    # pyre-fixme[9]: B has type `Optional[float]`; used as
-    #  `Optional[Union[AcquisitionFunction, float, int, str]]`.
     B: Optional[float] = options.get("utility_baseline", None)
-    # pyre-fixme[9]: threshold has type `float`; used as `Union[AcquisitionFunction,
-    #  float, int, str]`.
     threshold: float = options.get("probability_threshold", 0.95)
-    # pyre-fixme[9]: nsamp has type `int`; used as `Union[AcquisitionFunction,
-    #  float, int, str]`.
     nsamp: int = options.get("feasibility_mc_samples", 10000)
     # Get points observed for all objective and constraint outcomes
     if objective_weights is None:
@@ -418,6 +413,7 @@ def best_in_sample_point(
         if B is None:
             B = obj.min()
         utility = (obj - B) * pfeas
+    # pyre-fixme[61]: `utility` may not be initialized here.
     i = np.argmax(utility)
     if utility[i] == -np.Inf:
         return None
@@ -539,3 +535,34 @@ def filter_constraints_and_fixed_features(
         return torch.from_numpy(X_feas).to(device=X.device, dtype=X.dtype)
     else:
         return X_feas
+
+
+def mk_discrete_choices(
+    ssd: SearchSpaceDigest,
+    fixed_features: Optional[Dict[int, float]] = None,
+) -> Dict[int, List[Union[int, float]]]:
+    discrete_choices = ssd.discrete_choices
+    # Add in fixed features.
+    if fixed_features is not None:
+        # Note: if any discrete features are fixed we won't enumerate those.
+        discrete_choices = {
+            **discrete_choices,
+            **{k: [v] for k, v in fixed_features.items()},
+        }
+    return discrete_choices
+
+
+def enumerate_discrete_combinations(
+    discrete_choices: Dict[int, List[Union[int, float]]],
+) -> List[Dict[int, Union[float, int]]]:
+    n_combos = np.prod([len(v) for v in discrete_choices.values()])
+    if n_combos > 50:
+        warnings.warn(
+            f"Enumerating {n_combos} combinations of discrete parameter values "
+            "while optimizing over a mixed search space. This can be very slow."
+        )
+    fixed_features_list = [
+        dict(zip(discrete_choices.keys(), c))
+        for c in itertools.product(*discrete_choices.values())
+    ]
+    return fixed_features_list
