@@ -7,6 +7,7 @@
 from unittest import mock
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.models.torch.botorch_mes import MaxValueEntropySearch, _instantiate_MES
 from ax.utils.common.testutils import TestCase
 from botorch.acquisition.max_value_entropy_search import (
@@ -54,11 +55,11 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+            ),
             metric_names=self.metric_names,
-            task_features=[],
-            fidelity_features=[],
         )
 
         # test model.gen()
@@ -139,11 +140,11 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+            ),
             metric_names=self.metric_names,
-            task_features=[],
-            fidelity_features=[],
         )
         self.assertTrue(model.use_input_warping)
         self.assertTrue(hasattr(model.model, "input_transform"))
@@ -156,11 +157,11 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+            ),
             metric_names=self.metric_names,
-            task_features=[],
-            fidelity_features=[],
         )
         self.assertTrue(model.use_loocv_pseudo_likelihood)
 
@@ -170,11 +171,12 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            task_features=[],
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                fidelity_features=[-1],
+            ),
             metric_names=self.metric_names,
-            fidelity_features=[-1],
         )
 
         # Check best point selection within bounds (some numerical tolerance)
@@ -232,11 +234,12 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            task_features=[],
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                fidelity_features=[-1],
+            ),
             metric_names=self.metric_names,
-            fidelity_features=[-1],
         )
         self.assertTrue(model.use_input_warping)
         self.assertTrue(hasattr(model.model, "input_transform"))
@@ -249,11 +252,12 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            task_features=[],
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                fidelity_features=[-1],
+            ),
             metric_names=self.metric_names,
-            fidelity_features=[-1],
         )
         self.assertTrue(model.use_loocv_pseudo_likelihood)
 
@@ -264,11 +268,11 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+            ),
             metric_names=self.metric_names,
-            task_features=[],
-            fidelity_features=[],
         )
 
         # test acquisition setting
@@ -295,11 +299,12 @@ class MaxValueEntropySearchTest(TestCase):
             Xs=self.Xs,
             Ys=self.Ys,
             Yvars=self.Yvars,
-            bounds=self.bounds,
-            task_features=[],
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                fidelity_features=[-1],
+            ),
             metric_names=self.metric_names,
-            fidelity_features=[-1],
         )
 
         candidate_set = torch.rand(10, 3, dtype=self.dtype, device=self.device)
