@@ -11,6 +11,7 @@ from typing import Any, Callable, Dict, List, MutableMapping, Optional, Tuple, U
 import gpytorch
 import numpy as np
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
 from ax.models.random.alebo_initializer import ALEBOInitializer
 from ax.models.torch.botorch import BotorchModel
@@ -263,6 +264,8 @@ def get_map_model(
     for _ in range(restarts):
         m = ALEBOGP(B=B, train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         if init_state_dict is not None:
+            # pyre-fixme[6]: Expected `OrderedDict[typing.Any, typing.Any]` for 1st
+            #  param but got `Dict[str, Tensor]`.
             m.load_state_dict(init_state_dict)
         mll = ExactMarginalLogLikelihood(m.likelihood, m)
         mll.train()
@@ -491,7 +494,7 @@ def alebo_acqf_optimizer(
     candidate_list, acq_value_list = [], []
     candidates = torch.tensor([], device=B.device, dtype=B.dtype)
     try:
-        base_X_pending = acq_function.X_pending  # pyre-ignore
+        base_X_pending = acq_function.X_pending
         acq_has_X_pend = True
     except AttributeError:
         base_X_pending = None
@@ -527,12 +530,17 @@ def alebo_acqf_optimizer(
             candidates = torch.cat(candidate_list, dim=-2)
             if acq_has_X_pend:
                 acq_function.set_X_pending(
+                    # pyre-fixme[6]: Expected `Union[List[Tensor],
+                    #  typing.Tuple[Tensor, ...]]` for 1st param but got
+                    #  `List[Union[Tensor, torch.nn.Module]]`.
                     torch.cat([base_X_pending, candidates], dim=-2)
                     if base_X_pending is not None
                     else candidates
                 )
         logger.info(f"Generated sequential candidate {i+1} of {n}")
     if acq_has_X_pend:
+        # pyre-fixme[6]: Expected `Optional[Tensor]` for 1st param but got
+        #  `Union[None, Tensor, torch.nn.Module]`.
         acq_function.set_X_pending(base_X_pending)
     return candidates, torch.stack(acq_value_list)
 
@@ -575,16 +583,13 @@ class ALEBO(BotorchModel):
         Xs: List[Tensor],
         Ys: List[Tensor],
         Yvars: List[Tensor],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
-        assert len(task_features) == 0
-        assert len(fidelity_features) == 0
-        for b in bounds:
+        assert len(search_space_digest.task_features) == 0
+        assert len(search_space_digest.fidelity_features) == 0
+        for b in search_space_digest.bounds:
             assert b == (-1, 1)
         # GP is fit in the low-d space, so project Xs down.
         self.Xs = [(self.B @ X.t()).t() for X in Xs]
