@@ -11,6 +11,7 @@ from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import numpy as np
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
 from ax.models.torch.botorch_defaults import (
     get_and_fit_model,
@@ -35,7 +36,6 @@ from botorch.models.model import Model
 from botorch.models.model_list_gp_regression import ModelListGP
 from torch import Tensor
 
-
 logger = get_logger(__name__)
 
 
@@ -70,6 +70,7 @@ TOptimizer = Callable[
         Tensor,
         int,
         Optional[List[Tuple[Tensor, Tensor, float]]],
+        Optional[List[Tuple[Tensor, Tensor, float]]],
         Optional[Dict[int, float]],
         Optional[Callable[[Tensor], Tensor]],
         Any,
@@ -183,6 +184,7 @@ class BotorchModel(TorchModel):
             bounds,
             n,
             inequality_constraints,
+            equality_constraints,
             fixed_features,
             rounding_func,
             **kwargs,
@@ -273,11 +275,8 @@ class BotorchModel(TorchModel):
         Xs: List[Tensor],
         Ys: List[Tensor],
         Yvars: List[Tensor],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
         self.dtype = Xs[0].dtype
@@ -285,9 +284,12 @@ class BotorchModel(TorchModel):
         self.Xs = Xs
         self.Ys = Ys
         self.Yvars = Yvars
-        # ensure indices are non-negative
-        self.task_features = normalize_indices(task_features, d=Xs[0].size(-1))
-        self.fidelity_features = normalize_indices(fidelity_features, d=Xs[0].size(-1))
+        self.task_features = normalize_indices(
+            search_space_digest.task_features, d=Xs[0].size(-1)
+        )
+        self.fidelity_features = normalize_indices(
+            search_space_digest.fidelity_features, d=Xs[0].size(-1)
+        )
         self.metric_names = metric_names
         self.model = self.model_constructor(  # pyre-ignore [28]
             Xs=Xs,
@@ -341,11 +343,14 @@ class BotorchModel(TorchModel):
 
         # subset model only to the outcomes we need for the optimization	357
         if options.get(Keys.SUBSET_MODEL, True):
-            model, objective_weights, outcome_constraints, _ = subset_model(
+            subset_model_results = subset_model(
                 model=model,  # pyre-ignore [6]
                 objective_weights=objective_weights,
                 outcome_constraints=outcome_constraints,
             )
+            model = subset_model_results.model
+            objective_weights = subset_model_results.objective_weights
+            outcome_constraints = subset_model_results.outcome_constraints
 
         bounds_ = torch.tensor(bounds, dtype=self.dtype, device=self.device)
         bounds_ = bounds_.transpose(0, 1)
