@@ -14,6 +14,7 @@ from ax.models.torch.utils import (  # noqa F401
     predict_from_model,
 )
 from ax.models.torch_base import TorchModel
+from ax.utils.common.constants import Keys
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.fixed_feature import FixedFeatureAcquisitionFunction
 from botorch.acquisition.objective import ConstrainedMCObjective, GenericMCObjective
@@ -163,6 +164,8 @@ def get_and_fit_model(
         model = ModelListGP(*models)
     model.to(Xs[0])
     if state_dict is not None:
+        # pyre-fixme[6]: Expected `OrderedDict[typing.Any, typing.Any]` for 1st
+        #  param but got `Dict[str, Tensor]`.
         model.load_state_dict(state_dict)
     if state_dict is None or refit_model:
         # TODO: Add bounds for optimization stability - requires revamp upstream
@@ -174,7 +177,6 @@ def get_and_fit_model(
         if isinstance(model, ModelListGP):
             mll = SumMarginalLogLikelihood(model.likelihood, model, mll_cls=mll_cls)
         else:
-            # pyre-ignore: [16]
             mll = mll_cls(model.likelihood, model)
         mll = fit_gpytorch_model(mll, bounds=bounds)
     return model
@@ -188,11 +190,33 @@ def get_NEI(
     X_pending: Optional[Tensor] = None,
     **kwargs: Any,
 ) -> AcquisitionFunction:
-    r"""Instantiates a qNoisyExpectedImprovement acquisition function.
+    r"""Instantiates a qNoisyExpectedImprovement acquisition function."""
+    return _get_acqusition_func(
+        model=model,
+        acquisition_function_name="qNEI",
+        objective_weights=objective_weights,
+        outcome_constraints=outcome_constraints,
+        X_observed=X_observed,
+        X_pending=X_pending,
+        **kwargs,
+    )
+
+
+def _get_acqusition_func(
+    model: Model,
+    acquisition_function_name: str,
+    objective_weights: Tensor,
+    outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+    X_observed: Optional[Tensor] = None,
+    X_pending: Optional[Tensor] = None,
+    **kwargs: Any,
+) -> AcquisitionFunction:
+    r"""Instantiates a acquisition function.
 
     Args:
         model: The underlying model which the acqusition function uses
             to estimate acquisition values of candidates.
+        acquisition_function_name: Name of the acquisition function.
         objective_weights: The objective is to maximize a weighted sum of
             the columns of f(x). These are the weights.
         outcome_constraints: A tuple of (A, b). For k outcome constraints
@@ -211,16 +235,15 @@ def get_NEI(
         chebyshev_scalarization: Use augmented Chebyshev scalarization.
 
     Returns:
-        qNoisyExpectedImprovement: The instantiated acquisition function.
+        The instantiated acquisition function.
     """
     if X_observed is None:
         raise ValueError("There are no feasible observed points.")
     # construct Objective module
     if kwargs.get("chebyshev_scalarization", False):
-        if "Ys" not in kwargs:
-            raise ValueError("Chebyshev Scalarization requires Ys argument")
-        Y_tensor = torch.cat(kwargs.get("Ys"), dim=-1)
-        obj_tf = get_chebyshev_scalarization(weights=objective_weights, Y=Y_tensor)
+        with torch.no_grad():
+            Y = model.posterior(X_observed).mean
+        obj_tf = get_chebyshev_scalarization(weights=objective_weights, Y=Y)
     else:
         obj_tf = get_objective_weights_transform(objective_weights)
 
@@ -236,7 +259,7 @@ def get_NEI(
             objective=objective, constraints=con_tfs or [], infeasible_cost=inf_cost
         )
     return get_acquisition_function(
-        acquisition_function_name="qNEI",
+        acquisition_function_name=acquisition_function_name,
         model=model,
         objective=objective,
         X_observed=X_observed,
@@ -247,6 +270,7 @@ def get_NEI(
         # pyre-fixme[6]: Expected `Optional[int]` for 9th param but got
         #  `Union[float, int]`.
         seed=torch.randint(1, 10000, (1,)).item(),
+        marginalize_dim=kwargs.get("marginalize_dim"),
     )
 
 
@@ -255,6 +279,7 @@ def scipy_optimizer(
     bounds: Tensor,
     n: int,
     inequality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
+    equality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
     fixed_features: Optional[Dict[int, float]] = None,
     rounding_func: Optional[Callable[[Tensor], Tensor]] = None,
     **kwargs: Any,
@@ -269,6 +294,9 @@ def scipy_optimizer(
         inequality constraints: A list of tuples (indices, coefficients, rhs),
             with each tuple encoding an inequality constraint of the form
             `\sum_i (X[indices[i]] * coefficients[i]) >= rhs`
+        equality constraints: A list of tuples (indices, coefficients, rhs),
+            with each tuple encoding an equality constraint of the form
+            `\sum_i (X[indices[i]] * coefficients[i]) == rhs`
         fixed_features: A map {feature_index: value} for features that should
             be fixed to a particular value during generation.
         rounding_func: A function that rounds an optimization result
@@ -284,16 +312,13 @@ def scipy_optimizer(
           values, where `i`-th element is the expected acquisition value
           conditional on having observed candidates `0,1,...,i-1`.
     """
-    num_restarts: int = kwargs.get("num_restarts", 20)
-    raw_samples: int = kwargs.get("num_raw_samples", 50 * num_restarts)
+    num_restarts: int = kwargs.pop(Keys.NUM_RESTARTS, 20)
+    raw_samples: int = kwargs.pop(Keys.RAW_SAMPLES, 50 * num_restarts)
 
     if kwargs.get("joint_optimization", False):
         sequential = False
     else:
         sequential = True
-        # use SLSQP by default for small problems since it yields faster wall times
-        if "method" not in kwargs:
-            kwargs["method"] = "SLSQP"
     X, expected_acquisition_value = optimize_acqf(
         acq_function=acq_function,
         bounds=bounds,
@@ -302,6 +327,7 @@ def scipy_optimizer(
         raw_samples=raw_samples,
         options=kwargs,
         inequality_constraints=inequality_constraints,
+        equality_constraints=equality_constraints,
         fixed_features=fixed_features,
         sequential=sequential,
         post_processing_func=rounding_func,
@@ -486,7 +512,7 @@ def _get_model(
     Returns:
         A GPyTorchModel (unfitted).
     """
-    Yvar = Yvar.clamp_min_(MIN_OBSERVED_NOISE_LEVEL)
+    Yvar = Yvar.clamp_min(MIN_OBSERVED_NOISE_LEVEL)  # pyre-ignore[16]
     is_nan = torch.isnan(Yvar)
     any_nan_Yvar = torch.any(is_nan)
     all_nan_Yvar = torch.all(is_nan)
@@ -503,6 +529,7 @@ def _get_model(
         warp_tf = get_warping_transform(
             d=X.shape[-1],
             task_feature=task_feature,
+            batch_shape=X.shape[:-2],  # pyre-ignore [6]
         )
     else:
         warp_tf = None
@@ -576,13 +603,15 @@ def _get_model(
 
 def get_warping_transform(
     d: int,
+    batch_shape: Optional[torch.Size] = None,
     task_feature: Optional[int] = None,
 ) -> Warp:
     """Construct input warping transform.
 
     Args:
         d: The dimension of the input, including task features
-        task_feature: the index of the task feature
+        batch_shape: The batch_shape of the model
+        task_feature: The index of the task feature
 
     Returns:
         The input warping transform.
@@ -597,5 +626,6 @@ def get_warping_transform(
         # prior with a median of 1
         concentration1_prior=LogNormalPrior(0.0, 0.75 ** 0.5),
         concentration0_prior=LogNormalPrior(0.0, 0.75 ** 0.5),
+        batch_shape=batch_shape,
     )
     return tf
