@@ -6,10 +6,12 @@
 
 from __future__ import annotations
 
+import dataclasses
 from logging import Logger
 from typing import Any, Dict, List, Optional, Tuple, Type
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.core.types import TCandidateMetadata, TConfig
 from ax.models.model_utils import best_in_sample_point
 from ax.models.torch.utils import (
@@ -32,7 +34,7 @@ from torch import Tensor
 
 
 NOT_YET_FIT_MSG = (
-    "Underlying BoTorch `Model` has not yet received its training_data."
+    "Underlying BoTorch `Model` has not yet received its training_data. "
     "Please fit the model first."
 )
 
@@ -69,7 +71,7 @@ class Surrogate(Base):
     kernel_class: Optional[Type[Kernel]] = None
     _training_data: Optional[TrainingData] = None
     _model: Optional[Model] = None
-    # Special setting for surrogates instantiated via `Surrogate.from_BoTorch`,
+    # Special setting for surrogates instantiated via `Surrogate.from_botorch`,
     # to avoid re-constructing the underlying BoTorch model on `Surrogate.fit`
     # when set to `False`.
     _constructed_manually: bool = False
@@ -125,7 +127,7 @@ class Surrogate(Base):
         return self.training_data.X.device
 
     @classmethod
-    def from_BoTorch(
+    def from_botorch(
         cls,
         model: Model,
         mll_class: Type[MarginalLogLikelihood] = ExactMarginalLogLikelihood,
@@ -148,7 +150,7 @@ class Surrogate(Base):
 
         Args:
             training_data: Training data for the model (for one outcome for
-                the default ``Surrogate``, with the exception of batched
+                the default `Surrogate`, with the exception of batched
                 multi-output case, where training data is formatted with just
                 one X and concatenated Ys).
             **kwargs: Optional keyword arguments, expects any of:
@@ -162,12 +164,11 @@ class Surrogate(Base):
                 "Base `Surrogate` expects training data for single outcome."
             )
 
-        kwargs = kwargs or {}
+        input_constructor_kwargs = {**self.model_options, **(kwargs or {})}
         self._training_data = training_data
 
         formatted_model_inputs = self.botorch_model_class.construct_inputs(
-            training_data=self.training_data,
-            fidelity_features=kwargs.get(Keys.FIDELITY_FEATURES),
+            training_data=self.training_data, **input_constructor_kwargs
         )
         # pyre-ignore[45]: Py raises informative msg if `model_cls` abstract.
         self._model = self.botorch_model_class(**formatted_model_inputs)
@@ -175,12 +176,8 @@ class Surrogate(Base):
     def fit(
         self,
         training_data: TrainingData,
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
-        target_fidelities: Optional[Dict[int, float]] = None,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
         state_dict: Optional[Dict[str, Tensor]] = None,
         refit: bool = True,
@@ -206,13 +203,9 @@ class Surrogate(Base):
             training data: BoTorch ``TrainingData`` container with Xs, Ys, and
                 possibly Yvars, to be passed to ``Model.construct_inputs`` in
                 BoTorch.
-            bounds: A list of d (lower, upper) tuples for each column of X.
-            task_features: Columns of X that take integer values and should be
-                treated as task parameters.
-            feature_names: Names of each column of X.
+            search_space_digest: A SearchSpaceDigest object containing
+                metadata on the features in the trainig data.
             metric_names: Names of each outcome Y in Ys.
-            fidelity_features: Columns of X that should be treated as fidelity
-                parameters.
             candidate_metadata: Model-produced metadata for candidates, in
                 the order corresponding to the Xs.
             state_dict: Optional state dict to load.
@@ -220,24 +213,22 @@ class Surrogate(Base):
         """
         if self._constructed_manually:
             logger.debug(
-                "For manually constructed surrogates (via `Surrogate.from_BoTorch`), "
+                "For manually constructed surrogates (via `Surrogate.from_botorch`), "
                 "`fit` skips setting the training data on model and only reoptimizes "
                 "its parameters if `refit=True`."
             )
-        if self._model is None and not self._constructed_manually:
+        else:
             self.construct(
                 training_data=training_data,
-                fidelity_features=fidelity_features,
-                # Kwargs below are unused in base `Surrogate`, but used in subclasses.
                 metric_names=metric_names,
-                task_features=task_features,
+                **dataclasses.asdict(search_space_digest)
             )
         if state_dict:
+            # pyre-fixme[6]: Expected `OrderedDict[typing.Any, typing.Any]` for 1st
+            #  param but got `Dict[str, Tensor]`.
             self.model.load_state_dict(not_none(state_dict))
 
         if state_dict is None or refit:
-            # pyre-ignore[16]: Model has no attribute likelihood.
-            # All BoTorch `Model`-s expected to work with this setup have likelihood.
             mll = self.mll_class(self.model.likelihood, self.model)
             fit_gpytorch_model(mll)
 
@@ -256,7 +247,7 @@ class Surrogate(Base):
 
     def best_in_sample_point(
         self,
-        bounds: List[Tuple[float, float]],
+        search_space_digest: SearchSpaceDigest,
         objective_weights: Optional[Tensor],
         outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
         linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
@@ -274,7 +265,7 @@ class Surrogate(Base):
             # TODO: When we move `botorch_modular` directory to OSS, we will extend
             # the annotation for `model` kwarg to accept `Surrogate` too.
             model=self,
-            bounds=bounds,
+            bounds=search_space_digest.bounds,
             objective_weights=objective_weights,
             outcome_constraints=outcome_constraints,
             linear_constraints=linear_constraints,
@@ -288,13 +279,11 @@ class Surrogate(Base):
 
     def best_out_of_sample_point(
         self,
-        bounds: List[Tuple[float, float]],
+        search_space_digest: SearchSpaceDigest,
         objective_weights: Tensor,
         outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
         linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
         fixed_features: Optional[Dict[int, float]] = None,
-        fidelity_features: Optional[List[int]] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
         options: Optional[TConfig] = None,
     ) -> Tuple[Tensor, Tensor]:
         """Finds the best predicted point and the corresponding value of the
@@ -320,19 +309,16 @@ class Surrogate(Base):
         acqf = Acquisition(  # TODO: For multi-fidelity, might need diff. class.
             surrogate=self,
             botorch_acqf_class=acqf_class,
-            bounds=bounds,
+            search_space_digest=search_space_digest,
             objective_weights=objective_weights,
             outcome_constraints=outcome_constraints,
             linear_constraints=linear_constraints,
             fixed_features=fixed_features,
-            target_fidelities=target_fidelities,
             options=acqf_options,
         )
         candidates, acqf_values = acqf.optimize(
-            # pyre-ignore[6]: Exp. Tensor, got List[Tuple[float, float]].
-            # TODO: Fix typing of `bounds` in `TorchModel`-s.
-            bounds=bounds,
             n=1,
+            search_space_digest=search_space_digest,
             inequality_constraints=_to_inequality_constraints(
                 linear_constraints=linear_constraints
             ),
@@ -343,19 +329,15 @@ class Surrogate(Base):
     def update(
         self,
         training_data: TrainingData,
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
-        target_fidelities: Optional[Dict[int, float]] = None,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
         state_dict: Optional[Dict[str, Tensor]] = None,
         refit: bool = True,
     ) -> None:
         """Updates the surrogate model with new data. In the base ``Surrogate``,
         just calls ``fit`` after checking that this surrogate was not created
-        via ``Surrogate.from_BoTorch`` (in which case the ``Model`` comes premade,
+        via ``Surrogate.from_botorch`` (in which case the ``Model`` comes premade,
         constructed manually and then supplied to ``Surrogate``).
 
         NOTE: Expects `training_data` to be all available data,
@@ -364,15 +346,9 @@ class Surrogate(Base):
         Args:
             training_data: Surrogate training_data containing all the data the model
                 should use for inference.
-            bounds: A list of d (lower, upper) tuples for each column of X.
-            task_features: Columns of X that take integer values and should be
-                treated as task parameters.
-            feature_names: Names of each column of X.
+            search_space_digest: A SearchSpaceDigest object containing
+                metadata on the features in the training data.
             metric_names: Names of each outcome Y in Ys.
-            fidelity_features: Columns of X that should be treated as fidelity
-                parameters.
-            target_fidelities: Target values for fidelity parameters, representing
-                full-fidelity value.
             candidate_metadata: Model-produced metadata for candidates, in
                 the order corresponding to the Xs.
             state_dict: Optional state dict to load.
@@ -386,16 +362,12 @@ class Surrogate(Base):
                 "`update` not yet implemented for models that are "
                 "constructed manually, but it is possible to create a new "
                 "surrogate in the same way as the current manually constructed one, "
-                "via `Surrogate.from_BoTorch`."
+                "via `Surrogate.from_botorch`."
             )
         self.fit(
             training_data=training_data,
-            bounds=bounds,
-            task_features=task_features,
-            feature_names=feature_names,
+            search_space_digest=search_space_digest,
             metric_names=metric_names,
-            fidelity_features=fidelity_features,
-            target_fidelities=target_fidelities,
             candidate_metadata=candidate_metadata,
             state_dict=state_dict,
             refit=refit,
