@@ -12,22 +12,28 @@ References
     Improvement for Parallel Multi-Objective Bayesian Optimization. Advances in Neural
     Information Processing Systems 33, 2020.
 
+.. [Daulton2021nehvi]
+    S. Daulton, M. Balandat, and E. Bakshy. Parallel Bayesian Optimization of
+    Multiple Noisy Objectives. ArXiv, 2021.
+
 """
 
-import warnings
 from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import torch
-from ax.exceptions.core import AxWarning
 from ax.models.torch.utils import (  # noqa F40
     _to_inequality_constraints,
     get_outcome_constraint_transforms,
     predict_from_model,
 )
 from ax.models.torch_base import TorchModel
+from ax.utils.common.constants import Keys
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.multi_objective.objective import WeightedMCMultiOutputObjective
-from botorch.acquisition.utils import get_acquisition_function
+from botorch.acquisition.multi_objective.utils import get_default_partitioning_alpha
+from botorch.acquisition.utils import (
+    get_acquisition_function,
+)
 from botorch.models.model import Model
 from botorch.optim.optimize import optimize_acqf_list
 from botorch.utils.multi_objective.pareto import is_non_dominated
@@ -38,7 +44,8 @@ DEFAULT_EHVI_MC_SAMPLES = 128
 
 
 # Callable that takes tensors of observations and model parameters,
-# then returns means of observations that make up a pareto frontier.
+# then returns means of observations that make up a pareto frontier,
+# along with their covariances and their index in the input observations.
 TFrontierEvaluator = Callable[
     [
         TorchModel,
@@ -49,7 +56,7 @@ TFrontierEvaluator = Callable[
         Optional[Tensor],
         Optional[Tuple[Tensor, Tensor]],
     ],
-    Tuple[Tensor, Tensor],
+    Tuple[Tensor, Tensor, Tensor],
 ]
 
 
@@ -79,6 +86,7 @@ def get_weighted_mc_objective_and_objective_thresholds(
     # pyre-ignore [16]
     nonzero_idcs = objective_weights.nonzero(as_tuple=False).view(-1)
     objective_weights = objective_weights[nonzero_idcs]
+    objective_thresholds = objective_thresholds[nonzero_idcs]
     objective = WeightedMCMultiOutputObjective(
         weights=objective_weights, outcomes=nonzero_idcs.tolist()
     )
@@ -86,6 +94,75 @@ def get_weighted_mc_objective_and_objective_thresholds(
     return objective, objective_thresholds
 
 
+def get_NEHVI(
+    model: Model,
+    objective_weights: Tensor,
+    objective_thresholds: Tensor,
+    outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+    X_observed: Optional[Tensor] = None,
+    X_pending: Optional[Tensor] = None,
+    **kwargs: Any,
+) -> AcquisitionFunction:
+    r"""Instantiates a qNoisyExpectedHyperVolumeImprovement acquisition function.
+
+    Args:
+        model: The underlying model which the acqusition function uses
+            to estimate acquisition values of candidates.
+        objective_weights: The objective is to maximize a weighted sum of
+            the columns of f(x). These are the weights.
+        outcome_constraints: A tuple of (A, b). For k outcome constraints
+            and m outputs at f(x), A is (k x m) and b is (k x 1) such that
+            A f(x) <= b. (Not used by single task models)
+        X_observed: A tensor containing points observed for all objective
+            outcomes and outcomes that appear in the outcome constraints (if
+            there are any).
+        X_pending: A tensor containing points whose evaluation is pending (i.e.
+            that have been submitted for evaluation) present for all objective
+            outcomes and outcomes that appear in the outcome constraints (if
+            there are any).
+        mc_samples: The number of MC samples to use (default: 512).
+        qmc: If True, use qMC instead of MC (default: True).
+        prune_baseline: If True, prune the baseline points for NEI (default: True).
+        chebyshev_scalarization: Use augmented Chebyshev scalarization.
+
+    Returns:
+        qNoisyExpectedHyperVolumeImprovement: The instantiated acquisition function.
+    """
+    if X_observed is None:
+        raise ValueError("There are no feasible observed points.")
+    # construct Objective module
+    (
+        objective,
+        objective_thresholds,
+    ) = get_weighted_mc_objective_and_objective_thresholds(
+        objective_weights=objective_weights, objective_thresholds=objective_thresholds
+    )
+    # For EHVI acquisition functions we pass the constraint transform directly.
+    if outcome_constraints is None:
+        cons_tfs = None
+    else:
+        cons_tfs = get_outcome_constraint_transforms(outcome_constraints)
+    num_objectives = objective_thresholds.shape[0]
+    return get_acquisition_function(
+        acquisition_function_name="qNEHVI",
+        model=model,
+        objective=objective,  # pyre-ignore [6]
+        X_observed=X_observed,
+        X_pending=X_pending,
+        constraints=cons_tfs,
+        prune_baseline=kwargs.get("prune_baseline", True),
+        mc_samples=kwargs.get("mc_samples", DEFAULT_EHVI_MC_SAMPLES),
+        alpha=kwargs.get(
+            "alpha", get_default_partitioning_alpha(num_objectives=num_objectives)
+        ),
+        qmc=kwargs.get("qmc", True),
+        seed=torch.randint(1, 10000, (1,)).item(),
+        ref_point=objective_thresholds.tolist(),
+        marginalize_dim=kwargs.get("marginalize_dim"),
+        match_right_most_batch_dim=kwargs.get("match_right_most_batch_dim", False),
+    )
+
+
 def get_EHVI(
     model: Model,
     objective_weights: Tensor,
@@ -130,9 +207,8 @@ def get_EHVI(
     ) = get_weighted_mc_objective_and_objective_thresholds(
         objective_weights=objective_weights, objective_thresholds=objective_thresholds
     )
-    if "Ys" not in kwargs:
-        raise ValueError("Expected Hypervolume Improvement requires Ys argument")
-    Y_tensor = torch.stack(kwargs.get("Ys")).transpose(0, 1).squeeze(-1)
+    with torch.no_grad():
+        Y = model.posterior(X_observed).mean
     # For EHVI acquisition functions we pass the constraint transform directly.
     if outcome_constraints is None:
         cons_tfs = None
@@ -157,7 +233,7 @@ def get_EHVI(
         ),
         seed=torch.randint(1, 10000, (1,)).item(),
         ref_point=objective_thresholds.tolist(),
-        Y=Y_tensor,
+        Y=Y,
     )
 
 
@@ -196,8 +272,8 @@ def scipy_optimizer_list(
           values, where `i`-th element is the expected acquisition value
           conditional on having observed candidates `0,1,...,i-1`.
     """
-    num_restarts: int = kwargs.get("num_restarts", 20)
-    raw_samples: int = kwargs.get("num_raw_samples", 50 * num_restarts)
+    num_restarts: int = kwargs.pop(Keys.NUM_RESTARTS, 20)
+    raw_samples: int = kwargs.pop(Keys.RAW_SAMPLES, 50 * num_restarts)
 
     # use SLSQP by default for small problems since it yields faster wall times
     if "method" not in kwargs:
@@ -223,7 +299,7 @@ def pareto_frontier_evaluator(
     Y: Optional[Tensor] = None,
     Yvar: Optional[Tensor] = None,
     outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-) -> Tuple[Tensor, Tensor]:
+) -> Tuple[Tensor, Tensor, Tensor]:
     """Return outcomes predicted to lie on a pareto frontier.
 
     Given a model and a points to evaluate use the model to predict which points
@@ -238,18 +314,19 @@ def pareto_frontier_evaluator(
             dominate the objective_thresholds contribute nothing to hypervolume.
         X: A `n x d` tensor of features to evaluate.
         Y: A `n x m` tensor of outcomes to use instead of predictions.
-        Yvar: A `n x m` tensor of input variances (NaN if unobserved).
+        Yvar: A `n x m x m` tensor of input covariances (NaN if unobserved).
         outcome_constraints: A tuple of (A, b). For k outcome constraints
             and m outputs at f(x), A is (k x m) and b is (k x 1) such that
             A f(x) <= b.
 
     Returns:
-        2-element tuple containing
+        3-element tuple containing
 
         - A `j x m` tensor of outcome on the pareto frontier. j is the number
             of frontier points.
         - A `j x m x m` tensor of predictive covariances.
             cov[j, m1, m2] is Cov[m1@j, m2@j].
+        - A `j` tensor of the index of each frontier point in the input Y.
     """
     if X is not None:
         Y, Yvar = model.predict(X)
@@ -273,6 +350,7 @@ def pareto_frontier_evaluator(
         ),
     )
     Y_obj = obj(Y)
+    indx_frontier = torch.arange(Y.shape[0], dtype=torch.long, device=Y.device)
 
     # Filter Y, Yvar, Y_obj to items that dominate all objective thresholds
     if objective_thresholds is not None:
@@ -280,6 +358,7 @@ def pareto_frontier_evaluator(
         Y = Y[objective_thresholds_mask]
         Yvar = Yvar[objective_thresholds_mask]
         Y_obj = Y_obj[objective_thresholds_mask]
+        indx_frontier = indx_frontier[objective_thresholds_mask]
 
     # Get feasible points that do not violate outcome_constraints
     if outcome_constraints is not None:
@@ -289,6 +368,12 @@ def pareto_frontier_evaluator(
         Y = Y[feas]
         Yvar = Yvar[feas]
         Y_obj = Y_obj[feas]
+        indx_frontier = indx_frontier[feas]
+
+    if Y.shape[0] == 0:
+        # if there are no feasible points that are better than the reference point
+        # return empty tensors
+        return Y, Yvar, indx_frontier
 
     # calculate pareto front with only objective outcomes:
     frontier_mask = is_non_dominated(Y_obj)
@@ -296,18 +381,5 @@ def pareto_frontier_evaluator(
     # Apply masks
     Y_frontier = Y[frontier_mask]
     Yvar_frontier = Yvar[frontier_mask]
-    return Y_frontier, Yvar_frontier
-
-
-def get_default_partitioning_alpha(num_objectives: int) -> float:
-    """Adaptively selects a reasonable partitioning based on the number of objectives.
-
-    This strategy is derived from the results in [Daulton2020qehvi]_, which suggest
-    that this heuristic provides a reasonable trade-off between the closed-loop
-    performance and the wall time required for the partitioning.
-    """
-    if num_objectives == 2:
-        return 0.0
-    elif num_objectives > 6:
-        warnings.warn("EHVI works best for less than 7 objectives.", AxWarning)
-    return 10 ** (-8 + num_objectives)
+    indx_frontier = indx_frontier[frontier_mask]
+    return Y_frontier, Yvar_frontier, indx_frontier
