@@ -4,15 +4,23 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from itertools import chain
 from typing import Any
-from unittest.mock import patch
+from unittest import mock
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.models.torch.botorch_modular.acquisition import Acquisition
-from ax.models.torch.botorch_modular.list_surrogate import ListSurrogate
 from ax.models.torch.botorch_modular.surrogate import Surrogate
+from ax.models.torch.utils import SubsetModelData
 from ax.utils.common.constants import Keys
 from ax.utils.common.testutils import TestCase
+from botorch.acquisition.input_constructors import (
+    ACQF_INPUT_CONSTRUCTOR_REGISTRY,
+    get_acqf_input_constructor,
+    _register_acqf_input_constructor,
+)
+from botorch.acquisition.monte_carlo import qNoisyExpectedImprovement
 from botorch.acquisition.objective import LinearMCObjective
 from botorch.models.gp_regression import SingleTaskGP
 from botorch.utils.containers import TrainingData
@@ -35,59 +43,75 @@ class DummyACQFClass:
 
 class AcquisitionTest(TestCase):
     def setUp(self):
+        qNEI_input_constructor = get_acqf_input_constructor(qNoisyExpectedImprovement)
+        self.mock_input_constructor = mock.MagicMock(
+            qNEI_input_constructor, side_effect=qNEI_input_constructor
+        )
+        # Adding wrapping here to be able to count calls and inspect arguments.
+        _register_acqf_input_constructor(
+            acqf_cls=DummyACQFClass,
+            input_constructor=self.mock_input_constructor,
+        )
         self.botorch_model_class = SingleTaskGP
         self.surrogate = Surrogate(botorch_model_class=self.botorch_model_class)
         self.X = torch.tensor([[1.0, 2.0, 3.0], [2.0, 3.0, 4.0]])
         self.Y = torch.tensor([[3.0], [4.0]])
         self.Yvar = torch.tensor([[0.0], [2.0]])
-        self.training_data = TrainingData(X=self.X, Y=self.Y, Yvar=self.Yvar)
+        self.training_data = TrainingData.from_block_design(
+            X=self.X, Y=self.Y, Yvar=self.Yvar
+        )
         self.fidelity_features = [2]
         self.surrogate.construct(
             training_data=self.training_data, fidelity_features=self.fidelity_features
         )
-
-        self.bounds = [(0.0, 10.0), (0.0, 10.0), (0.0, 10.0)]
+        self.search_space_digest = SearchSpaceDigest(
+            feature_names=["a", "b", "c"],
+            bounds=[(0.0, 10.0), (0.0, 10.0), (0.0, 10.0)],
+            target_fidelities={2: 1.0},
+        )
         self.botorch_acqf_class = DummyACQFClass
         self.objective_weights = torch.tensor([1.0])
-        self.pending_observations = [
-            torch.tensor([[1.0, 3.0, 4.0]]),
-            torch.tensor([[2.0, 6.0, 8.0]]),
-        ]
+        self.objective_thresholds = None
+        self.pending_observations = [torch.tensor([[1.0, 3.0, 4.0]])]
         self.outcome_constraints = (torch.tensor([[1.0]]), torch.tensor([[0.5]]))
         self.linear_constraints = None
         self.fixed_features = {1: 2.0}
-        self.target_fidelities = {2: 1.0}
         self.options = {"best_f": 0.0}
         self.acquisition = Acquisition(
-            surrogate=self.surrogate,
-            bounds=self.bounds,
-            objective_weights=self.objective_weights,
             botorch_acqf_class=self.botorch_acqf_class,
+            surrogate=self.surrogate,
+            search_space_digest=self.search_space_digest,
+            objective_weights=self.objective_weights,
+            objective_thresholds=self.objective_thresholds,
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
             fixed_features=self.fixed_features,
-            target_fidelities=self.target_fidelities,
             options=self.options,
         )
-
         self.inequality_constraints = [
             (torch.tensor([0, 1]), torch.tensor([-1.0, 1.0]), 1)
         ]
         self.rounding_func = lambda x: x
         self.optimizer_options = {Keys.NUM_RESTARTS: 40, Keys.RAW_SAMPLES: 1024}
 
-    @patch(
-        f"{ACQUISITION_PATH}._get_X_pending_and_observed",
-        return_value=(torch.tensor([2.0]), torch.tensor([3.0])),
+    def tearDown(self):
+        # Avoid polluting the registry for other tests.
+        ACQF_INPUT_CONSTRUCTOR_REGISTRY.pop(DummyACQFClass)
+
+    @mock.patch(f"{ACQUISITION_PATH}._get_X_pending_and_observed")
+    @mock.patch(
+        f"{ACQUISITION_PATH}.subset_model",
+        return_value=SubsetModelData(None, None, None, None, None),
     )
-    @patch(f"{ACQUISITION_PATH}.subset_model", return_value=(None, None, None, None))
-    @patch(f"{ACQUISITION_PATH}.get_botorch_objective")
-    @patch(
+    @mock.patch(f"{ACQUISITION_PATH}.get_botorch_objective")
+    @mock.patch(
         f"{CURRENT_PATH}.Acquisition.compute_model_dependencies",
         return_value={"current_value": 1.2},
     )
-    @patch(f"{DummyACQFClass.__module__}.DummyACQFClass.__init__", return_value=None)
+    @mock.patch(
+        f"{DummyACQFClass.__module__}.DummyACQFClass.__init__", return_value=None
+    )
     def test_init(
         self,
         mock_botorch_acqf_class,
@@ -96,28 +120,25 @@ class AcquisitionTest(TestCase):
         mock_subset_model,
         mock_get_X,
     ):
-        self.acquisition.default_botorch_acqf_class = None
-        with self.assertRaisesRegex(
-            ValueError, ".*`botorch_acqf_class` argument must be specified."
-        ):
+        with self.assertRaisesRegex(TypeError, ".* missing .* 'botorch_acqf_class'"):
             Acquisition(
                 surrogate=self.surrogate,
-                bounds=self.bounds,
+                search_space_digest=self.search_space_digest,
                 objective_weights=self.objective_weights,
             )
 
         botorch_objective = LinearMCObjective(weights=torch.tensor([1.0]))
         mock_get_objective.return_value = botorch_objective
+        mock_get_X.return_value = (self.pending_observations[0], self.X[:1])
         acquisition = Acquisition(
             surrogate=self.surrogate,
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
             botorch_acqf_class=self.botorch_acqf_class,
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
             fixed_features=self.fixed_features,
-            target_fidelities=self.target_fidelities,
             options=self.options,
         )
 
@@ -127,84 +148,204 @@ class AcquisitionTest(TestCase):
             pending_observations=self.pending_observations,
             objective_weights=self.objective_weights,
             outcome_constraints=self.outcome_constraints,
-            bounds=self.bounds,
+            bounds=self.search_space_digest.bounds,
             linear_constraints=self.linear_constraints,
             fixed_features=self.fixed_features,
         )
         # Call `subset_model` only when needed
         mock_subset_model.assert_called_with(
-            acquisition.surrogate.model,
+            model=acquisition.surrogate.model,
             objective_weights=self.objective_weights,
             outcome_constraints=self.outcome_constraints,
+            objective_thresholds=self.objective_thresholds,
         )
         mock_subset_model.reset_mock()
+        mock_get_objective.reset_mock()
+        self.mock_input_constructor.reset_mock()
+        mock_botorch_acqf_class.reset_mock()
         self.options[Keys.SUBSET_MODEL] = False
         acquisition = Acquisition(
             surrogate=self.surrogate,
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
             botorch_acqf_class=self.botorch_acqf_class,
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
             fixed_features=self.fixed_features,
-            target_fidelities=self.target_fidelities,
             options=self.options,
         )
         mock_subset_model.assert_not_called()
         # Check `get_botorch_objective` kwargs
-        mock_get_objective.assert_called_with(
-            model=self.acquisition.surrogate.model,
-            objective_weights=self.objective_weights,
-            outcome_constraints=self.outcome_constraints,
-            X_observed=torch.tensor([3.0]),
-            use_scalarized_objective=False,
-        )
+        mock_get_objective.assert_called_once()
+        _, ckwargs = mock_get_objective.call_args
+        self.assertIs(ckwargs["model"], self.acquisition.surrogate.model)
+        self.assertIs(ckwargs["objective_weights"], self.objective_weights)
+        self.assertIs(ckwargs["outcome_constraints"], self.outcome_constraints)
+        self.assertTrue(torch.equal(ckwargs["X_observed"], self.X[:1]))
+        self.assertFalse(ckwargs["use_scalarized_objective"])
         # Check final `acqf` creation
         model_deps = {Keys.CURRENT_VALUE: 1.2}
-        mock_botorch_acqf_class.assert_called_with(
-            model=self.acquisition.surrogate.model,
-            objective=botorch_objective,
-            X_pending=torch.tensor([2.0]),
-            X_baseline=torch.tensor([3.0]),
-            **self.options,
-            **model_deps,
-        )
+        self.mock_input_constructor.assert_called_once()
+        mock_botorch_acqf_class.assert_called_once()
+        _, ckwargs = self.mock_input_constructor.call_args
+        self.assertIs(ckwargs["model"], self.acquisition.surrogate.model)
+        self.assertIs(ckwargs["objective"], botorch_objective)
+        self.assertTrue(torch.equal(ckwargs["X_pending"], self.pending_observations[0]))
+        for k, v in chain(self.options.items(), model_deps.items()):
+            self.assertEqual(ckwargs[k], v)
 
-    @patch(f"{ACQUISITION_PATH}.optimize_acqf")
+    @mock.patch(f"{ACQUISITION_PATH}.optimize_acqf")
     def test_optimize(self, mock_optimize_acqf):
         self.acquisition.optimize(
-            bounds=self.bounds,
             n=3,
-            optimizer_class=None,
+            search_space_digest=self.search_space_digest,
             inequality_constraints=self.inequality_constraints,
             fixed_features=self.fixed_features,
             rounding_func=self.rounding_func,
             optimizer_options=self.optimizer_options,
         )
         mock_optimize_acqf.assert_called_with(
-            self.acquisition.acqf,
-            bounds=self.bounds,
+            acq_function=self.acquisition.acqf,
+            bounds=mock.ANY,
             q=3,
             inequality_constraints=self.inequality_constraints,
             fixed_features=self.fixed_features,
             post_processing_func=self.rounding_func,
             **self.optimizer_options,
         )
+        # can't use assert_called_with on bounds due to ambiguous bool comparison
+        expected_bounds = torch.tensor(
+            self.search_space_digest.bounds,
+            dtype=self.acquisition.dtype,
+            device=self.acquisition.device,
+        ).transpose(0, 1)
+        self.assertTrue(
+            torch.equal(mock_optimize_acqf.call_args[1]["bounds"], expected_bounds)
+        )
 
-    @patch(f"{SURROGATE_PATH}.Surrogate.best_in_sample_point")
+    @mock.patch(f"{ACQUISITION_PATH}.optimize_acqf_discrete")
+    def test_optimize_discrete(self, mock_optimize_acqf_discrete):
+        tkwargs = {
+            "dtype": self.acquisition.dtype,
+            "device": self.acquisition.device,
+        }
+        ssd1 = SearchSpaceDigest(
+            feature_names=["a"],
+            bounds=[(0, 2)],
+            categorical_features=[0],
+            discrete_choices={0: [0, 1, 2]},
+        )
+        # check fixed_feature index validation
+        with self.assertRaisesRegex(ValueError, "Invalid fixed_feature index"):
+            self.acquisition.optimize(
+                n=3,
+                search_space_digest=ssd1,
+                inequality_constraints=self.inequality_constraints,
+                fixed_features=self.fixed_features,
+                rounding_func=self.rounding_func,
+                optimizer_options=self.optimizer_options,
+            )
+        # check this works without any fixed_feature specified
+        self.acquisition.optimize(
+            n=3,
+            search_space_digest=ssd1,
+            inequality_constraints=self.inequality_constraints,
+            fixed_features=None,
+            rounding_func=self.rounding_func,
+            optimizer_options=self.optimizer_options,
+        )
+        mock_optimize_acqf_discrete.assert_called_with(
+            acq_function=self.acquisition.acqf,
+            q=3,
+            choices=mock.ANY,
+            **self.optimizer_options,
+        )
+        # can't use assert_called_with on choices due to ambiguous bool comparison
+        expected_choices = torch.tensor([[0], [1], [2]], **tkwargs)
+        self.assertTrue(
+            torch.equal(
+                mock_optimize_acqf_discrete.call_args[1]["choices"], expected_choices
+            )
+        )
+        # check with fixed feature
+        ssd2 = SearchSpaceDigest(
+            feature_names=["a", "b"],
+            bounds=[(0, 2), (0, 1)],
+            categorical_features=[0],
+            discrete_choices={0: [0, 1, 2]},
+        )
+        self.acquisition.optimize(
+            n=3,
+            search_space_digest=ssd2,
+            inequality_constraints=self.inequality_constraints,
+            fixed_features=self.fixed_features,
+            rounding_func=self.rounding_func,
+            optimizer_options=self.optimizer_options,
+        )
+        mock_optimize_acqf_discrete.assert_called_with(
+            acq_function=self.acquisition.acqf,
+            q=3,
+            choices=mock.ANY,
+            **self.optimizer_options,
+        )
+        # can't use assert_called_with on choices due to ambiguous bool comparison
+        expected_choices = torch.tensor([[0, 2.0], [1, 2.0], [2, 2.0]], **tkwargs)
+        self.assertTrue(
+            torch.equal(
+                mock_optimize_acqf_discrete.call_args[1]["choices"], expected_choices
+            )
+        )
+
+    @mock.patch(f"{ACQUISITION_PATH}.optimize_acqf_mixed")
+    def test_optimize_mixed(self, mock_optimize_acqf_mixed):
+        tkwargs = {
+            "dtype": self.acquisition.dtype,
+            "device": self.acquisition.device,
+        }
+        ssd = SearchSpaceDigest(
+            feature_names=["a", "b"],
+            bounds=[(0, 1), (0, 2)],
+            categorical_features=[1],
+            discrete_choices={1: [0, 1, 2]},
+        )
+        self.acquisition.optimize(
+            n=3,
+            search_space_digest=ssd,
+            inequality_constraints=self.inequality_constraints,
+            fixed_features=None,
+            rounding_func=self.rounding_func,
+            optimizer_options=self.optimizer_options,
+        )
+        mock_optimize_acqf_mixed.assert_called_with(
+            acq_function=self.acquisition.acqf,
+            bounds=mock.ANY,
+            q=3,
+            fixed_features_list=[{1: 0}, {1: 1}, {1: 2}],
+            inequality_constraints=self.inequality_constraints,
+            post_processing_func=self.rounding_func,
+            **self.optimizer_options,
+        )
+        # can't use assert_called_with on bounds due to ambiguous bool comparison
+        expected_bounds = torch.tensor(ssd.bounds, **tkwargs).transpose(0, 1)
+        self.assertTrue(
+            torch.equal(
+                mock_optimize_acqf_mixed.call_args[1]["bounds"], expected_bounds
+            )
+        )
+
+    @mock.patch(f"{SURROGATE_PATH}.Surrogate.best_in_sample_point")
     def test_best_point(self, mock_best_point):
         self.acquisition.best_point(
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
             fixed_features=self.fixed_features,
-            target_fidelities=self.target_fidelities,
             options=self.options,
         )
         mock_best_point.assert_called_with(
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
@@ -212,20 +353,9 @@ class AcquisitionTest(TestCase):
             options=self.options,
         )
 
-    @patch(f"{DummyACQFClass.__module__}.DummyACQFClass.__call__", return_value=None)
+    @mock.patch(
+        f"{DummyACQFClass.__module__}.DummyACQFClass.__call__", return_value=None
+    )
     def test_evaluate(self, mock_call):
         self.acquisition.evaluate(X=self.X)
         mock_call.assert_called_with(X=self.X)
-
-    def test_extract_training_data(self):
-        self.assertEqual(  # Base `Surrogate` case.
-            self.acquisition._extract_training_data(surrogate=self.surrogate),
-            self.training_data,
-        )
-        # `ListSurrogate` case.
-        list_surrogate = ListSurrogate(botorch_submodel_class=self.botorch_model_class)
-        list_surrogate._training_data_per_outcome = {"a": self.training_data}
-        self.assertEqual(
-            self.acquisition._extract_training_data(surrogate=list_surrogate),
-            list_surrogate._training_data_per_outcome,
-        )
