@@ -7,6 +7,7 @@
 from unittest.mock import patch
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.list_surrogate import (
     NOT_YET_FIT_MSG,
@@ -18,7 +19,7 @@ from ax.utils.common.testutils import TestCase
 from ax.utils.testing.torch_stubs import get_torch_test_data
 from botorch.models.model import TrainingData
 from botorch.models.model_list_gp_regression import ModelListGP
-from botorch.models.multitask import FixedNoiseMultiTaskGP
+from botorch.models.multitask import FixedNoiseMultiTaskGP, MultiTaskGP
 from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
 
 
@@ -33,19 +34,22 @@ class ListSurrogateTest(TestCase):
         self.outcomes = ["outcome_1", "outcome_2"]
         self.mll_class = SumMarginalLogLikelihood
         self.dtype = torch.float
+        self.search_space_digest = SearchSpaceDigest(
+            feature_names=[], bounds=[], task_features=[0]
+        )
         self.task_features = [0]
         Xs1, Ys1, Yvars1, bounds, _, _, _ = get_torch_test_data(
-            dtype=self.dtype, task_features=self.task_features
+            dtype=self.dtype, task_features=self.search_space_digest.task_features
         )
         Xs2, Ys2, Yvars2, _, _, _, _ = get_torch_test_data(
-            dtype=self.dtype, task_features=self.task_features
+            dtype=self.dtype, task_features=self.search_space_digest.task_features
         )
         self.botorch_submodel_class_per_outcome = {
             self.outcomes[0]: choose_model_class(
-                Yvars=Yvars1, task_features=self.task_features, fidelity_features=[]
+                Yvars=Yvars1, search_space_digest=self.search_space_digest
             ),
             self.outcomes[1]: choose_model_class(
-                Yvars=Yvars2, task_features=self.task_features, fidelity_features=[]
+                Yvars=Yvars2, search_space_digest=self.search_space_digest
             ),
         }
         self.expected_submodel_type = FixedNoiseMultiTaskGP
@@ -54,10 +58,7 @@ class ListSurrogateTest(TestCase):
         self.Xs = Xs1 + Xs2
         self.Ys = Ys1 + Ys2
         self.Yvars = Yvars1 + Yvars2
-        self.training_data = [
-            TrainingData(X=X, Y=Y, Yvar=Yvar)
-            for X, Y, Yvar in zip(self.Xs, self.Ys, self.Yvars)
-        ]
+        self.training_data = TrainingData(Xs=self.Xs, Ys=self.Ys, Yvars=self.Yvars)
         self.submodel_options_per_outcome = {
             self.outcomes[0]: {RANK: 1},
             self.outcomes[1]: {RANK: 2},
@@ -86,8 +87,6 @@ class ListSurrogateTest(TestCase):
             self.botorch_submodel_class_per_outcome,
         )
         self.assertEqual(self.surrogate.mll_class, self.mll_class)
-        with self.assertRaises(NotImplementedError):
-            self.surrogate.training_data
         with self.assertRaisesRegex(ValueError, NOT_YET_FIT_MSG):
             self.surrogate.training_data_per_outcome
         with self.assertRaisesRegex(
@@ -95,10 +94,10 @@ class ListSurrogateTest(TestCase):
         ):
             self.surrogate.model
 
-    @patch(
-        f"{CURRENT_PATH}.FixedNoiseMultiTaskGP.construct_inputs",
-        # Mock to register calls, but still execute the function.
-        side_effect=FixedNoiseMultiTaskGP.construct_inputs,
+    @patch.object(
+        FixedNoiseMultiTaskGP,
+        "construct_inputs",
+        wraps=FixedNoiseMultiTaskGP.construct_inputs,
     )
     def test_construct_per_outcome_options(self, mock_MTGP_construct_inputs):
         with self.assertRaisesRegex(ValueError, ".* are required"):
@@ -123,17 +122,45 @@ class ListSurrogateTest(TestCase):
                 {
                     "fidelity_features": [],
                     "task_features": self.task_features,
-                    "training_data": self.training_data[idx],
+                    "training_data": self.training_data.from_block_design(
+                        X=self.Xs[idx], Y=self.Ys[idx], Yvar=self.Yvars[idx]
+                    ),
                     "rank": self.submodel_options_per_outcome[self.outcomes[idx]][
                         "rank"
                     ],
                 },
             )
 
-    @patch(
-        f"{CURRENT_PATH}.FixedNoiseMultiTaskGP.construct_inputs",
-        # Mock to register calls, but still execute the function.
-        side_effect=FixedNoiseMultiTaskGP.construct_inputs,
+    @patch.object(
+        MultiTaskGP,
+        "construct_inputs",
+        wraps=MultiTaskGP.construct_inputs,
+    )
+    def test_construct_per_outcome_options_no_Yvar(self, _):
+        surrogate = ListSurrogate(
+            botorch_submodel_class=MultiTaskGP,
+            mll_class=self.mll_class,
+            submodel_options_per_outcome=self.submodel_options_per_outcome,
+        )
+
+        # Test that splitting the training data works correctly when Yvar is None.
+        training_data_no_Yvar = TrainingData(Xs=self.Xs, Ys=self.Ys)
+        surrogate.construct(
+            training_data=training_data_no_Yvar,
+            task_features=self.task_features,
+            metric_names=self.outcomes,
+        )
+        self.assertTrue(
+            all(
+                trd.Yvar is None for trd in surrogate.training_data_per_outcome.values()
+            )
+        )
+        self.assertEqual(len(surrogate.training_data_per_outcome), 2)
+
+    @patch.object(
+        FixedNoiseMultiTaskGP,
+        "construct_inputs",
+        wraps=FixedNoiseMultiTaskGP.construct_inputs,
     )
     def test_construct_shared_shortcut_options(self, mock_construct_inputs):
         surrogate = ListSurrogate(
@@ -162,7 +189,9 @@ class ListSurrogateTest(TestCase):
                     "individual_option": f"val_{idx}",
                     "shared_option": True,
                     "task_features": [0],
-                    "training_data": self.training_data[idx],
+                    "training_data": self.training_data.from_block_design(
+                        X=self.Xs[idx], Y=self.Ys[idx], Yvar=self.Yvars[idx]
+                    ),
                 },
             )
 
@@ -180,11 +209,12 @@ class ListSurrogateTest(TestCase):
         # is `None`.
         surrogate.fit(
             training_data=self.training_data,
-            bounds=self.bounds,
-            task_features=self.task_features,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                task_features=self.task_features,
+            ),
             metric_names=self.outcomes,
-            fidelity_features=[],
         )
         mock_state_dict.assert_not_called()
         mock_MLL.assert_called_once()
@@ -197,11 +227,12 @@ class ListSurrogateTest(TestCase):
         state_dict = {"state_attribute": "value"}
         surrogate.fit(
             training_data=self.training_data,
-            bounds=self.bounds,
-            task_features=self.task_features,
-            feature_names=self.feature_names,
+            search_space_digest=SearchSpaceDigest(
+                feature_names=self.feature_names,
+                bounds=self.bounds,
+                task_features=self.task_features,
+            ),
             metric_names=self.outcomes,
-            fidelity_features=[],
             refit=False,
             state_dict=state_dict,
         )
