@@ -4,9 +4,11 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import dataclasses
 from unittest.mock import patch
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.multi_fidelity import MultiFidelityAcquisition
 from ax.models.torch.botorch_modular.surrogate import Surrogate
@@ -31,15 +33,20 @@ class MultiFidelityAcquisitionTest(TestCase):
         self.X = torch.tensor([[1.0, 2.0, 3.0], [2.0, 3.0, 4.0]])
         self.Y = torch.tensor([[3.0], [4.0]])
         self.Yvar = torch.tensor([[0.0], [2.0]])
-        self.training_data = TrainingData(X=self.X, Y=self.Y, Yvar=self.Yvar)
+        self.training_data = TrainingData.from_block_design(
+            X=self.X, Y=self.Y, Yvar=self.Yvar
+        )
         self.fidelity_features = [2]
         self.surrogate.construct(
             training_data=self.training_data, fidelity_features=self.fidelity_features
         )
         self.acquisition_options = {Keys.NUM_FANTASIES: 64}
-        self.bounds = [(0.0, 10.0), (0.0, 10.0), (0.0, 10.0)]
+        self.search_space_digest = SearchSpaceDigest(
+            feature_names=["a", "b", "c"],
+            bounds=[(0.0, 10.0), (0.0, 10.0), (0.0, 10.0)],
+            target_fidelities={2: 1.0},
+        )
         self.objective_weights = torch.tensor([1.0])
-        self.target_fidelities = {2: 1.0}
         self.pending_observations = [
             torch.tensor([[1.0, 3.0, 4.0]]),
             torch.tensor([[2.0, 6.0, 8.0]]),
@@ -52,23 +59,6 @@ class MultiFidelityAcquisitionTest(TestCase):
             Keys.COST_INTERCEPT: 1.0,
             Keys.NUM_TRACE_OBSERVATIONS: 0,
         }
-        with patch(f"{MFKG_PATH}.__init__", return_value=None):
-            # We don't actually need to instantiate the BoTorch acqf in these tests.
-            self.acquisition = MultiFidelityAcquisition(
-                surrogate=self.surrogate,
-                bounds=self.bounds,
-                objective_weights=self.objective_weights,
-                botorch_acqf_class=qMultiFidelityKnowledgeGradient,
-                target_fidelities=self.target_fidelities,
-            )
-
-    @patch(f"{ACQUISITION_PATH}.Acquisition.__init__", return_value=None)
-    @patch(f"{ACQUISITION_PATH}.Acquisition.optimize")
-    def test_optimize(self, mock_Acquisition_optimize, mock_Acquisition_init):
-        # `self.acquisition.optimize()` should call `Acquisition.optimize()`
-        # once.
-        self.acquisition.optimize(bounds=self.bounds, n=1)
-        mock_Acquisition_optimize.assert_called_once()
 
     @patch(
         f"{ACQUISITION_PATH}.Acquisition.compute_model_dependencies", return_value={}
@@ -85,14 +75,30 @@ class MultiFidelityAcquisitionTest(TestCase):
         mock_affine_model,
         mock_Acquisition_compute,
     ):
-        # Raise Error if `fidelity_weights` and `target_fidelities` do
-        # not align.
-        with self.assertRaisesRegex(RuntimeError, "Must provide the same indices"):
-            self.acquisition.compute_model_dependencies(
+        # TODO: Patch only `MFKG_PATH.__init__` once `construct_inputs`
+        # implemented for qMFKG.
+        with patch(
+            f"{MULTI_FIDELITY_PATH}.MultiFidelityAcquisition.__init__",
+            return_value=None,
+        ):
+            # We don't actually need to instantiate the BoTorch acqf in these tests.
+            mf_acquisition = MultiFidelityAcquisition(
                 surrogate=self.surrogate,
-                bounds=self.bounds,
+                search_space_digest=self.search_space_digest,
+                objective_weights=self.objective_weights,
+                botorch_acqf_class=qMultiFidelityKnowledgeGradient,
+            )
+        # Raise Error if `fidelity_weights` and `target_fidelities` do not align.
+        with self.assertRaisesRegex(RuntimeError, "Must provide the same indices"):
+            mf_acquisition.compute_model_dependencies(
+                surrogate=self.surrogate,
+                search_space_digest=SearchSpaceDigest(
+                    **{
+                        **dataclasses.asdict(self.search_space_digest),
+                        "target_fidelities": {1: 5.0},
+                    }
+                ),
                 objective_weights=self.objective_weights,
-                target_fidelities={1: 5.0},
                 pending_observations=self.pending_observations,
                 outcome_constraints=self.outcome_constraints,
                 linear_constraints=self.linear_constraints,
@@ -100,11 +106,15 @@ class MultiFidelityAcquisitionTest(TestCase):
                 options=self.options,
             )
         # Make sure `fidelity_weights` are set when they are not passed in.
-        self.acquisition.compute_model_dependencies(
+        mf_acquisition.compute_model_dependencies(
             surrogate=self.surrogate,
-            bounds=self.bounds,
+            search_space_digest=SearchSpaceDigest(
+                **{
+                    **dataclasses.asdict(self.search_space_digest),
+                    "target_fidelities": {2: 5.0, 3: 5.0},
+                }
+            ),
             objective_weights=self.objective_weights,
-            target_fidelities={2: 5.0, 3: 5.0},
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
@@ -115,11 +125,10 @@ class MultiFidelityAcquisitionTest(TestCase):
             fidelity_weights={2: 1.0, 3: 1.0}, fixed_cost=1.0
         )
         # Usual case.
-        dependencies = self.acquisition.compute_model_dependencies(
+        dependencies = mf_acquisition.compute_model_dependencies(
             surrogate=self.surrogate,
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
-            target_fidelities=self.target_fidelities,
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
@@ -128,9 +137,8 @@ class MultiFidelityAcquisitionTest(TestCase):
         )
         mock_Acquisition_compute.assert_called_with(
             surrogate=self.surrogate,
-            bounds=self.bounds,
+            search_space_digest=self.search_space_digest,
             objective_weights=self.objective_weights,
-            target_fidelities=self.target_fidelities,
             pending_observations=self.pending_observations,
             outcome_constraints=self.outcome_constraints,
             linear_constraints=self.linear_constraints,
@@ -149,12 +157,13 @@ class MultiFidelityAcquisitionTest(TestCase):
         project = dependencies.get(Keys.PROJECT)
         project(torch.tensor([1.0]))
         mock_project.assert_called_with(
-            X=torch.tensor([1.0]), target_fidelities=self.target_fidelities
+            X=torch.tensor([1.0]),
+            target_fidelities=self.search_space_digest.target_fidelities,
         )
         expand = dependencies.get(Keys.EXPAND)
         expand(torch.tensor([1.0]))
         mock_expand.assert_called_with(
             X=torch.tensor([1.0]),
-            fidelity_dims=sorted(self.target_fidelities),
+            fidelity_dims=sorted(self.search_space_digest.target_fidelities),
             num_trace_obs=self.options.get(Keys.NUM_TRACE_OBSERVATIONS),
         )
