@@ -4,6 +4,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from dataclasses import dataclass
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, cast
 
 import numpy as np
@@ -18,6 +19,7 @@ from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.analytic import PosteriorMean
 from botorch.acquisition.fixed_feature import FixedFeatureAcquisitionFunction
 from botorch.acquisition.monte_carlo import qSimpleRegret
+from botorch.acquisition.multi_objective.objective import WeightedMCMultiOutputObjective
 from botorch.acquisition.objective import (
     AcquisitionObjective,
     ConstrainedMCObjective,
@@ -47,6 +49,15 @@ SIMPLEX = "simplex"
 HYPERSPHERE = "hypersphere"
 
 
+@dataclass
+class SubsetModelData:
+    model: Model
+    objective_weights: Tensor
+    outcome_constraints: Optional[Tuple[Tensor, Tensor]]
+    objective_thresholds: Optional[Tensor]
+    indices: Tensor
+
+
 def is_noiseless(model: Model) -> bool:
     """Check if a given (single-task) botorch model is noiseless"""
     if isinstance(model, ModelListGP):
@@ -112,6 +123,9 @@ def _get_X_pending_and_observed(
 ) -> Tuple[Optional[Tensor], Optional[Tensor]]:
     r"""Get pending and observed points.
 
+    If all points would otherwise be filtered, remove `linear_constraints`
+    and `fixed_features` from filter and retry.
+
     Args:
         Xs: The input tensors of a model.
         pending_observations:  A list of m (k_i x d) feature tensors X
@@ -146,7 +160,7 @@ def _get_X_pending_and_observed(
             linear_constraints=linear_constraints,
             fixed_features=fixed_features,
         )
-    X_observed = _filter_X_observed(
+    filtered_X_observed = _filter_X_observed(
         Xs=Xs,
         objective_weights=objective_weights,
         outcome_constraints=outcome_constraints,
@@ -154,7 +168,16 @@ def _get_X_pending_and_observed(
         linear_constraints=linear_constraints,
         fixed_features=fixed_features,
     )
-    return X_pending, X_observed
+    if filtered_X_observed is not None and len(filtered_X_observed) > 0:
+        return X_pending, filtered_X_observed
+    else:
+        unfiltered_X_observed = _filter_X_observed(
+            Xs=Xs,
+            objective_weights=objective_weights,
+            bounds=bounds,
+            outcome_constraints=outcome_constraints,
+        )
+        return X_pending, unfiltered_X_observed
 
 
 def _generate_sobol_points(
@@ -217,8 +240,8 @@ def subset_model(
     model: Model,
     objective_weights: Tensor,
     outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-    Ys: Optional[List[Tensor]] = None,
-) -> Tuple[Model, Tensor, Optional[Tuple[Tensor, Tensor]], Optional[List[Tensor]]]:
+    objective_thresholds: Optional[Tensor] = None,
+) -> SubsetModelData:
     """Subset a botorch model to the outputs used in the optimization.
 
     Args:
@@ -227,25 +250,33 @@ def subset_model(
             input arguments.
         objective_weights: The objective is to maximize a weighted sum of
             the columns of f(x). These are the weights.
+        objective_thresholds:  The `m`-dim tensor of objective thresholds. There
+            is one for each modeled metric.
         outcome_constraints: A tuple of (A, b). For k outcome constraints
             and m outputs at f(x), A is (k x m) and b is (k x 1) such that
             A f(x) <= b. (Not used by single task models)
-        Ys: The corresponding list of m (k_i x 1) outcome tensors Y, for
-            each outcome.
 
     Returns:
-        A three-tuple of model, objective_weights, and outcome_constraints, all
-        subset to only those outputs that appear in either the objective weights
-        or the outcome constraints.
+        A SubsetModelData dataclass containing the model, objective_weights,
+        outcome_constraints, objective thresholds, all subset to only those
+        outputs that appear in either the objective weights or the outcome
+        constraints, along with the indices of the outputs.
     """
     nonzero = objective_weights != 0
     if outcome_constraints is not None:
         A, _ = outcome_constraints
         nonzero = nonzero | torch.any(A != 0, dim=0)
-    idcs = torch.arange(nonzero.size(0))[nonzero].tolist()
+    idcs_t = torch.arange(nonzero.size(0))[nonzero]
+    idcs = idcs_t.tolist()
     if len(idcs) == model.num_outputs:
         # if we use all model outputs, just return the inputs
-        return model, objective_weights, outcome_constraints, Ys
+        return SubsetModelData(
+            model=model,
+            objective_weights=objective_weights,
+            outcome_constraints=outcome_constraints,
+            objective_thresholds=objective_thresholds,
+            indices=idcs_t,
+        )
     elif len(idcs) > model.num_outputs:
         raise RuntimeError(
             "Model size inconsistency. Tryting to subset a model with "
@@ -257,10 +288,17 @@ def subset_model(
         if outcome_constraints is not None:
             A, b = outcome_constraints
             outcome_constraints = A[:, nonzero], b
-        Ys = [Ys[i] for i in idcs] if Ys is not None else None
+        if objective_thresholds is not None:
+            objective_thresholds = objective_thresholds[nonzero]
     except NotImplementedError:
         pass
-    return model, objective_weights, outcome_constraints, Ys
+    return SubsetModelData(
+        model=model,
+        objective_weights=objective_weights,
+        outcome_constraints=outcome_constraints,
+        objective_thresholds=objective_thresholds,
+        indices=idcs_t,
+    )
 
 
 def _to_inequality_constraints(
@@ -283,7 +321,7 @@ def _to_inequality_constraints(
 def tensor_callable_to_array_callable(
     tensor_func: Callable[[Tensor], Tensor], device: torch.device
 ) -> Callable[[np.ndarray], np.ndarray]:
-    """"transfer a tensor callable to an array callable"""
+    """transfer a tensor callable to an array callable"""
     # TODO: move this reuseable function and its  equivalent reverse functions
     # to some utils files
     def array_func(x: np.ndarray) -> np.ndarray:
@@ -297,6 +335,7 @@ def get_botorch_objective(
     objective_weights: Tensor,
     use_scalarized_objective: bool = True,
     outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+    objective_thresholds: Optional[Tensor] = None,
     X_observed: Optional[Tensor] = None,
 ) -> AcquisitionObjective:
     """Constructs a BoTorch `AcquisitionObjective` object.
@@ -312,6 +351,9 @@ def get_botorch_objective(
         outcome_constraints: A tuple of (A, b). For k outcome constraints
             and m outputs at f(x), A is (k x m) and b is (k x 1) such that
             A f(x) <= b. (Not used by single task models)
+        objective_thresholds: A tensor containing thresholds forming a reference point
+            from which to calculate pareto frontier hypervolume. Points that do not
+            dominate the objective_thresholds contribute nothing to hypervolume.
         X_observed: Observed points that are feasible and appear in the
             objective or the constraints. None if there are no such points.
 
@@ -319,6 +361,12 @@ def get_botorch_objective(
         A BoTorch `AcquisitionObjective` object. It will be one of:
         `ScalarizedObjective`, `LinearMCOObjective`, `ConstrainedMCObjective`.
     """
+    if objective_thresholds is not None:
+        nonzero_idcs = torch.nonzero(objective_weights).view(-1)
+        objective_weights = objective_weights[nonzero_idcs]
+        return WeightedMCMultiOutputObjective(
+            weights=objective_weights, outcomes=nonzero_idcs.tolist()
+        )
     if X_observed is None:
         raise UnsupportedError(
             "X_observed is required to construct a BoTorch Objective."
@@ -370,11 +418,14 @@ def get_out_of_sample_best_point_acqf(
 
     # subset model only to the outcomes we need for the optimization
     if kwargs.get(Keys.SUBSET_MODEL, True):
-        model, objective_weights, outcome_constraints, _ = subset_model(
+        subset_model_results = subset_model(
             model=model,
             objective_weights=objective_weights,
             outcome_constraints=outcome_constraints,
         )
+        model = subset_model_results.model
+        objective_weights = subset_model_results.objective_weights
+        outcome_constraints = subset_model_results.outcome_constraints
 
     fixed_features = fixed_features or {}
     target_fidelities = target_fidelities or {}
@@ -498,5 +549,6 @@ def randomize_objective_weights(
                 len(objective_weights), dtype=dtype, device=device
             ).squeeze()
         )
+    # pyre-fixme[61]: `random_weights` may not be initialized here.
     objective_weights = torch.mul(objective_weights, random_weights)
     return objective_weights
