@@ -7,14 +7,16 @@
 from typing import Callable, Dict, List, Optional, Tuple
 
 import torch
+from ax.core.search_space import SearchSpaceDigest
 from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
-from ax.models.base import Model
+from ax.models.base import Model as BaseModel
+from botorch.models.model import Model
 from torch import Tensor
 
 
 # pyre-fixme[13]: Attribute `device` is never initialized.
 # pyre-fixme[13]: Attribute `dtype` is never initialized.
-class TorchModel(Model):
+class TorchModel(BaseModel):
     """This class specifies the interface for a torch-based model.
 
     These methods should be implemented to have access to all of the features
@@ -29,11 +31,8 @@ class TorchModel(Model):
         Xs: List[Tensor],
         Ys: List[Tensor],
         Yvars: List[Tensor],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
         """Fit model to m outcomes.
@@ -44,13 +43,9 @@ class TorchModel(Model):
             Ys: The corresponding list of m (k_i x 1) outcome tensors Y, for
                 each outcome.
             Yvars: The variances of each entry in Ys, same shape.
-            bounds: A list of d (lower, upper) tuples for each column of X.
-            task_features: Columns of X that take integer values and should be
-                treated as task parameters.
-            feature_names: Names of each column of X.
+            search_space_digest: A SearchSpaceDigest object containing
+                metadata on the features in X.
             metric_names: Names of each outcome Y in Ys.
-            fidelity_features: Columns of X that should be treated as fidelity
-                parameters.
             candidate_metadata: Model-produced metadata for candidates, in
                 the order corresponding to the Xs.
         """
@@ -166,11 +161,8 @@ class TorchModel(Model):
         Ys_train: List[Tensor],
         Yvars_train: List[Tensor],
         X_test: Tensor,
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
     ) -> Tuple[Tensor, Tensor]:
         """Do cross validation with the given training and test sets.
 
@@ -184,13 +176,9 @@ class TorchModel(Model):
                 for each outcome.
             Yvars_train: The variances of each entry in Ys, same shape.
             X_test: (j x d) tensor of the j points at which to make predictions.
-            bounds: A list of d (lower, upper) tuples for each column of X.
-            task_features: Columns of X that take integer values and should be
-                treated as task parameters.
-            feature_names: Names of each column of X.
+            search_space_digest: A SearchSpaceDigest object containing
+                metadata on the features in X.
             metric_names: Names of each outcome Y in Ys.
-            fidelity_features: Columns of X that should be treated as fidelity
-                parameters.
 
         Returns:
             2-element tuple containing
@@ -206,12 +194,8 @@ class TorchModel(Model):
         Xs: List[Tensor],
         Ys: List[Tensor],
         Yvars: List[Tensor],
-        bounds: List[Tuple[float, float]],
-        task_features: List[int],
-        feature_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         metric_names: List[str],
-        fidelity_features: List[int],
-        target_fidelities: Optional[Dict[int, float]] = None,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
         """Update the model.
@@ -226,15 +210,9 @@ class TorchModel(Model):
                 in the same format as for `fit`.
             Yvars: Existing + additional data for the model,
                 in the same format as for `fit`.
-            bounds: A list of d (lower, upper) tuples for each column of X.
-            task_features: Columns of X that take integer values and should be
-                treated as task parameters.
-            feature_names: Names of each column of X.
+            search_space_digest: A SearchSpaceDigest object containing
+                metadata on the features in X.
             metric_names: Names of each outcome Y in Ys.
-            fidelity_features: Columns of X that should be treated as fidelity
-                parameters.
-            target_fidelities: Target values for fidelity parameters, representing
-                full-fidelity value.
             candidate_metadata: Model-produced metadata for candidates, in
                 the order corresponding to the Xs.
         """
@@ -251,3 +229,35 @@ class TorchModel(Model):
             A single-element tensor with the acquisition value for these points.
         """
         raise NotImplementedError
+
+    def infer_objective_thresholds(
+        self,
+        objective_weights: Tensor,  # objective_directions
+        bounds: Optional[List[Tuple[float, float]]] = None,
+        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
+        fixed_features: Optional[Dict[int, float]] = None,
+        X_observed: Optional[Tensor] = None,
+        model: Optional[Model] = None,
+        subset_idcs: Optional[Tensor] = None,
+    ) -> Tensor:
+        """Infer objective thresholds.
+
+        Args:
+            objective_weights: The objective is to maximize a weighted sum of
+                the columns of f(x). These are the weights.
+            bounds: A list of (lower, upper) tuples for each column of X.
+            outcome_constraints: A tuple of (A, b). For k outcome constraints
+                and m outputs at f(x), A is (k x m) and b is (k x 1) such that
+                A f(x) <= b.
+            linear_constraints: A tuple of (A, b). For k linear constraints on
+                d-dimensional x, A is (k x d) and b is (k x 1) such that
+                A x <= b.
+            fixed_features: A map {feature_index: value} for features that
+                should be fixed to a particular value during generation.
+            X_observed: A `n x d`-dim tensor of observed in-sample points
+            model: The model
+            subset_idcs: The indices of the outcomes are that are used in the
+                optimization config (if the model has been subset'd).
+        """
+        raise NotImplementedError
