@@ -12,13 +12,12 @@ from typing import Any, Dict, List, Optional, Tuple, Union, TypeVar, Type
 import ax.service.utils.best_point as best_point_utils
 import numpy as np
 import pandas as pd
+from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial
 from ax.core.batch_trial import BatchTrial
-from ax.core.data import Data
-from ax.core.experiment import Experiment
+from ax.core.experiment import DataType, Experiment
 from ax.core.generator_run import GeneratorRun
-from ax.core.metric import Metric
 from ax.core.trial import Trial
 from ax.core.types import (
     TEvaluationOutcome,
@@ -33,7 +32,6 @@ from ax.modelbridge.generation_strategy import GenerationStrategy
 from ax.modelbridge.modelbridge_utils import get_pending_observation_features
 from ax.plot.base import AxPlotConfig
 from ax.plot.contour import plot_contour
-from ax.plot.exp_utils import exp_to_df
 from ax.plot.feature_importances import plot_feature_importance_by_feature
 from ax.plot.helper import _format_dict, _get_in_sample_arms
 from ax.plot.trace import optimization_trace_single_method
@@ -42,6 +40,7 @@ from ax.service.utils.instantiation import (
     make_experiment,
     raw_data_to_evaluation,
 )
+from ax.service.utils.report_utils import exp_to_df
 from ax.service.utils.with_db_settings_base import DBSettings, WithDBSettingsBase
 from ax.storage.json_store.decoder import (
     generation_strategy_from_json,
@@ -168,7 +167,11 @@ class AxClient(WithDBSettingsBase):
         status_quo: Optional[TParameterization] = None,
         overwrite_existing_experiment: bool = False,
         experiment_type: Optional[str] = None,
+        tracking_metric_names: Optional[List[str]] = None,
         choose_generation_strategy_kwargs: Optional[Dict[str, Any]] = None,
+        support_intermediate_data: bool = False,
+        immutable_search_space_and_opt_config: bool = True,
+        is_test: bool = False,
     ) -> None:
         """Create a new experiment and save it if DBSettings available.
 
@@ -211,44 +214,23 @@ class AxClient(WithDBSettingsBase):
                 To protect experiments in production, one cannot overwrite existing
                 experiments if the experiment is already stored in the database,
                 regardless of the value of `overwrite_existing_experiment`.
+            tracking_metric_names: Names of additional tracking metrics not used for
+                optimization.
             choose_generation_strategy_kwargs: Keyword arguments to pass to
                 `choose_generation_strategy` function which determines what
                 generation strategy should be used when none was specified on init.
+            support_intermediate_data: Whether trials may report intermediate results
+                for trials that are still running (i.e. have not been completed via
+                `ax_client.complete_trial`).
+            immutable_search_space_and_opt_config: Whether it's possible to update the
+                search space and optimization config on this experiment after creation.
+                Defaults to True. If set to True, we won't store or load copies of the
+                search space and optimization config on each generator run, which will
+                improve storage performance.
+            is_test: Whether this experiment will be a test experiment (useful for
+                marking test experiments in storage etc). Defaults to False.
         """
-        if self.db_settings_set and not name:
-            raise ValueError(  # pragma: no cover
-                "Must give the experiment a name if `db_settings` is not None."
-            )
-        if self.db_settings_set:
-            experiment_id, _ = self._get_experiment_and_generation_strategy_db_id(
-                experiment_name=not_none(name)
-            )
-            if experiment_id:
-                raise ValueError(
-                    f"Experiment {name} already exists in the database. "
-                    "To protect experiments that are running in production, "
-                    "overwriting stored experiments is not allowed. To "
-                    "start a new experiment and store it, change the "
-                    "experiment's name."
-                )
-        if self._experiment is not None:
-            if overwrite_existing_experiment:
-                exp_name = self.experiment._name or "untitled"
-                new_exp_name = name or "untitled"
-                logger.info(
-                    f"Overwriting existing experiment ({exp_name}) on this client "
-                    f"with new experiment ({new_exp_name}) and restarting the "
-                    "generation strategy."
-                )
-                self._generation_strategy = None
-            else:
-                raise ValueError(
-                    "Experiment already created for this client instance. "
-                    "Set the `overwrite_existing_experiment` to `True` to overwrite "
-                    "with new experiment."
-                )
-
-        self._experiment = make_experiment(
+        experiment = make_experiment(
             name=name,
             parameters=parameters,
             objective_name=objective_name,
@@ -257,22 +239,15 @@ class AxClient(WithDBSettingsBase):
             outcome_constraints=outcome_constraints,
             status_quo=status_quo,
             experiment_type=experiment_type,
+            tracking_metric_names=tracking_metric_names,
+            support_intermediate_data=support_intermediate_data,
+            immutable_search_space_and_opt_config=immutable_search_space_and_opt_config,
+            is_test=is_test,
+        )
+        self._set_experiment(
+            experiment=experiment,
+            overwrite_existing_experiment=overwrite_existing_experiment,
         )
-
-        try:
-            self._save_experiment_to_db_if_possible(
-                experiment=self.experiment,
-                suppress_all_errors=self._suppress_storage_errors,
-            )
-        except Exception:
-            # Unset the experiment on this `AxClient` instance if encountered and
-            # raising an error from saving the experiment, to avoid a case where
-            # overall `create_experiment` call fails with a storage error, but
-            # `self._experiment` is still set and user has to specify the
-            # `ooverwrite_existing_experiment` kwarg to re-attempt exp. creation.
-            self._experiment = None
-            raise
-
         self._set_generation_strategy(
             choose_generation_strategy_kwargs=choose_generation_strategy_kwargs
         )
@@ -312,7 +287,7 @@ class AxClient(WithDBSettingsBase):
             f"{_round_floats_for_logging(item=not_none(trial.arm).parameters)}."
         )
         trial.mark_running(no_runner_required=True)
-        self._save_new_trial_to_db_if_possible(
+        self._save_or_update_trial_in_db_if_possible(
             experiment=self.experiment,
             trial=trial,
             suppress_all_errors=self._suppress_storage_errors,
@@ -335,6 +310,63 @@ class AxClient(WithDBSettingsBase):
         trial = self._get_trial(trial_index=trial_index)
         trial.mark_abandoned(reason=reason)
 
+    def update_running_trial_with_intermediate_data(
+        self,
+        trial_index: int,
+        raw_data: TEvaluationOutcome,
+        metadata: Optional[Dict[str, Union[str, int]]] = None,
+        sample_size: Optional[int] = None,
+    ) -> None:
+        """
+        Updates the trial with given metric values without completing it. Also
+        adds optional metadata to it. Useful for intermediate results like
+        the metrics of a partially optimized machine learning model. In these
+        cases it should be called instead of `complete_trial` until it is
+        time to complete the trial.
+
+        NOTE: When ``raw_data`` does not specify SEM for a given metric, Ax
+        will default to the assumption that the data is noisy (specifically,
+        corrupted by additive zero-mean Gaussian noise) and that the
+        level of noise should be inferred by the optimization model. To
+        indicate that the data is noiseless, set SEM to 0.0, for example:
+
+        .. code-block:: python
+
+          ax_client.update_trial(
+              trial_index=0,
+              raw_data={"my_objective": (objective_mean_value, 0.0)}
+          )
+
+        Args:
+            trial_index: Index of trial within the experiment.
+            raw_data: Evaluation data for the trial. Can be a mapping from
+                metric name to a tuple of mean and SEM, just a tuple of mean and
+                SEM if only one metric in optimization, or just the mean if SEM is
+                unknown (then Ax will infer observation noise level).
+                Can also be a list of (fidelities, mapping from
+                metric name to a tuple of mean and SEM).
+            metadata: Additional metadata to track about this run.
+            sample_size: Number of samples collected for the underlying arm,
+                optional.
+        """
+        if not isinstance(trial_index, int):  # pragma: no cover
+            raise ValueError(f"Trial index must be an int, got: {trial_index}.")
+        if not self.experiment.default_data_type == DataType.MAP_DATA:
+            raise ValueError(
+                "`update_running_trial_with_intermediate_data` requires that "
+                "this client's `experiment` be constructed with "
+                "`support_intermediate_data=True` and have `default_data_type` of "
+                "`DataType.MAP_DATA`."
+            )
+        # TODO(jej)[T86911509]: Add support for efficient updates with single results.
+        data_update_repr = self._update_trial_with_raw_data(
+            trial_index=trial_index,
+            raw_data=raw_data,
+            metadata=metadata,
+            sample_size=sample_size,
+        )
+        logger.info(f"Updated trial {trial_index} with data: " f"{data_update_repr}.")
+
     def complete_trial(
         self,
         trial_index: int,
@@ -372,33 +404,18 @@ class AxClient(WithDBSettingsBase):
                 optional.
         """
         # Validate that trial can be completed.
-        if not isinstance(trial_index, int):  # pragma: no cover
-            raise ValueError(f"Trial index must be an int, got: {trial_index}.")
         trial = self._get_trial(trial_index=trial_index)
         self._validate_can_complete_trial(trial=trial)
-
-        # Format the data to save.
-        sample_sizes = {not_none(trial.arm).name: sample_size} if sample_size else {}
-        evaluations, data = self._make_evaluations_and_data(
-            trial=trial, raw_data=raw_data, metadata=metadata, sample_sizes=sample_sizes
-        )
-        self._validate_trial_data(trial=trial, data=data)
-        trial._run_metadata = metadata or {}
-
-        self.experiment.attach_data(data=data)
-        trial.mark_completed()
-        data_for_logging = _round_floats_for_logging(
-            item=evaluations[next(iter(evaluations.keys()))]
-        )
-        logger.info(
-            f"Completed trial {trial_index} with data: "
-            f"{_round_floats_for_logging(item=data_for_logging)}."
-        )
-        self._save_updated_trial_to_db_if_possible(
-            experiment=self.experiment,
-            trial=trial,
-            suppress_all_errors=self._suppress_storage_errors,
+        if not isinstance(trial_index, int):  # pragma: no cover
+            raise ValueError(f"Trial index must be an int, got: {trial_index}.")
+        data_update_repr = self._update_trial_with_raw_data(
+            trial_index=trial_index,
+            raw_data=raw_data,
+            metadata=metadata,
+            sample_size=sample_size,
+            complete_trial=True,
         )
+        logger.info(f"Completed trial {trial_index} with data: " f"{data_update_repr}.")
 
     def update_trial_data(
         self,
@@ -423,22 +440,21 @@ class AxClient(WithDBSettingsBase):
             sample_size: Number of samples collected for the underlying arm,
                 optional.
         """
-        assert isinstance(
-            trial_index, int
-        ), f"Trial index must be an int, got: {trial_index}."  # pragma: no cover
+        if not isinstance(trial_index, int):  # pragma: no cover
+            raise ValueError(f"Trial index must be an int, got: {trial_index}.")
         trial = self._get_trial(trial_index=trial_index)
         if not trial.status.is_completed:
             raise ValueError(
                 f"Trial {trial.index} has not yet been completed with data."
                 "To complete it, use `ax_client.complete_trial`."
             )
-        sample_sizes = {not_none(trial.arm).name: sample_size} if sample_size else {}
-        evaluations, data = self._make_evaluations_and_data(
-            trial=trial, raw_data=raw_data, metadata=metadata, sample_sizes=sample_sizes
+        data_update_repr = self._update_trial_with_raw_data(
+            trial_index=trial_index,
+            raw_data=raw_data,
+            metadata=metadata,
+            sample_size=sample_size,
+            combine_with_last_data=True,
         )
-        self._validate_trial_data(trial=trial, data=data)
-        trial._run_metadata.update(metadata or {})
-
         # Registering trial data update is needed for generation strategies that
         # leverage the `update` functionality of model and bridge setup and therefore
         # need to be aware of new data added to experiment. Usually this happends
@@ -446,19 +462,8 @@ class AxClient(WithDBSettingsBase):
         # status does not change, so we manually register the new data.
         # Currently this call will only result in a `NotImplementedError` if generation
         # strategy uses `update` (`GenerationStep.use_update` is False by default).
-        self.generation_strategy._register_trial_data_update(trial=trial, data=data)
-        self.experiment.attach_data(data, combine_with_last_data=True)
-        data_for_logging = _round_floats_for_logging(
-            item=evaluations[next(iter(evaluations.keys()))]
-        )
-        logger.info(
-            f"Added data: {_round_floats_for_logging(item=data_for_logging)} "
-            f"to trial {trial.index}."
-        )
-        self._save_experiment_to_db_if_possible(
-            experiment=self.experiment,
-            suppress_all_errors=self._suppress_storage_errors,
-        )
+        self.generation_strategy._register_trial_data_update(trial=trial)
+        logger.info(f"Added data: {data_update_repr} to trial {trial.index}.")
 
     def log_trial_failure(
         self, trial_index: int, metadata: Optional[Dict[str, str]] = None
@@ -502,7 +507,7 @@ class AxClient(WithDBSettingsBase):
             "Attached custom parameterization "
             f"{_round_floats_for_logging(item=parameters)} as trial {trial.index}."
         )
-        self._save_new_trial_to_db_if_possible(
+        self._save_or_update_trial_in_db_if_possible(
             experiment=self.experiment,
             trial=trial,
             suppress_all_errors=self._suppress_storage_errors,
@@ -885,6 +890,105 @@ class AxClient(WithDBSettingsBase):
         opt_config = not_none(self.experiment.optimization_config)
         return opt_config.objective.metric.name
 
+    def _update_trial_with_raw_data(
+        self,
+        trial_index: int,
+        raw_data: TEvaluationOutcome,
+        metadata: Optional[Dict[str, Union[str, int]]] = None,
+        sample_size: Optional[int] = None,
+        complete_trial: bool = False,
+        combine_with_last_data: bool = False,
+    ) -> str:
+        """Helper method attaches data to a trial, returns a str of update."""
+        # Format the data to save.
+        trial = self._get_trial(trial_index=trial_index)
+        sample_sizes = {not_none(trial.arm).name: sample_size} if sample_size else {}
+        evaluations, data = self._make_evaluations_and_data(
+            trial=trial, raw_data=raw_data, metadata=metadata, sample_sizes=sample_sizes
+        )
+        metadata = metadata or {}
+        self._validate_trial_data(trial=trial, data=data)
+        trial.update_run_metadata(metadata=metadata)
+
+        self.experiment.attach_data(
+            data=data, combine_with_last_data=combine_with_last_data
+        )
+        if complete_trial:
+            trial.mark_completed()
+        self._save_or_update_trial_in_db_if_possible(
+            experiment=self.experiment,
+            trial=trial,
+            suppress_all_errors=self._suppress_storage_errors,
+        )
+        return str(
+            _round_floats_for_logging(item=evaluations[next(iter(evaluations.keys()))])
+        )
+
+    def _set_experiment(
+        self,
+        experiment: Experiment,
+        overwrite_existing_experiment: bool = False,
+    ) -> None:
+        """Sets the ``_experiment`` attribute on this `AxClient`` instance and saves the
+        experiment if this instance uses SQL storage.
+
+        NOTE: This setter **should not be used outside of this file in production**.
+        It can be leveraged in development, but all checked-in code that uses the
+        Service API should leverage ``AxClient.create_experiment`` instead and extend it
+        as needed. If using ``create_experiment`` is impossible and this setter is
+        required, please raise your use case to the AE team or on our Github.
+        """
+        name = experiment._name
+
+        if self.db_settings_set and not name:
+            raise ValueError(  # pragma: no cover
+                "Must give the experiment a name if `db_settings` is not None."
+            )
+        if self.db_settings_set:
+            experiment_id, _ = self._get_experiment_and_generation_strategy_db_id(
+                experiment_name=not_none(name)
+            )
+            if experiment_id:
+                raise ValueError(
+                    f"Experiment {name} already exists in the database. "
+                    "To protect experiments that are running in production, "
+                    "overwriting stored experiments is not allowed. To "
+                    "start a new experiment and store it, change the "
+                    "experiment's name."
+                )
+        if self._experiment is not None:
+            if overwrite_existing_experiment:
+                exp_name = self.experiment._name or "untitled"
+                new_exp_name = name or "untitled"
+                logger.info(
+                    f"Overwriting existing experiment ({exp_name}) on this client "
+                    f"with new experiment ({new_exp_name}) and restarting the "
+                    "generation strategy."
+                )
+                self._generation_strategy = None
+            else:
+                raise ValueError(
+                    "Experiment already created for this client instance. "
+                    "Set the `overwrite_existing_experiment` to `True` to overwrite "
+                    "with new experiment."
+                )
+
+        self._experiment = experiment
+
+        try:
+            self._save_experiment_to_db_if_possible(
+                experiment=self.experiment,
+                suppress_all_errors=self._suppress_storage_errors,
+            )
+        except Exception:
+            # Unset the experiment on this `AxClient` instance if encountered and
+            # raising an error from saving the experiment, to avoid a case where
+            # overall `create_experiment` call fails with a storage error, but
+            # `self._experiment` is still set and user has to specify the
+            # `ooverwrite_existing_experiment` kwarg to re-attempt exp. creation.
+            self._experiment = None
+            raise
+
     def _set_generation_strategy(
         self, choose_generation_strategy_kwargs: Optional[Dict[str, Any]] = None
     ) -> None:
@@ -959,7 +1063,7 @@ class AxClient(WithDBSettingsBase):
         raw_data: Union[TEvaluationOutcome, Dict[str, TEvaluationOutcome]],
         metadata: Optional[Dict[str, Union[str, int]]],
         sample_sizes: Optional[Dict[str, int]] = None,
-    ) -> Tuple[Dict[str, TEvaluationOutcome], Data]:
+    ) -> Tuple[Dict[str, TEvaluationOutcome], AbstractDataFrameData]:
         """Formats given raw data as Ax evaluations and `Data`.
 
         Args:
@@ -1042,14 +1146,18 @@ class AxClient(WithDBSettingsBase):
                     f"on experiment creation for {p_name}."
                 )
 
-    def _validate_trial_data(self, trial: Trial, data: Data) -> None:
+    def _validate_trial_data(self, trial: Trial, data: AbstractDataFrameData) -> None:
         for metric_name in data.df["metric_name"].values:
             if metric_name not in self.experiment.metrics:
                 logger.info(
                     f"Data was logged for metric {metric_name} that was not yet "
-                    "tracked on the experiment. Adding it as tracking metric."
+                    "tracked on the experiment. Please specify `tracking_metric_"
+                    "names` argument in AxClient.create_experiment to add tracking "
+                    "metrics to the experiment. Without those, all data users "
+                    "specify is still attached to the experiment, but will not be "
+                    "fetched in `experiment.fetch_data()`, but you can still use "
+                    "`experiment.lookup_data_for_trial` to get all attached data."
                 )
-                self.experiment.add_tracking_metric(Metric(name=metric_name))
 
     # -------- Backward-compatibility with old save / load method names. -------
 
