@@ -29,11 +29,11 @@ try:  # We don't require SQLAlchemy by default.
         _load_generation_strategy_by_experiment_name,
     )
     from ax.storage.sqa_store.save import (
+        _save_or_update_trials,
         _save_experiment,
         _save_generation_strategy,
-        _save_new_trials,
         _update_generation_strategy,
-        _update_trials,
+        update_properties_on_experiment,
     )
     from ax.storage.sqa_store.structs import DBSettings
     from sqlalchemy.exc import OperationalError
@@ -45,6 +45,8 @@ except ModuleNotFoundError:  # pragma: no cover
     DBSettings = None
 
 
+STORAGE_MINI_BATCH_SIZE = 50
+
 logger = get_logger(__name__)
 
 
@@ -94,7 +96,7 @@ class WithDBSettingsBase:
             return None, None
 
         exp_id = _get_experiment_id(
-            experiment_name=experiment_name, decoder=self.db_settings.decoder
+            experiment_name=experiment_name, config=self.db_settings.decoder.config
         )
         if not exp_id:
             return None, None
@@ -140,7 +142,7 @@ class WithDBSettingsBase:
                     "Experiment was associated with generation strategy in DB, "
                     f"but a new generation strategy {generation_strategy.name} "
                     "was provided. To use the generation strategy currently in DB,"
-                    " instantiate scheduler via: `Scheduler.with_stored_experiment`."
+                    " instantiate scheduler via: `Scheduler.from_stored_experiment`."
                 )
             if not gs_id or generation_strategy._db_id is None:
                 # There is no GS associated with experiment or the generation
@@ -246,7 +248,11 @@ class WithDBSettingsBase:
         """
         if self.db_settings_set:
             start_time = time.time()
-            _save_experiment(experiment, encoder=self.db_settings.encoder)
+            _save_experiment(
+                experiment,
+                encoder=self.db_settings.encoder,
+                decoder=self.db_settings.decoder,
+            )
             logger.debug(
                 f"Saved experiment {experiment.name} in "
                 f"{_round_floats_for_logging(time.time() - start_time)} seconds."
@@ -254,11 +260,10 @@ class WithDBSettingsBase:
             return True
         return False
 
-    def _save_and_update_trials_and_generation_strategy_if_possible(
+    def _save_or_update_trials_and_generation_strategy_if_possible(
         self,
         experiment: Experiment,
-        new_trials: List[BaseTrial],
-        trials_to_update: List[BaseTrial],
+        trials: List[BaseTrial],
         generation_strategy: GenerationStrategy,
         new_generator_runs: List[GeneratorRun],
         suppress_all_errors: bool = False,
@@ -269,21 +274,16 @@ class WithDBSettingsBase:
 
         Args:
             experiment: Experiment, on which to save new trials in DB.
-            new_trials: Newly added trials to save.
-            trials_to_update: Updated trials to update in DB.
+            trials: Newly added or updated trials to save or update in DB.
             generation_strategy: Generation strategy to update in DB.
             new_generator_runs: Generator runs to add to generation strategy.
             suppress_all_errors: Flag for `retry_on_exception` that makes
                 the decorator suppress the thrown exception even if it
                 occurred in all the retries (exception is still logged).
         """
-        logger.debug(f"Saving {len(new_trials)} trials to DB.")
-        self._save_new_trials_to_db_if_possible(
-            experiment=experiment, trials=new_trials
-        )
-        logger.debug(f"Updating {len(trials_to_update)} in DB.")
-        self._save_updated_trials_to_db_if_possible(
-            experiment=experiment, trials=trials_to_update
+        logger.debug(f"Saving or updating {len(trials)} trials in DB.")
+        self._save_or_update_trials_in_db_if_possible(
+            experiment=experiment, trials=trials
         )
         logger.debug(
             "Updating generation strategy in DB with "
@@ -295,12 +295,8 @@ class WithDBSettingsBase:
         )
         return
 
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_types=RETRY_EXCEPTION_TYPES,
-    )
-    def _save_new_trial_to_db_if_possible(
+    # No retries needed, covered in `self._save_or_update_trials_in_db_if_possible`
+    def _save_or_update_trial_in_db_if_possible(
         self,
         experiment: Experiment,
         trial: BaseTrial,
@@ -319,8 +315,10 @@ class WithDBSettingsBase:
         Returns:
             bool: Whether the trial was saved.
         """
-        return self._save_new_trials_to_db_if_possible(
-            experiment, [trial], suppress_all_errors
+        return self._save_or_update_trials_in_db_if_possible(
+            experiment=experiment,
+            trials=[trial],
+            suppress_all_errors=suppress_all_errors,
         )
 
     @retry_on_exception(
@@ -328,14 +326,14 @@ class WithDBSettingsBase:
         default_return_on_suppression=False,
         exception_types=RETRY_EXCEPTION_TYPES,
     )
-    def _save_new_trials_to_db_if_possible(
+    def _save_or_update_trials_in_db_if_possible(
         self,
         experiment: Experiment,
         trials: List[BaseTrial],
         suppress_all_errors: bool = False,
     ) -> bool:
-        """Saves new trials on given experiment if DB settings are set on this
-        `WithDBSettingsBase` instance.
+        """Saves new trials or update existing trials on given experiment if DB
+        settings are set on this `WithDBSettingsBase` instance.
 
         Args:
             experiment: Experiment, on which to save new trials in DB.
@@ -349,76 +347,17 @@ class WithDBSettingsBase:
         """
         if self.db_settings_set:
             start_time = time.time()
-            _save_new_trials(
-                experiment=experiment, trials=trials, encoder=self.db_settings.encoder
+            _save_or_update_trials(
+                experiment=experiment,
+                trials=trials,
+                encoder=self.db_settings.encoder,
+                decoder=self.db_settings.decoder,
+                batch_size=STORAGE_MINI_BATCH_SIZE,
             )
             logger.debug(
-                f"Saved trials {[trial.index for trial in trials]} in "
-                f"{_round_floats_for_logging(time.time() - start_time)} seconds."
-            )
-            return True
-        return False
-
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_types=RETRY_EXCEPTION_TYPES,
-    )
-    def _save_updated_trial_to_db_if_possible(
-        self,
-        experiment: Experiment,
-        trial: BaseTrial,
-        suppress_all_errors: bool = False,
-    ) -> bool:
-        """Saves updated trials on given experiment if DB settings are set on this
-        `WithDBSettingsBase` instance.
-
-        Args:
-            experiment: Experiment, on which to save updated trials in DB.
-            trial: Newly updated trial to save.
-            suppress_all_errors: Flag for `retry_on_exception` that makes
-                the decorator suppress the thrown exception even if it
-                occurred in all the retries (exception is still logged).
-
-        Returns:
-            bool: Whether the trial was saved.
-        """
-        return self._save_updated_trials_to_db_if_possible(
-            experiment, [trial], suppress_all_errors
-        )
-
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_types=RETRY_EXCEPTION_TYPES,
-    )
-    def _save_updated_trials_to_db_if_possible(
-        self,
-        experiment: Experiment,
-        trials: List[BaseTrial],
-        suppress_all_errors: bool = False,
-    ) -> bool:
-        """Saves updated trials on given experiment if DB settings are set on this
-        `WithDBSettingsBase` instance.
-
-        Args:
-            experiment: Experiment, on which to save updated trials in DB.
-            trials: Newly updated trials to save.
-            suppress_all_errors: Flag for `retry_on_exception` that makes
-                the decorator suppress the thrown exception even if it
-                occurred in all the retries (exception is still logged).
-
-        Returns:
-            bool: Whether the trials were saved.
-        """
-        if self.db_settings_set:
-            start_time = time.time()
-            _update_trials(
-                experiment=experiment, trials=trials, encoder=self.db_settings.encoder
-            )
-            logger.debug(
-                f"Updated trials {[trial.index for trial in trials]} in "
-                f"{_round_floats_for_logging(time.time() - start_time)} seconds."
+                f"Saved or updated trials {[trial.index for trial in trials]} in "
+                f"{_round_floats_for_logging(time.time() - start_time)} seconds "
+                f"in mini-batches of {STORAGE_MINI_BATCH_SIZE}."
             )
             return True
         return False
@@ -448,6 +387,7 @@ class WithDBSettingsBase:
             _save_generation_strategy(
                 generation_strategy=generation_strategy,
                 encoder=self.db_settings.encoder,
+                decoder=self.db_settings.decoder,
             )
             logger.debug(
                 f"Saved generation strategy {generation_strategy.name} in "
@@ -487,10 +427,30 @@ class WithDBSettingsBase:
                 generation_strategy=generation_strategy,
                 generator_runs=new_generator_runs,
                 encoder=self.db_settings.encoder,
+                decoder=self.db_settings.decoder,
+                batch_size=STORAGE_MINI_BATCH_SIZE,
             )
             logger.debug(
                 f"Updated generation strategy {generation_strategy.name} in "
-                f"{_round_floats_for_logging(time.time() - start_time)} seconds."
+                f"{_round_floats_for_logging(time.time() - start_time)} seconds in "
+                f"mini-batches of {STORAGE_MINI_BATCH_SIZE} generator runs."
+            )
+            return True
+        return False
+
+    @retry_on_exception(
+        retries=3,
+        default_return_on_suppression=False,
+        exception_types=RETRY_EXCEPTION_TYPES,
+    )
+    def _update_experiment_properties_in_db(
+        self, experiment_with_updated_properties: Experiment
+    ) -> bool:
+        exp = experiment_with_updated_properties
+        if self.db_settings_set:
+            update_properties_on_experiment(
+                experiment_with_updated_properties=exp,
+                config=self.db_settings.encoder.config,
             )
             return True
         return False
