@@ -4,8 +4,8 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from ax.storage.json_store.load import load_experiment as load
-from ax.storage.json_store.save import save_experiment as save
+from ax.storage.json_store.load import load_experiment as json_load
+from ax.storage.json_store.save import save_experiment as json_save
 
 
-__all__ = ["load", "save"]
+__all__ = ["json_load", "json_save"]
