@@ -14,6 +14,7 @@ from ax.core.batch_trial import BatchTrial
 from ax.core.data import Data
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
+from ax.core.map_data import MapData
 from ax.core.metric import Metric
 from ax.core.multi_type_experiment import MultiTypeExperiment
 from ax.core.objective import MultiObjective, Objective, ScalarizedObjective
@@ -59,6 +60,7 @@ def experiment_to_dict(experiment: Experiment) -> Dict[str, Any]:
         "is_test": experiment.is_test,
         "data_by_trial": experiment.data_by_trial,
         "properties": experiment._properties,
+        "default_data_type": experiment._default_data_type,
     }
 
 
@@ -242,9 +244,8 @@ def multi_objective_to_dict(objective: MultiObjective) -> Dict[str, Any]:
     """Convert Ax objective to a dictionary."""
     return {
         "__type": objective.__class__.__name__,
-        "metrics": objective.metrics,
+        "objectives": objective.objectives,
         "weights": objective.weights,
-        "minimize": objective.minimize,
     }
 
 
@@ -335,6 +336,16 @@ def data_to_dict(data: Data) -> Dict[str, Any]:
     }
 
 
+def map_data_to_dict(map_data: MapData) -> Dict[str, Any]:
+    """Convert Ax data to a dictionary."""
+    return {
+        "__type": map_data.__class__.__name__,
+        "df": map_data.df,
+        "map_keys": map_data.map_keys,
+        "description": map_data.description,
+    }
+
+
 def transform_type_to_dict(transform_type: Type[Transform]) -> Dict[str, Any]:
     """Convert a transform class to a dictionary."""
     return {
@@ -435,7 +446,7 @@ def botorch_model_to_dict(model: BoTorchModel) -> Dict[str, Any]:
         "surrogate": model.surrogate,
         "surrogate_options": model.surrogate_options,
         "acquisition_class": model.acquisition_class,
-        "botorch_acqf_class": model.botorch_acqf_class,
+        "botorch_acqf_class": model._botorch_acqf_class,
         "acquisition_options": model.acquisition_options or {},
         "refit_on_update": model.refit_on_update,
         "refit_on_cv": model.refit_on_cv,
