@@ -7,10 +7,10 @@
 # necessary to import this file so SQLAlchemy knows about the event listeners
 # see https://fburl.com/8mn7yjt2
 from ax.storage.sqa_store import validation
-from ax.storage.sqa_store.load import load_experiment as load
-from ax.storage.sqa_store.save import save_experiment as save
+from ax.storage.sqa_store.load import load_experiment as sqa_load
+from ax.storage.sqa_store.save import save_experiment as sqa_save
 
 
-__all__ = ["load", "save"]
+__all__ = ["sqa_load", "sqa_save"]
 
 del validation
