@@ -8,6 +8,7 @@ from typing import Dict, Type
 
 from ax.modelbridge.transforms.base import Transform
 from ax.modelbridge.transforms.cap_parameter import CapParameter
+from ax.modelbridge.transforms.choice_encode import ChoiceEncode, OrderedChoiceEncode
 from ax.modelbridge.transforms.convert_metric_names import ConvertMetricNames
 from ax.modelbridge.transforms.derelativize import Derelativize
 from ax.modelbridge.transforms.int_range_to_choice import IntRangeToChoice
@@ -15,7 +16,6 @@ from ax.modelbridge.transforms.int_to_float import IntToFloat
 from ax.modelbridge.transforms.ivw import IVW
 from ax.modelbridge.transforms.log import Log
 from ax.modelbridge.transforms.one_hot import OneHot
-from ax.modelbridge.transforms.ordered_choice_encode import OrderedChoiceEncode
 from ax.modelbridge.transforms.power_transform_y import PowerTransformY
 from ax.modelbridge.transforms.remove_fixed import RemoveFixed
 from ax.modelbridge.transforms.search_space_to_choice import SearchSpaceToChoice
@@ -62,6 +62,7 @@ TRANSFORM_REGISTRY: Dict[Type[Transform], int] = {
     Winsorize: 16,
     CapParameter: 17,
     PowerTransformY: 18,
+    ChoiceEncode: 19,
 }
 
 
