@@ -5,7 +5,8 @@
 # LICENSE file in the root directory of this source tree.
 
 import enum
-from typing import Dict, List, NamedTuple
+
+from ax.core.experiment import DataType  # noqa F401
 
 
 class DomainType(enum.Enum):
@@ -25,6 +26,7 @@ class MetricIntent(enum.Enum):
     # Additional objective is not yet supported in Ax open-source.
     ADDITIONAL_OBJECTIVE: str = "additional_objective"
     OUTCOME_CONSTRAINT: str = "outcome_constraint"
+    SCALARIZED_OUTCOME_CONSTRAINT: str = "scalarized_outcome_constraint"
     OBJECTIVE_THRESHOLD: str = "objective_threshold"
     TRACKING: str = "tracking"
 
@@ -39,15 +41,3 @@ class ParameterConstraintType(enum.Enum):
     LINEAR: int = 0
     ORDER: int = 1
     SUM: int = 2
-
-
-class EncodeDecodeFieldsMap(NamedTuple):
-    python_only: List[str] = []
-    encoded_only: List[str] = []
-    python_to_encoded: Dict[str, str] = {}
-
-
-def remove_prefix(text, prefix):
-    if text.startswith(prefix):
-        return text[len(prefix) :]
-    return text
