@@ -6,6 +6,7 @@
 
 from __future__ import annotations
 
+import abc
 from typing import Optional
 
 from ax.utils.common.equality import equality_typechecker, object_attribute_dicts_equal
@@ -31,3 +32,18 @@ class Base:
         return object_attribute_dicts_equal(
             one_dict=self.__dict__, other_dict=other.__dict__
         )
+
+
+class SortableBase(Base, metaclass=abc.ABCMeta):
+    """Extension to the base class that also provides an inequality check."""
+
+    @property
+    @abc.abstractmethod
+    def _unique_id(self) -> str:
+        """Returns an identification string that can be used to uniquely
+        identify this instance from others attached to the same experiment.
+        """
+        pass
+
+    def __lt__(self, other: SortableBase) -> bool:
+        return self._unique_id < other._unique_id
