@@ -7,16 +7,19 @@
 
 from collections import OrderedDict
 from datetime import datetime
-from typing import Dict, Iterable, List, MutableMapping, Optional, Type, cast
+from typing import Any, Dict, Iterable, List, MutableMapping, Optional, Type, cast
 
 import numpy as np
 import pandas as pd
 import torch
 from ax.core.arm import Arm
+from ax.core.base_trial import BaseTrial
 from ax.core.batch_trial import AbandonedArm, BatchTrial
 from ax.core.data import Data
-from ax.core.experiment import Experiment
+from ax.core.experiment import DataType, Experiment
 from ax.core.generator_run import GeneratorRun
+from ax.core.map_data import MapData
+from ax.core.map_metric import MapMetric
 from ax.core.metric import Metric
 from ax.core.multi_type_experiment import MultiTypeExperiment
 from ax.core.objective import MultiObjective, Objective, ScalarizedObjective
@@ -24,7 +27,11 @@ from ax.core.optimization_config import (
     MultiObjectiveOptimizationConfig,
     OptimizationConfig,
 )
-from ax.core.outcome_constraint import ObjectiveThreshold, OutcomeConstraint
+from ax.core.outcome_constraint import (
+    ObjectiveThreshold,
+    OutcomeConstraint,
+    ScalarizedOutcomeConstraint,
+)
 from ax.core.parameter import (
     ChoiceParameter,
     FixedParameter,
@@ -49,11 +56,11 @@ from ax.core.types import (
     TParameterization,
 )
 from ax.metrics.branin import AugmentedBraninMetric, BraninMetric
+from ax.metrics.branin_map import BraninTimestampMapMetric
 from ax.metrics.factorial import FactorialMetric
 from ax.metrics.hartmann6 import AugmentedHartmann6Metric, Hartmann6Metric
 from ax.modelbridge.factory import Cont_X_trans, get_factorial, get_sobol
 from ax.models.torch.botorch_modular.acquisition import Acquisition
-from ax.models.torch.botorch_modular.kg import KnowledgeGradient
 from ax.models.torch.botorch_modular.list_surrogate import ListSurrogate
 from ax.models.torch.botorch_modular.model import BoTorchModel
 from ax.models.torch.botorch_modular.surrogate import Surrogate
@@ -77,18 +84,31 @@ logger = get_logger(__name__)
 ##############################
 
 
-def get_experiment() -> Experiment:
+def get_experiment(with_status_quo: bool = True) -> Experiment:
     return Experiment(
         name="test",
         search_space=get_search_space(),
         optimization_config=get_optimization_config(),
-        status_quo=get_status_quo(),
+        status_quo=get_status_quo() if with_status_quo else None,
         description="test description",
         tracking_metrics=[Metric(name="tracking")],
         is_test=True,
     )
 
 
+def get_experiment_with_map_data_type():
+    return Experiment(
+        name="test_map_data",
+        search_space=get_search_space(),
+        optimization_config=get_map_optimization_config(),
+        status_quo=get_status_quo(),
+        description="test description",
+        tracking_metrics=[MapMetric(name="tracking")],
+        is_test=True,
+        default_data_type=DataType.MAP_DATA,
+    )
+
+
 def get_branin_experiment(
     has_optimization_config: bool = True,
     with_batch: bool = False,
@@ -98,13 +118,14 @@ def get_branin_experiment(
     with_choice_parameter: bool = False,
     search_space: Optional[SearchSpace] = None,
     minimize: bool = False,
+    named: bool = True,
 ) -> Experiment:
     search_space = search_space or get_branin_search_space(
         with_fidelity_parameter=with_fidelity_parameter,
         with_choice_parameter=with_choice_parameter,
     )
     exp = Experiment(
-        name="branin_test_experiment",
+        name="branin_test_experiment" if named else None,
         search_space=search_space,
         optimization_config=get_branin_optimization_config(minimize=minimize)
         if has_optimization_config
@@ -131,6 +152,24 @@ def get_branin_experiment(
     return exp
 
 
+def get_branin_experiment_with_timestamp_map_metric(rate: Optional[float] = None):
+    return Experiment(
+        name="branin_with_timestamp_map_metric",
+        search_space=get_branin_search_space(),
+        optimization_config=OptimizationConfig(
+            objective=Objective(
+                metric=BraninTimestampMapMetric(
+                    name="branin", param_names=["x1", "x2"], rate=rate
+                ),
+                minimize=True,
+            )
+        ),
+        tracking_metrics=[BraninTimestampMapMetric(name="b", param_names=["x1", "x2"])],
+        runner=SyntheticRunner(),
+        default_data_type=DataType.MAP_DATA,
+    )
+
+
 def get_multi_type_experiment(
     add_trial_type: bool = True, add_trials: bool = False
 ) -> MultiTypeExperiment:
@@ -281,6 +320,14 @@ def get_experiment_with_data() -> Experiment:
     return batch_trial.experiment
 
 
+def get_experiment_with_map_data() -> Experiment:
+    experiment = get_experiment_with_map_data_type()
+    experiment.new_trial()
+    experiment.add_tracking_metric(MapMetric("ax_test_metric"))
+    experiment.attach_data(data=get_map_data())
+    return experiment
+
+
 def get_experiment_with_multi_objective() -> Experiment:
     optimization_config = get_multi_objective_optimization_config()
 
@@ -299,6 +346,7 @@ def get_experiment_with_multi_objective() -> Experiment:
 
 def get_branin_experiment_with_multi_objective(
     has_optimization_config: bool = True,
+    has_objective_thresholds: bool = False,
     with_batch: bool = False,
     with_status_quo: bool = False,
     with_fidelity_parameter: bool = False,
@@ -308,7 +356,9 @@ def get_branin_experiment_with_multi_objective(
         search_space=get_branin_search_space(
             with_fidelity_parameter=with_fidelity_parameter
         ),
-        optimization_config=get_branin_multi_objective_optimization_config()
+        optimization_config=get_branin_multi_objective_optimization_config(
+            has_objective_thresholds=has_objective_thresholds
+        )
         if has_optimization_config
         else None,
         runner=SyntheticRunner(),
@@ -328,18 +378,21 @@ def get_branin_experiment_with_multi_objective(
     return exp
 
 
-def get_experiment_with_scalarized_objective() -> Experiment:
+def get_experiment_with_scalarized_objective_and_outcome_constraint() -> Experiment:
     objective = get_scalarized_objective()
-    outcome_constraints = [get_outcome_constraint()]
+    outcome_constraints = [
+        get_outcome_constraint(),
+        get_scalarized_outcome_constraint(),
+    ]
     optimization_config = OptimizationConfig(
         objective=objective, outcome_constraints=outcome_constraints
     )
     return Experiment(
-        name="test_experiment_scalarized_objective",
+        name="test_experiment_scalarized_objective and outcome constraint",
         search_space=get_search_space(),
         optimization_config=optimization_config,
         status_quo=get_status_quo(),
-        description="test experiment with scalarized objective",
+        description="test experiment with scalarized objective and outcome constraint",
         tracking_metrics=[Metric(name="tracking")],
         is_test=True,
     )
@@ -401,41 +454,43 @@ def get_branin_search_space(
 
 def get_factorial_search_space() -> SearchSpace:
     return SearchSpace(
-        # Expected `List[ax.core.parameter.Parameter]` for 2nd parameter
-        # `parameters` to call `ax.core.search_space.SearchSpace.__init__` but
-        # got `List[ChoiceParameter]`.
         parameters=[
             ChoiceParameter(
                 name="factor1",
                 parameter_type=ParameterType.STRING,
-                # Expected `List[typing.Optional[typing.Union[bool, float, str]]]` for
-                # 4th parameter `values` to call
-                # `ax.core.parameter.ChoiceParameter.__init__` but got
-                # `List[str]`.
                 values=["level11", "level12", "level13"],
             ),
             ChoiceParameter(
                 name="factor2",
                 parameter_type=ParameterType.STRING,
-                # Expected `List[typing.Optional[typing.Union[bool, float, str]]]` for
-                # 4th parameter `values` to call
-                # `ax.core.parameter.ChoiceParameter.__init__` but got
-                # `List[str]`.
                 values=["level21", "level22"],
             ),
             ChoiceParameter(
                 name="factor3",
                 parameter_type=ParameterType.STRING,
-                # Expected `List[typing.Optional[typing.Union[bool, float, str]]]` for
-                # 4th parameter `values` to call
-                # `ax.core.parameter.ChoiceParameter.__init__` but got
-                # `List[str]`.
                 values=["level31", "level32", "level33", "level34"],
             ),
         ]
     )
 
 
+def get_large_factorial_search_space() -> SearchSpace:
+    return SearchSpace(
+        parameters=[
+            ChoiceParameter(
+                name="factor1",
+                parameter_type=ParameterType.STRING,
+                values=[f"level1{i}" for i in range(7)],
+            ),
+            ChoiceParameter(
+                name="factor2",
+                parameter_type=ParameterType.STRING,
+                values=[f"level2{i}" for i in range(10)],
+            ),
+        ]
+    )
+
+
 def get_hartmann_search_space(with_fidelity_parameter: bool = False) -> SearchSpace:
     parameters = [
         RangeParameter(
@@ -559,6 +614,40 @@ def get_trial() -> Trial:
     return trial
 
 
+class TestTrial(BaseTrial):
+    "Trial class to test unsupported trial type error"
+
+    _arms: List[Arm] = []
+
+    def __repr__(self) -> str:
+        return "test"
+
+    def _get_candidate_metadata(self, arm_name: str) -> Optional[Dict[str, Any]]:
+        return None
+
+    def _get_candidate_metadata_from_all_generator_runs(
+        self,
+    ) -> Dict[str, Optional[Dict[str, Any]]]:
+        return {"test": None}
+
+    def abandoned_arms(self) -> str:
+        return "test"
+
+    @property
+    def arms(self) -> List[Arm]:
+        return self._arms
+
+    @arms.setter
+    def arms(self, val: List[Arm]):
+        self._arms = val
+
+    def arms_by_name(self) -> str:
+        return "test"
+
+    def generator_runs(self) -> str:
+        return "test"
+
+
 ##############################
 # Parameters
 ##############################
@@ -590,6 +679,30 @@ def get_choice_parameter() -> ChoiceParameter:
     )
 
 
+def get_ordered_choice_parameter() -> ChoiceParameter:
+    return ChoiceParameter(
+        name="y",
+        parameter_type=ParameterType.INT,
+        # Expected `List[typing.Optional[typing.Union[bool, float, str]]]` for 4th
+        # parameter `values` to call
+        # `ax.core.parameter.ChoiceParameter.__init__` but got `List[str]`.
+        values=[1, 2, 3],
+        is_ordered=True,
+    )
+
+
+def get_task_choice_parameter() -> ChoiceParameter:
+    return ChoiceParameter(
+        name="y",
+        parameter_type=ParameterType.INT,
+        # Expected `List[typing.Optional[typing.Union[bool, float, str]]]` for 4th
+        # parameter `values` to call
+        # `ax.core.parameter.ChoiceParameter.__init__` but got `List[str]`.
+        values=[1, 2, 3],
+        is_task=True,
+    )
+
+
 def get_fixed_parameter() -> FixedParameter:
     return FixedParameter(name="z", parameter_type=ParameterType.BOOL, value=True)
 
@@ -684,6 +797,15 @@ def get_outcome_constraint() -> OutcomeConstraint:
     return OutcomeConstraint(metric=Metric(name="m2"), op=ComparisonOp.GEQ, bound=-0.25)
 
 
+def get_scalarized_outcome_constraint() -> ScalarizedOutcomeConstraint:
+    return ScalarizedOutcomeConstraint(
+        metrics=[Metric(name="oc_m3"), Metric(name="oc_m4")],
+        weights=[0.2, 0.8],
+        op=ComparisonOp.GEQ,
+        bound=-0.25,
+    )
+
+
 def get_branin_outcome_constraint() -> OutcomeConstraint:
     return OutcomeConstraint(metric=get_branin_metric(), op=ComparisonOp.LEQ, bound=0)
 
@@ -697,10 +819,16 @@ def get_objective() -> Objective:
     return Objective(metric=Metric(name="m1"), minimize=False)
 
 
+def get_map_objective() -> Objective:
+    return Objective(metric=MapMetric(name="m1"), minimize=False)
+
+
 def get_multi_objective() -> Objective:
     return MultiObjective(
-        metrics=[Metric(name="m1"), Metric(name="m3", lower_is_better=True)],
-        minimize=False,
+        objectives=[
+            Objective(metric=Metric(name="m1")),
+            Objective(metric=Metric(name="m3", lower_is_better=True), minimize=True),
+        ],
     )
 
 
@@ -718,11 +846,10 @@ def get_branin_objective(minimize: bool = False) -> Objective:
 
 def get_branin_multi_objective() -> Objective:
     return MultiObjective(
-        metrics=[
-            get_branin_metric(name="branin_a"),
-            get_branin_metric(name="branin_b"),
+        objectives=[
+            Objective(metric=get_branin_metric(name="branin_a")),
+            Objective(metric=get_branin_metric(name="branin_b")),
         ],
-        minimize=False,
     )
 
 
@@ -751,6 +878,11 @@ def get_optimization_config() -> OptimizationConfig:
     )
 
 
+def get_map_optimization_config() -> OptimizationConfig:
+    objective = get_map_objective()
+    return OptimizationConfig(objective=objective)
+
+
 def get_multi_objective_optimization_config() -> OptimizationConfig:
     objective = get_multi_objective()
     outcome_constraints = [get_outcome_constraint()]
@@ -770,8 +902,31 @@ def get_branin_optimization_config(minimize: bool = False) -> OptimizationConfig
     return OptimizationConfig(objective=get_branin_objective(minimize=minimize))
 
 
-def get_branin_multi_objective_optimization_config() -> OptimizationConfig:
-    return MultiObjectiveOptimizationConfig(objective=get_branin_multi_objective())
+def get_branin_multi_objective_optimization_config(
+    has_objective_thresholds: bool = False,
+) -> OptimizationConfig:
+    objective_thresholds = (
+        [
+            ObjectiveThreshold(
+                metric=get_branin_metric(name="branin_a"),
+                bound=10,
+                op=ComparisonOp.GEQ,
+                relative=False,
+            ),
+            ObjectiveThreshold(
+                metric=get_branin_metric(name="branin_b"),
+                bound=20,
+                op=ComparisonOp.GEQ,
+                relative=False,
+            ),
+        ]
+        if has_objective_thresholds
+        else None
+    )
+    return MultiObjectiveOptimizationConfig(
+        objective=get_branin_multi_objective(),
+        objective_thresholds=objective_thresholds,
+    )
 
 
 def get_augmented_branin_optimization_config() -> OptimizationConfig:
@@ -911,18 +1066,55 @@ def get_synthetic_runner() -> SyntheticRunner:
 ##############################
 
 
-def get_data(trial_index: int = 0) -> Data:
+def get_data(
+    metric_name: str = "ax_test_metric",
+    trial_index: int = 0,
+    num_non_sq_arms: int = 4,
+    include_sq: bool = True,
+) -> Data:
+    assert num_non_sq_arms < 5, "Only up to 4 arms currently handled."
+    arm_names = ["status_quo"] if include_sq else []
+    arm_names += [f"{trial_index}_{i}" for i in range(num_non_sq_arms)]
+    num_arms = num_non_sq_arms + 1 if include_sq else num_non_sq_arms
     df_dict = {
         "trial_index": trial_index,
-        "metric_name": "ax_test_metric",
-        "arm_name": ["status_quo"] + [f"{trial_index}_i" for i in range(4)],
-        "mean": [1, 3, 2, 2.25, 1.75],
-        "sem": [0, 0.5, 0.25, 0.40, 0.15],
-        "n": [100, 100, 100, 100, 100],
+        "metric_name": metric_name,
+        "arm_name": arm_names,
+        "mean": [1, 3, 2, 2.25, 1.75][:num_arms],
+        "sem": [0, 0.5, 0.25, 0.40, 0.15][:num_arms],
+        "n": [100, 100, 100, 100, 100][:num_arms],
     }
     return Data(df=pd.DataFrame.from_records(df_dict))
 
 
+def get_map_data(trial_index: int = 0) -> MapData:
+    evaluations = {
+        "status_quo": [
+            ({"epoch": 1}, {"ax_test_metric": (1.0, 0.5)}),
+            ({"epoch": 2}, {"ax_test_metric": (2.0, 0.5)}),
+            ({"epoch": 3}, {"ax_test_metric": (3.0, 0.5)}),
+            ({"epoch": 4}, {"ax_test_metric": (4.0, 0.5)}),
+        ],
+        "0_0": [
+            ({"epoch": 1}, {"ax_test_metric": (3.7, 0.5)}),
+            ({"epoch": 2}, {"ax_test_metric": (3.8, 0.5)}),
+            ({"epoch": 3}, {"ax_test_metric": (3.9, 0.5)}),
+            ({"epoch": 4}, {"ax_test_metric": (4.0, 0.5)}),
+        ],
+        "0_1": [
+            ({"epoch": 1}, {"ax_test_metric": (3.0, 0.5)}),
+            ({"epoch": 2}, {"ax_test_metric": (5.0, 0.5)}),
+            ({"epoch": 3}, {"ax_test_metric": (6.0, 0.5)}),
+            ({"epoch": 4}, {"ax_test_metric": (1.0, 0.5)}),
+        ],
+    }
+    return MapData.from_map_evaluations(
+        evaluations=evaluations,  # pyre-ignore [6]: Spurious param type mismatch.
+        trial_index=trial_index,
+        map_keys=["epoch"],
+    )
+
+
 def get_branin_data(
     trial_indices: Optional[Iterable[int]] = None,
     trials: Optional[Iterable[Trial]] = None,
@@ -1070,7 +1262,7 @@ def get_list_surrogate() -> Surrogate:
 
 
 def get_acquisition_type() -> Type[Acquisition]:
-    return KnowledgeGradient
+    return Acquisition
 
 
 def get_model_type() -> Type[Model]:
