@@ -140,14 +140,17 @@ def get_observation2trans() -> Observation:
 # Modeling layer
 
 
-def get_generation_strategy(with_experiment: bool = False) -> GenerationStrategy:
+def get_generation_strategy(
+    with_experiment: bool = False, with_callable_model_kwarg: bool = True
+) -> GenerationStrategy:
     gs = choose_generation_strategy(search_space=get_search_space())
     if with_experiment:
         gs._experiment = get_experiment()
     fake_func = get_experiment
-    # pyre-ignore[16]: testing hack to test serialization of callable kwargs
-    # in generation steps.
-    gs._steps[0].model_kwargs["model_constructor"] = fake_func
+    if with_callable_model_kwarg:
+        # pyre-ignore[16]: testing hack to test serialization of callable kwargs
+        # in generation steps.
+        gs._steps[0].model_kwargs["model_constructor"] = fake_func
     return gs
 
 
